@@ -3,7 +3,7 @@
 #  include <sys/types.h>
 #  include <unistd.h>
 #endif
-int main(void) { struct stat buf; 
-  if (stat (".", &buf) || !(buf.st_mode & S_IFDIR)) return 1;
+int main(void) { struct stat buf;
+  if (stat (".", &buf) || !S_ISDIR(buf.st_mode)) return 1;
   return 0;
 }
