@@ -175,21 +175,20 @@ GEN
 Fq_to_FF(GEN x, GEN ff)
 {
   ulong pp;
-  GEN r, T, p, z=_initFF(ff,&T,&p,&pp);
-  int is_int = typ(x)==t_INT;
-  switch(ff[1])
+  GEN r, T, p, z = _initFF(ff,&T,&p,&pp);
+  if (typ(x) == t_INT) switch(ff[1])
   {
-  case t_FF_FpXQ:
-    r= is_int ? scalarpol(x, varn(T)): ZX_copy(x);
-    break;
-  case t_FF_F2xq:
-    r= is_int ? Z_to_F2x(x,T[1]): ZX_to_F2x(x);
-    break;
-  default:
-    r= is_int ? Z_to_Flx(x,pp,T[1]): ZX_to_Flx(x,pp);
+    case t_FF_FpXQ: r = scalarpol(x, varn(T)); break;
+    case t_FF_F2xq: r = Z_to_F2x(x,T[1]); break;
+    default: r = Z_to_Flx(x,pp,T[1]);
   }
-  setvarn(r, varn(T)); /* paranoia */
-  return _mkFF_i(ff,z,r);
+  else switch(ff[1])
+  {
+    case t_FF_FpXQ: r = ZX_copy(x); setvarn(r, varn(T)); break;
+    case t_FF_F2xq: r = ZX_to_F2x(x); r[1] = T[1]; break;
+    default: r = ZX_to_Flx(x,pp); r[1] = T[1];
+  }
+  return _mkFF_i(ff, z, r);
 }
 
 /*************************************************************************/
@@ -1845,6 +1844,32 @@ FFX_halfgcd(GEN Pf, GEN Qf, GEN ff)
   return gerepilecopy(av, raw_to_FFXM(r, ff));
 }
 
+GEN
+FFX_halfgcd_all(GEN Pf, GEN Qf, GEN ff, GEN *a, GEN *b)
+{
+  pari_sp av = avma;
+  GEN r,T,p,R;
+  ulong pp;
+  GEN P = FFX_to_raw(Pf, ff);
+  GEN Q = FFX_to_raw(Qf, ff);
+  _getFF(ff,&T,&p,&pp);
+  switch(ff[1])
+  {
+  case t_FF_FpXQ:
+    r = FpXQX_halfgcd_all(P, Q, T, p, a, b);
+    break;
+  case t_FF_F2xq:
+    r = F2xqX_halfgcd_all(P, Q, T, a, b);
+    break;
+  default:
+    r = FlxqX_halfgcd_all(P, Q, T, pp, a, b);
+  }
+  if (*a) *a = raw_to_FFX(*a, ff);
+  if (*b) *b = raw_to_FFX(*b, ff);
+  R = raw_to_FFXM(r, ff);
+  return !a && b ? gc_all(av, 2, &R, b): gc_all(av, 1+!!a+!!b, &R, a, b);
+}
+
 GEN
 FFXQ_sqr(GEN Pf, GEN Qf, GEN ff)
 { return FFX_wrap2(Pf, Qf, ff, FpXQXQ_sqr, F2xqXQ_sqr, FlxqXQ_sqr); }
@@ -2453,3 +2478,24 @@ FFM_mul(GEN M, GEN N, GEN ff)
 {
   return FFM_FFM_wrap(M, N, ff, FqM_mul, FlxqM_mul, F2xqM_mul);
 }
+
+GEN
+FFV_roots_to_pol(GEN V, GEN ff, long v)
+{
+  pari_sp av = avma;
+  ulong pp;
+  GEN P, T, p;
+  long w = fetch_var_higher();
+  _getFF(ff,&T,&p,&pp); V = FFC_to_raw(V, ff);
+  switch(ff[1])
+  {
+  case t_FF_FpXQ: P = FqV_roots_to_pol(V, T, p, w); break;
+  case t_FF_F2xq: P = F2xqV_roots_to_pol(V, T, w); break;
+  default:        P = FlxqV_roots_to_pol(V, T, pp, w); break;
+  }
+  if (!P) return gc_NULL(av);
+  P = raw_to_FFX(P, ff);
+  setvarn(P, v);
+  delete_var();
+  return gerepilecopy(av, P);
+}
