@@ -18,16 +18,11 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 /* Not so fast arithmetic with points over elliptic curves over Fl */
 
 /***********************************************************************/
-/**                                                                   **/
 /**                              Flj                                  **/
-/**                                                                   **/
 /***********************************************************************/
-
-/* Arithmetic is implemented using Jacobian coordinates, representing
- * a projective point (x : y : z) on E by [z*x , z^2*y , z].  This is
- * probably not the fastest representation available for the given
- * problem, but they're easy to implement and up to 60% faster than
- * the school-book method. */
+/* Jacobian coordinates: we represent a projective point (x:y:z) on E by
+ * [z*x, z^2*y, z]. Not the fastest representation available for the problem,
+ * but easy to implement and up to 60% faster than the school-book method. */
 
 /* Cost: 1M + 8S + 1*a + 10add + 1*8 + 2*2 + 1*3.
  * http://www.hyperelliptic.org/EFD/g1p/auto-shortw-jacobian.html#doubling-dbl-2007-bl */
@@ -212,42 +207,35 @@ _Flj_mul(void *E, GEN P, GEN n)
   Q = Flj_mulu_pre(P, itou(n), ell->a4, ell->p, ell->pi);
   return s>0 ? Q : Flj_neg(Q, ell->p);
 }
+static GEN
+_Flj_one(void *E)
+{ (void) E; return mkvecsmall3(1, 1, 0); }
 
 GEN
 FljV_factorback_pre(GEN P, GEN L, ulong a4, ulong p, ulong pi)
 {
   struct _Flj E;
   E.a4 = a4; E.p = p; E.pi = pi;
-  return gen_factorback(P, L, (void*)&E, &_Flj_add, &_Flj_mul);
+  return gen_factorback(P, L, (void*)&E, &_Flj_add, &_Flj_mul, &_Flj_one);
 }
 
 ulong
-Flj_order_ufact(GEN P, ulong n, GEN F, ulong a4, ulong p, ulong pi)
+Flj_order_ufact(GEN P, ulong n, GEN fa, ulong a4, ulong p, ulong pi)
 {
   pari_sp av = avma;
+  GEN T = gel(fa,1), E = gel(fa,2);
+  long i, l = lg(T);
   ulong res = 1;
-  long i, nfactors;
-  GEN primes, exps;
 
-  primes = gel(F, 1);
-  nfactors = lg(primes);
-  exps = gel(F, 2);
-
-  for (i = 1; i < nfactors; ++i) {
-    ulong q, pp = primes[i];
-    long j, k, ei = exps[i];
-    naf_t x;
-    GEN b;
-
-    for (q = pp, j = 1; j < ei; ++j) q *= pp;
-    b = Flj_mulu_pre(P, n / q, a4, p, pi);
-
-    naf_repr(&x, pp);
-    for (j = 0; j < ei && b[3]; ++j)
-      b = Flj_mulu_pre_naf(b, pp, a4, p, pi, &x);
+  for (i = 1; i < l; i++, set_avma(av))
+  {
+    ulong j, t = T[i], e = E[i];
+    GEN b = P;
+    naf_t x; naf_repr(&x, t);
+    if (l != 2) b = Flj_mulu_pre(b, n / upowuu(t,e), a4, p, pi);
+    for (j = 0; j < e && b[3]; j++) b = Flj_mulu_pre_naf(b, t, a4, p, pi, &x);
     if (b[3]) return 0;
-    for (k = 0; k < j; ++k) res *= pp;
-    set_avma(av);
+    res *= upowuu(t, j);
   }
   return res;
 }
@@ -255,7 +243,20 @@ Flj_order_ufact(GEN P, ulong n, GEN F, ulong a4, ulong p, ulong pi)
 GEN
 Fle_to_Flj(GEN P)
 { return ell_is_inf(P) ? mkvecsmall3(1UL, 1UL, 0UL):
-                         mkvecsmall3(P[1], P[2], 1UL);
+                         mkvecsmall3(P[1], P[2], 1UL); }
+
+GEN
+Flj_to_Fle(GEN P, ulong p)
+{
+  if (P[3] == 0) return ellinf();
+  else
+  {
+    ulong Z = Fl_inv(P[3], p);
+    ulong Z2 = Fl_sqr(Z, p);
+    ulong X3 = Fl_mul(P[1], Z2, p);
+    ulong Y3 = Fl_mul(P[2], Fl_mul(Z, Z2, p), p);
+    return mkvecsmall2(X3, Y3);
+  }
 }
 
 GEN
@@ -317,9 +318,7 @@ Flj_changepointinv_pre(GEN P, GEN ch, ulong p, ulong pi)
 }
 
 /***********************************************************************/
-/**                                                                   **/
 /**                              Fle                                  **/
-/**                                                                   **/
 /***********************************************************************/
 GEN
 Fle_changepoint(GEN P, GEN ch, ulong p)
@@ -690,3 +689,174 @@ FleV_mulu_pre_inplace(GEN P, ulong n, GEN a4, ulong p, ulong pi)
   naf_t x; naf_repr(&x, n);
   FleV_mulu_pre_naf_inplace(P, n, a4, p, pi, &x);
 }
+
+/***********************************************************************/
+/**                            Pairings                               **/
+/**               Derived from APIP by Jerome Milan, 2012             **/
+/***********************************************************************/
+static ulong
+Fle_vert(GEN P, GEN Q, ulong a4, ulong p)
+{
+  if (ell_is_inf(P))
+    return 1;
+  if (uel(Q, 1) != uel(P, 1))
+    return Fl_sub(uel(Q, 1), uel(P, 1), p);
+  if (uel(P,2) != 0 ) return 1;
+  return Fl_inv(Fl_add(Fl_triple(Fl_sqr(uel(P,1),p), p), a4, p), p);
+}
+
+static ulong
+Fle_Miller_line(GEN R, GEN Q, ulong slope, ulong a4, ulong p)
+{
+  ulong x = uel(Q, 1), y = uel(Q, 2);
+  ulong tmp1 = Fl_sub(x, uel(R, 1), p);
+  ulong tmp2 = Fl_add(Fl_mul(tmp1, slope, p), uel(R,2), p);
+  if (y != tmp2)
+    return Fl_sub(y, tmp2, p);
+  if (y == 0)
+    return 1;
+  else
+  {
+    ulong s1, s2;
+    ulong y2i = Fl_inv(Fl_double(y, p), p);
+    s1 = Fl_mul(Fl_add(Fl_triple(Fl_sqr(x, p), p), a4, p), y2i, p);
+    if (s1 != slope)
+      return Fl_sub(s1, slope, p);
+    s2 = Fl_mul(Fl_sub(Fl_triple(x, p), Fl_sqr(s1, p), p), y2i, p);
+    return s2 ? s2: y2i;
+  }
+}
+
+/* Computes the equation of the line tangent to R and returns its
+ * evaluation at the point Q. Also doubles the point R. */
+static ulong
+Fle_tangent_update(GEN R, GEN Q, ulong a4, ulong p, GEN *pt_R)
+{
+  if (ell_is_inf(R)) { *pt_R = ellinf(); return 1; }
+  else if (uel(R,2) == 0) { *pt_R = ellinf(); return Fle_vert(R, Q, a4, p); }
+  else
+  {
+    ulong slope;
+    *pt_R = Fle_dbl_slope(R, a4, p, &slope);
+    return Fle_Miller_line(R, Q, slope, a4, p);
+  }
+}
+
+/* Computes the equation of the line through R and P, and returns its
+ * evaluation at the point Q. Also adds P to the point R. */
+static ulong
+Fle_chord_update(GEN R, GEN P, GEN Q, ulong a4, ulong p, GEN *pt_R)
+{
+  if (ell_is_inf(R)) { *pt_R = P; return Fle_vert(P, Q, a4, p); }
+  else if (ell_is_inf(P)) { *pt_R = R; return Fle_vert(R, Q, a4, p); }
+  else if (uel(P, 1) == uel(R, 1))
+  {
+    if (uel(P, 2) == uel(R, 2)) return Fle_tangent_update(R, Q, a4, p, pt_R);
+    else { *pt_R = ellinf(); return Fle_vert(R, Q, a4, p); }
+  }
+  else
+  {
+    ulong slope;
+    *pt_R = Fle_add_slope(P, R, a4, p, &slope);
+    return Fle_Miller_line(R, Q, slope, a4, p);
+  }
+}
+
+struct _Fle_miller { ulong p, a4; GEN P; };
+static GEN
+Fle_Miller_dbl(void* E, GEN d)
+{
+  struct _Fle_miller *m = (struct _Fle_miller *)E;
+  ulong p = m->p, a4 = m->a4;
+  GEN P = m->P;
+  ulong v, line;
+  ulong N = Fl_sqr(umael(d,1,1), p);
+  ulong D = Fl_sqr(umael(d,1,2), p);
+  GEN point = gel(d,2);
+  line = Fle_tangent_update(point, P, a4, p, &point);
+  N  = Fl_mul(N, line, p);
+  v = Fle_vert(point, P, a4, p);
+  D = Fl_mul(D, v, p); return mkvec2(mkvecsmall2(N, D), point);
+}
+static GEN
+Fle_Miller_add(void* E, GEN va, GEN vb)
+{
+  struct _Fle_miller *m = (struct _Fle_miller *)E;
+  ulong p = m->p, a4= m->a4;
+  GEN P = m->P, point;
+  ulong v, line;
+  ulong na = umael(va,1,1), da = umael(va,1,2);
+  ulong nb = umael(vb,1,1), db = umael(vb,1,2);
+  GEN pa = gel(va,2), pb = gel(vb,2);
+  ulong N = Fl_mul(na, nb, p);
+  ulong D = Fl_mul(da, db, p);
+  line = Fle_chord_update(pa, pb, P, a4, p, &point);
+  N = Fl_mul(N, line, p);
+  v = Fle_vert(point, P, a4, p);
+  D = Fl_mul(D, v, p); return mkvec2(mkvecsmall2(N, D), point);
+}
+
+/* Returns the Miller function f_{m, Q} evaluated at the point P using
+ * the standard Miller algorithm. */
+static ulong
+Fle_Miller(GEN Q, GEN P, ulong m, ulong a4, ulong p)
+{
+  pari_sp av = avma;
+  struct _Fle_miller d;
+  GEN v;
+  ulong N, D;
+
+  d.a4 = a4; d.p = p; d.P = P;
+  v = gen_powu_i(mkvec2(mkvecsmall2(1,1), Q), m, (void*)&d,
+                Fle_Miller_dbl, Fle_Miller_add);
+  N = umael(v,1,1); D = umael(v,1,2);
+  return gc_ulong(av, Fl_div(N, D, p));
+}
+
+ulong
+Fle_weilpairing(GEN P, GEN Q, ulong m, ulong a4, ulong p)
+{
+  pari_sp ltop = avma;
+  ulong N, D, w;
+  if (ell_is_inf(P) || ell_is_inf(Q) || zv_equal(P,Q)) return 1;
+  N = Fle_Miller(P, Q, m, a4, p);
+  D = Fle_Miller(Q, P, m, a4, p);
+  w = Fl_div(N, D, p);
+  if (odd(m)) w  = Fl_neg(w, p);
+  return gc_ulong(ltop, w);
+}
+
+ulong
+Fle_tatepairing(GEN P, GEN Q, ulong m, ulong a4, ulong p)
+{
+  if (ell_is_inf(P) || ell_is_inf(Q)) return 1;
+  return Fle_Miller(P, Q, m, a4, p);
+}
+
+GEN
+Fl_ellptors(ulong l, ulong N, ulong a4, ulong a6, ulong p)
+{
+  long v = z_lval(N, l);
+  ulong N0, N1;
+  GEN F;
+  if (v==0) return cgetg(1,t_VEC);
+  N0 = upowuu(l, v); N1 = N/N0;
+  F = mkmat2(mkcols(l), mkcols(v));
+  while(1)
+  {
+    pari_sp av2 = avma;
+    GEN P, Q;
+    ulong d, s, t;
+
+    P = Fle_mulu(random_Fle(a4, a6, p), N1, a4, p);
+    Q = Fle_mulu(random_Fle(a4, a6, p), N1, a4, p);
+    s = itou(Fle_order(P, F, a4, p));
+    t = itou(Fle_order(Q, F, a4, p));
+    if (s < t) { swap(P,Q); lswap(s,t); }
+    if (s == N0) retmkvec(Fle_mulu(P, s/l, a4, p));
+    d = Fl_order(Fle_weilpairing(P, Q, s, a4, p), s, p);
+    if (s*d == N0)
+      retmkvec2(Fle_mulu(P, s/l, a4, p), Fle_mulu(Q, t/l, a4, p));
+    set_avma(av2);
+  }
+}
