@@ -966,6 +966,7 @@ get_trace_Robert(GEN J, GEN phi, GEN Xm, GEN T, GEN q, ulong p, long e)
   return FpX_neg(FpXQ_mul(Dx, C, T, q), q);
 }
 
+/* in p^2, so p is tiny */
 static GEN
 Flxq_ellcard_Harley(GEN a4, GEN a6, GEN T, ulong p)
 {
@@ -975,7 +976,7 @@ Flxq_ellcard_Harley(GEN a4, GEN a6, GEN T, ulong p)
   GEN pp = utoipos(p), q = powuu(p, N);
   GEN T2, j, t, phi, J1, sqx, Xm, c2, tc2, c2p, Nc2, Np;
   long ispcyc = zx_is_pcyc(get_Flx_mod(T));
-  ulong pi = SMALL_ULONG(p)? 0: get_Fl_red(p);
+  ulong pi = SMALL_ULONG(p)? 0: get_Fl_red(p); /* = 0 here */
   timer_start(&ti);
   if (!ispcyc)
   {
@@ -1201,10 +1202,9 @@ Flxq_ellcard_Shanks(GEN a4, GEN a6, GEN q, GEN T, ulong p)
   GEN bound = addiu(sqrti(gmul2n(q,4)), 1); /* ceil( 4sqrt(q) ) */
   struct _FlxqE e;
   e.p = p; e.pi = pi; e.T = Flx_get_red_pre(T, p, pi);
-  /* once #E(Flxq) is know mod B >= bound, it is completely determined */
-  /* how many 2-torsion points ? */
+  /* once #E(Flxq) is known mod B >= bound, it is determined */
   switch(FlxqX_nbroots(mkpoln(4, pol1_Flx(v), pol0_Flx(v), a4, a6), T, p))
-  {
+  { /* how many 2-torsion points ? */
   case 3:  A = gen_0; B = utoipos(4); break;
   case 1:  A = gen_0; B = gen_2; break;
   default: A = gen_1; B = gen_2; break; /* 0 */
@@ -1435,38 +1435,199 @@ Flxq_ellcardj(GEN a4, GEN a6, ulong j, GEN T, GEN q, ulong p, long n)
   } else
   {
     ulong g = Fl_div(j, Fl_sub(1728%p, j, p), p);
-    GEN l = Flxq_div(Flx_triple(a6,p),Flx_double(a4,p),T,p);
     GEN N = Fp_ffellcard(utoi(Fl_triple(g,p)),utoi(Fl_double(g,p)),q,n,utoipos(p));
+    GEN l = Flxq_mul(Flx_triple(a6,p),Flx_double(a4,p),T,p);
     if (Flxq_issquare(l,T,p)) return N;
     return subii(shifti(q1,1),N);
   }
 }
 
+static GEN
+Flxq_ffellcard(GEN a4, GEN a6, GEN M, GEN q, GEN T, ulong p, long n)
+{
+  long m = degpol(M);
+  GEN j = polx_Flx(M[1]);
+  GEN g = Flxq_div(j, mkvecsmall3(M[1],1728%p,p-1), M, p);
+  GEN N = Flxq_ellcard(Flx_triple(g, p), Flx_double(g, p), M, p);
+  GEN qm =  powuu(p, m), q1 = addiu(q, 1), qm1 = addiu(qm, 1);
+  GEN l = Flxq_mul(Flx_triple(a6,p), Flx_double(a4,p), T, p);
+  GEN te = elltrace_extension(subii(qm1, N), n/m, qm);
+  return Flxq_issquare(l,T,p) ? subii(q1, te): addii(q1, te);
+}
+
+static GEN
+Flxq_ellcard_i(GEN a4, GEN a6, GEN T, ulong p)
+{
+  long n = get_Flx_degree(T);
+  GEN J, M, q = powuu(p,  n);
+  if (typ(a4)==t_VEC)
+    return F3xq_ellcard(gel(a4,1), a6, T);
+  if (p==3)
+    return F3xq_ellcardj(a4, a6, T, q, n);
+  if (degpol(a4)<=0 && degpol(a6)<=0)
+    return Fp_ffellcard(utoi(Flx_eval(a4,0,p)),utoi(Flx_eval(a6,0,p)),q,n,utoipos(p));
+  J = Flxq_ellj(a4,a6,T,p);
+  if (degpol(J)<=0)
+    return Flxq_ellcardj(a4,a6,lgpol(J)?J[2]:0,T,q,p,n);
+  M = Flxq_minpoly(J, T, p);
+  if (degpol(M) < n)
+    return Flxq_ffellcard(a4, a6, M, q, T, p, n);
+  if (p <= 7)
+    return Flxq_ellcard_Satoh(a4, a6, J, T, p);
+  if (cmpis(q,100)<0)
+    return utoi(Flxq_ellcard_naive(a4, a6, T, p));
+  if (p == 13 || (7*p <= (ulong)10*n && (BITS_IN_LONG==64 || p <= 103)))
+    return Flxq_ellcard_Satoh(a4, a6, J, T, p);
+  if (p <= (ulong)2*n)
+    return Flxq_ellcard_Kedlaya(a4, a6, T, p);
+  if (expi(q)<=62)
+    return Flxq_ellcard_Shanks(a4, a6, q, T, p);
+  else
+    return Fq_ellcard_SEA(Flx_to_ZX(a4),Flx_to_ZX(a6),q,Flx_to_ZX(T),utoipos(p),0);
+}
+
 GEN
 Flxq_ellcard(GEN a4, GEN a6, GEN T, ulong p)
 {
   pari_sp av = avma;
-  long n = get_Flx_degree(T);
-  GEN J, r, q = powuu(p,  n);
-  if (typ(a4)==t_VEC)
-    r = F3xq_ellcard(gel(a4,1), a6, T);
-  else if (p==3)
-    r = F3xq_ellcardj(a4, a6, T, q, n);
-  else if (degpol(a4)<=0 && degpol(a6)<=0)
-    r = Fp_ffellcard(utoi(Flx_eval(a4,0,p)),utoi(Flx_eval(a6,0,p)),q,n,utoipos(p));
-  else if (degpol(J=Flxq_ellj(a4,a6,T,p))<=0)
-    r = Flxq_ellcardj(a4,a6,lgpol(J)?J[2]:0,T,q,p,n);
-  else if (p <= 7)
-    r = Flxq_ellcard_Satoh(a4, a6, J, T, p);
-  else if (cmpis(q,100)<0)
-    r = utoi(Flxq_ellcard_naive(a4, a6, T, p));
-  else if (p == 13 || (7*p <= (ulong)10*n && (BITS_IN_LONG==64 || p <= 103)))
-    r = Flxq_ellcard_Satoh(a4, a6, J, T, p);
-  else if (p <= (ulong)2*n)
-    r = Flxq_ellcard_Kedlaya(a4, a6, T, p);
-  else if (expi(q)<=62)
-    r = Flxq_ellcard_Shanks(a4, a6, q, T, p);
-  else
-    r = Fq_ellcard_SEA(Flx_to_ZX(a4),Flx_to_ZX(a6),q,Flx_to_ZX(T),utoipos(p),0);
-  return gerepileuptoint(av, r);
+  return gerepileuptoint(av, Flxq_ellcard_i(a4, a6, T, p));
+}
+
+static long
+Fl_ellj_trace(ulong j, ulong p)
+{
+  ulong a4, a6;
+  Fl_ellj_to_a4a6(j, p, &a4, &a6);
+  return Fl_elltrace(a4, a6, p);
+}
+
+/* Given ordinary E/Fq, a prime ell, and the height of the ell-volcano
+ * containing j(E) (= v_ell(conductor of Z[pi_E]) returns the height of j(E)
+ * on its ell-volcano (= v_ell(conductor of the order End(E)). */
+static long
+Fl_ellheightabovefloor(ulong j, long ell, long e, ulong p)
+{
+  pari_sp av = avma;
+  GEN Xp, G, phi, phix, j0, j1;
+  long h, i, nj1;
+  if (e==0) return 0;
+  if (j==0 || j==1728%p) return e;
+  phi = ZXX_to_FlxX(polmodular_ZXX(ell, 0, 0, 1), p, 1);
+  phix = FlxY_evalx(phi, j, p);
+  Xp = Flx_Frobenius(phix, p);
+  G  = Flx_gcd(Flx_sub(Xp, polx_Flx(0), p), phix, p);
+  nj1 = degpol(G);
+  if (nj1 < ell) return 0;
+  if (e==1 || nj1 != ell+1) return e;
+  j1 = Flx_roots(G, p);
+  nj1 = lg(j1)-1;
+  if (nj1 < 3) return 0;
+  j0 = mkvecsmall3(j,j,j);
+  for (h = 1; ; h++)
+    for(i = 1; i <= 3; i++)
+    {
+      GEN P = Flx_div_by_X_x(FlxY_evalx(phi, uel(j1,i), p), uel(j0,i), p, NULL);
+      GEN r = Flx_roots(P, p);
+      if (lg(r) == 1) return gc_long(av, h);
+      j0[i] = j1[i];
+      j1[i] = r[1];
+    }
+}
+
+/* Given an ordinary elliptic curve E/Fp and an integer h, returns
+ * D = disc(End(E)) assuming h(D) = h, using the approach sketched in
+ * Remark 13. If the algorithm returns 0 it has proved that h(D) != h, but it
+ * is under no obligation to do so and is allowed to return any value when the
+ * assumption h(d) = h is false. */
+static long
+Fl_end13(ulong j, ulong h, ulong p)
+{
+  ulong D0, v, h0;
+  long i, lL, lc, lD, nc;
+  GEN D, DF, cs, L, vP, vE;
+  ulong t = Fl_ellj_trace(j, p);
+
+  D0 = coredisc2u_fact(factoru(4*p-t*t), -1, &vP, &vE);
+  h0 = itou(classno(stoi(-D0)));
+  if (h % h0 != 0) return 0;
+  h /= h0;
+  D = divisorsu_fact_factored(mkmat2(vP,vE));
+  DF = gel(D,2); D = gel(D,1);
+  lD = lg(D); v = D[lD-1];
+  cs = cgetg(lD,t_VECSMALL); nc = 0;
+  for (i = 1; i < lD; i++)
+  {
+    GEN F = gel(DF,i);
+    ulong w = uquadclassnoF_fact(D0, -1, gel(F,1), gel(F,2));
+    if (w == h) uel(cs,++nc) = v / uel(D,i);
+  }
+  if (nc==0) return 0;
+  if (nc==1) { v /= uel(cs,1); return -D0*v*v; }
+  L = cgetg(nc+1, t_VEC);
+  for (i = 1; i <= nc; i++) gel(L,i) = gel(factoru(uel(cs,i)), 1);
+  L = vecsmall_uniq(shallowconcat1(L));
+  lL = lg(L); lc = nc+1;
+  for (i = 1; i < lL; i++)
+  {
+    ulong ell = L[i];
+    long k, e = Fl_ellheightabovefloor(j, ell, z_lval(v,ell), p);
+    for (k = 1; k < lc; k++)
+      if(cs[k] && z_lval(cs[k], ell) != e) { cs[k] = 0; nc--; }
+    if (nc==0) return 0;
+    if (nc==1)
+    {
+      for (k = 1; k < lc; k++)
+        if (cs[k]) { v /= uel(cs,k); return -D0*v*v; }
+    }
+  }
+  return 0;
+}
+
+INLINE int
+RgX_is_monic_ZX(GEN pol)
+{ return RgX_is_ZX(pol) && ZX_is_monic(pol); }
+
+long
+polisclass(GEN H)
+{
+  pari_sp av = avma, btop;
+  long h = degpol(H), hl, i, pmin, vH = varn(H), vh;
+  double lmin;
+  ulong p;
+  GEN h2list;
+  forprime_t T;
+
+  if (typ(H)!= t_POL) pari_err_TYPE("polsisclass",H);
+  if (h <= 0 || !RgX_is_monic_ZX(H)) return 0;
+  vh = vals(h);
+  h2list = cgetg(vh+2, t_VECSMALL); hl = 1;
+  for (i = 0; i <= vh; i++)
+  {
+    ulong d = 1UL<<i;
+    if (((d-h)&1)==0) h2list[hl++] = d;
+  }
+  setlg(h2list, hl);
+  lmin = h * (log(log(h+2))+2);
+  pmin = 33 * ceil(lmin*lmin);
+  u_forprime_init(&T, pmin, ULONG_MAX);
+  btop = avma;
+  while((p = u_forprime_next(&T)))
+  {
+    ulong r;
+    long D, nroots;
+    GEN Xp, G, Hp = ZX_to_Flx(H,p);
+    if (!Flx_is_squarefree(Hp, p)) { set_avma(btop); continue; }
+    Xp = Flx_Frobenius(Hp, p);
+    G  = Flx_gcd(Flx_sub(Xp, polx_Flx(evalvarn(vH)), p), Hp, p);
+    nroots = degpol(G);
+    if (nroots==0) { set_avma(btop); continue; }
+    if (nroots < h && !zv_search(h2list,nroots)) return gc_long(av, 0);
+    r = Flx_oneroot(G, p);
+    if (Fp_elljissupersingular(utoi(r), utoi(p))) { set_avma(btop); continue; }
+    D = Fl_end13(r, h, p);
+    if (D && gequal(H, polclass(stoi(D), 0, vH))) return gc_long(av, D);
+    return gc_long(av, 0);
+  }
+  pari_err_BUG("polisclass");
+  return 0; /* LCOV_EXCL_LINE */
 }
