@@ -315,7 +315,7 @@ FpX_mulu(GEN y, ulong x,GEN p)
   if (!x) return zeropol(varn(y));
   z = cgetg_copy(y, &l); z[1] = y[1];
   for(i=2; i<l; i++) gel(z,i) = Fp_mulu(gel(y,i), x, p);
-  return z;
+  return ZX_renormalize(z, l);
 }
 
 GEN
@@ -706,11 +706,15 @@ FpX_halfres_basecase(GEN a, GEN b, GEN p, GEN *pa, GEN *pb, struct FpX_res *res)
     if (gc_needed(av,2))
     {
       if (DEBUGMEM>1) pari_warn(warnmem,"FpX_halfgcd (d = %ld)",degpol(b));
-      gerepileall(av,res ? 8: 6, &a,&b,&u1,&v1,&u,&v,&res->res,&res->lc);
+      if (res)
+        gerepileall(av, 8, &a,&b,&u1,&v1,&u,&v,&res->res,&res->lc);
+      else
+        gerepileall(av, 6, &a,&b,&u1,&v1,&u,&v);
     }
   }
   M = mkmat22(u,v,u1,v1); *pa = a; *pb = b;
-  return gc_all(av, res ? 5: 3, &M, pa, pb, &res->res, &res->lc);
+  return res ? gc_all(av, 5, &M, pa, pb, &res->res, &res->lc)
+             : gc_all(av, 3, &M, pa, pb);
 }
 
 static GEN FpX_halfres_i(GEN x, GEN y, GEN p, GEN *a, GEN *b, struct FpX_res *res);
@@ -744,7 +748,8 @@ FpX_halfres_split(GEN x, GEN y, GEN p, GEN *a, GEN *b, struct FpX_res *res)
   if (lgpol(y1) <= n)
   {
     *a = x1; *b = y1;
-    return gc_all(av, res ? 5: 3, &R, a, b, &res->res, &res->lc);
+    return res ? gc_all(av, 5, &R, a, b, &res->res, &res->lc)
+               : gc_all(av, 3, &R, a, b);
   }
   k = 2*n-degpol(y1);
   q = FpX_divrem(x1, y1, p, &r);
@@ -777,7 +782,8 @@ FpX_halfres_split(GEN x, GEN y, GEN p, GEN *a, GEN *b, struct FpX_res *res)
   V2 = FpXM_FpX_mul2(S, FpXn_red(y1,k), FpXn_red(r,k), p);
   *a = FpX_add(FpX_shift(*a,k), gel(V2,1), p);
   *b = FpX_add(FpX_shift(*b,k), gel(V2,2), p);
-  return gc_all(av, res ? 5: 3, &T, a, b, &res->res, &res->lc);
+  return res ? gc_all(av, 5, &T, a, b, &res->res, &res->lc)
+             : gc_all(av, 3, &T, a, b);
 }
 
 static GEN
@@ -2993,10 +2999,14 @@ gener_FpXQ(GEN T, GEN p, GEN *po)
 GEN
 gener_FpXQ_local(GEN T, GEN p, GEN L)
 {
-  GEN Lp, Lq, p_1 = subiu(p,1), q_1, N, Q;
-  long f, i, ip, iq, l = lg(L);
+  GEN Lp, Lq, p_1, q_1, N, Q;
+  long f, i, ip, iq, l;
+
   T = get_FpX_mod(T);
   f = degpol(T);
+  if (f == 1) return pgener_Fp_local(p, L);
+  l = lg(L);
+  p_1 = subiu(p,1);
   q_1 = subiu(powiu(p,f), 1);
   N = diviiexact(q_1, p_1);
 
