@@ -41,29 +41,66 @@ RgM_check_ZM(GEN A, const char *s)
   }
 }
 
+/* assume m > 1 */
 static long
 ZV_max_lg_i(GEN x, long m)
 {
-  long i, prec = 2;
-  for (i = 1; i < m; i++) prec = maxss(prec, lgefint(gel(x,i)));
-  return prec;
+  long i, l = lgefint(gel(x,1));
+  for (i = 2; i < m; i++) l = maxss(l, lgefint(gel(x,i)));
+  return l;
 }
 long
-ZV_max_lg(GEN x) { return ZV_max_lg_i(x, lg(x)); }
+ZV_max_lg(GEN x)
+{
+  long m = lg(x);
+  return m == 1? 2: ZV_max_lg_i(x, m);
+}
 
+/* assume n > 1 and m > 1 */
 static long
 ZM_max_lg_i(GEN x, long n, long m)
 {
-  long j, prec = 2;
-  for (j = 1; j < n; j++) prec = maxss(prec, ZV_max_lg_i(gel(x,j), m));
-  return prec;
+  long j, l = ZV_max_lg_i(gel(x,1), m);
+  for (j = 2; j < n; j++) l = maxss(l, ZV_max_lg_i(gel(x,j), m));
+  return l;
 }
 long
 ZM_max_lg(GEN x)
 {
-  long n = lg(x);
+  long n = lg(x), m;
   if (n == 1) return 2;
-  return ZM_max_lg_i(x, n, lgcols(x));
+  m = lgcols(x); return m == 1? 2: ZM_max_lg_i(x, n, m);
+}
+
+/* assume m > 1 */
+static long
+ZV_max_expi_i(GEN x, long m)
+{
+  long i, prec = expi(gel(x,1));
+  for (i = 2; i < m; i++) prec = maxss(prec, expi(gel(x,i)));
+  return prec;
+}
+long
+ZV_max_expi(GEN x)
+{
+  long m = lg(x);
+  return m == 1? 2: ZV_max_expi_i(x, m);
+}
+
+/* assume n > 1 and m > 1 */
+static long
+ZM_max_expi_i(GEN x, long n, long m)
+{
+  long j, prec = ZV_max_expi_i(gel(x,1), m);
+  for (j = 2; j < n; j++) prec = maxss(prec, ZV_max_expi_i(gel(x,j), m));
+  return prec;
+}
+long
+ZM_max_expi(GEN x)
+{
+  long n = lg(x), m;
+  if (n == 1) return 2;
+  m = lgcols(x); return m == 1? 2: ZM_max_expi_i(x, n, m);
 }
 
 GEN
@@ -424,15 +461,27 @@ static long
 sw_bound(long s)
 { return s > 60 ? 2: s > 25 ? 4: s > 15 ? 8 : s > 8 ? 16 : 32; }
 
+/* assume lx > 1 and ly > 1; x is (l-1) x (lx-1), y is (lx-1) x (ly-1).
+ * Return x * y */
 static GEN
 ZM_mul_i(GEN x, GEN y, long l, long lx, long ly)
 {
   long sx, sy, B;
-  if (lx==3 && l==3 && ly==3) return ZM2_mul(x,y);
+#ifdef LONG_IS_64BIT /* From Flm_mul_i */
+  long Flm_sw_bound = 70;
+#else
+  long Flm_sw_bound = 120;
+#endif
+  if (l == 1) return zeromat(0, ly-1);
+  if (lx==2 && l==2 && ly==2)
+  { retmkmat(mkcol(mulii(gcoeff(x,1,1), gcoeff(y,1,1)))); }
+  if (lx==3 && l==3 && ly==3) return ZM2_mul(x, y);
   sx = ZM_max_lg_i(x, lx, l);
   sy = ZM_max_lg_i(y, ly, lx);
-  if ((lx > 70 && ly > 70 && l > 70) && (sx <= 10 * sy && sy <= 10 * sx))
-    return ZM_mul_fast(x, y, lx, ly, sx, sy);
+  /* Use modular reconstruction if Flm_mul would use Strassen and the input
+   * sizes look balanced */
+  if (lx > Flm_sw_bound && ly > Flm_sw_bound && l > Flm_sw_bound
+      && sx <= 10 * sy && sy <= 10 * sx) return ZM_mul_fast(x,y, lx,ly, sx,sy);
 
   B = sw_bound(minss(sx, sy));
   if (l <= B || lx <= B || ly <= B)
@@ -450,6 +499,52 @@ ZM_mul(GEN x, GEN y)
   return ZM_mul_i(x, y, lgcols(x), lx, ly);
 }
 
+static GEN
+ZM_sqr_slice(GEN A, GEN P, GEN *mod)
+{
+  pari_sp av = avma;
+  long i, n = lg(P)-1;
+  GEN H, T;
+  if (n == 1)
+  {
+    ulong p = uel(P,1);
+    GEN a = ZM_to_Flm(A, p);
+    GEN Hp = gerepileupto(av, Flm_to_ZM(Flm_sqr(a, p)));
+    *mod = utoi(p); return Hp;
+  }
+  T = ZV_producttree(P);
+  A = ZM_nv_mod_tree(A, P, T);
+  H = cgetg(n+1, t_VEC);
+  for(i=1; i <= n; i++)
+    gel(H,i) = Flm_sqr(gel(A,i), P[i]);
+  H = nmV_chinese_center_tree_seq(H, P, T, ZV_chinesetree(P, T));
+  *mod = gmael(T, lg(T)-1, 1); return gc_all(av, 2, &H, mod);
+}
+
+GEN
+ZM_sqr_worker(GEN P, GEN A)
+{
+  GEN V = cgetg(3, t_VEC);
+  gel(V,1) = ZM_sqr_slice(A, P, &gel(V,2));
+  return V;
+}
+
+static GEN
+ZM_sqr_fast(GEN A, long l, long s)
+{
+  pari_sp av = avma;
+  forprime_t S;
+  GEN H, worker;
+  long h;
+  if (s == 2) return zeromat(l-1,l-1);
+  h = 1 + (2*s - 4) * BITS_IN_LONG + expu(l-1);
+  init_modular_big(&S);
+  worker = snm_closure(is_entry("_ZM_sqr_worker"), mkvec(A));
+  H = gen_crt("ZM_sqr", worker, &S, NULL, h, 0, NULL,
+              nmV_chinese_center, FpM_center);
+  return gerepileupto(av, H);
+}
+
 GEN
 QM_mul(GEN x, GEN y)
 {
@@ -596,11 +691,17 @@ ZM_transmul(GEN x, GEN y)
   return M;
 }
 
+/* assume l > 1; x is (l-1) x (l-1), return x^2.
+ * FIXME: we ultimately rely on Strassen-Winograd which uses 7M + 15A.
+ * Should use Bodrato's variant of Winograd, using 3M + 4S + 11A */
 static GEN
 ZM_sqr_i(GEN x, long l)
 {
-  long s = ZM_max_lg_i(x, l, l);
-  if (l > 70) return ZM_mul_fast(x, x, l, l, s, s);
+  long s;
+  if (l == 2) { retmkmat(mkcol(sqri(gcoeff(x,1,1)))); }
+  if (l == 3) return ZM2_sqr(x);
+  s = ZM_max_lg_i(x, l, l);
+  if (l > 70) return ZM_sqr_fast(x, l, s);
   if (l <= sw_bound(s))
     return ZM_mul_classical(x, x, l, l, l);
   else
@@ -732,6 +833,8 @@ _ZM_mul(void *data /*ignored*/, GEN x, GEN y)
 static GEN
 _ZM_sqr(void *data /*ignored*/, GEN x)
 { (void)data; return ZM_sqr(x); }
+/* FIXME: Using Bodrato's squaring, precomputations attached to fixed
+ * multiplicand should be reused. And some postcomputations can be fused */
 GEN
 ZM_pow(GEN x, GEN n)
 {
@@ -1375,18 +1478,38 @@ _mulii(void *E, GEN a, GEN b)
 GEN
 zv_prod_Z(GEN v)
 {
-  pari_sp av = avma;
+  pari_sp av;
   long k, m, n = lg(v)-1;
+  int stop = 0;
   GEN V;
   switch(n) {
     case 0: return gen_1;
     case 1: return utoi(v[1]);
     case 2: return muluu(v[1], v[2]);
   }
-  m = n >> 1;
+  av = avma; m = n >> 1;
   V = cgetg(m + (odd(n)? 2: 1), t_VEC);
+  for (k = n; k; k--) /* start from the end: v is usually sorted */
+    if (v[k] & HIGHMASK) { stop = 1; break; }
+  while (!stop)
+  { /* HACK: handle V as a t_VECSMALL; gain a few iterations */
+    for (k = 1; k <= m; k++)
+    {
+      V[k] = uel(v,k<<1) * uel(v,(k<<1)-1);
+      if (V[k] & HIGHMASK) stop = 1; /* last "free" iteration */
+    }
+    if (odd(n))
+    {
+      if (n == 1) { set_avma(av); return utoi(v[1]); }
+      V[++m] = v[n];
+    }
+    v = V; n = m; m = n >> 1;
+  }
+  /* n > 1; m > 0 */
+  if (n == 2) { set_avma(av); return muluu(v[1], v[2]); }
   for (k = 1; k <= m; k++) gel(V,k) = muluu(v[k<<1], v[(k<<1)-1]);
-  if (odd(n)) gel(V,k) = utoipos(v[n]);
+  if (odd(n)) gel(V, ++m) = utoipos(v[n]);
+  setlg(V, m+1); /* HACK: now V is a bona fide t_VEC */
   return gerepileuptoint(av, gen_product(V, NULL, &_mulii));
 }
 GEN
