@@ -35,9 +35,9 @@ rawchecknf(GEN v) { return typ(v)==t_VEC && lg(v)==10; }
 /* v a t_VEC, lg(v) = 11, sanity check for true bnf */
 static int
 v11checkbnf(GEN v) { return rawchecknf(bnf_get_nf(v)); }
-/* v a t_VEC, lg(v) = 12, sanity check for true bnf */
+/* v a t_VEC, lg(v) = 13, sanity check for true nf and true bnf */
 static int
-v13checkgchar(GEN v) { return rawcheckbnf(gchar_get_bnf(v)); }
+v13checkgchar(GEN v) { return rawchecknf(gchar_get_nf(v)) && rawcheckbnf(gchar_get_bnf(v)); }
 /* v a t_VEC, lg(v) = 10, sanity check for true nf */
 static int
 v10checknf(GEN v) { return typ(gel(v,1))==t_POL; }
@@ -185,6 +185,7 @@ typv6(GEN x)
     }
   }
   if (typ(gel(x,2)) == t_COL && typ(gel(x,3)) == t_INT) return typ_PRID;
+  if (typ(gel(x,1)) == t_INT && typ(gel(x,2)) == t_VEC) return typ_QUA;
   return typ_NULL;
 }
 
@@ -1051,7 +1052,7 @@ idealramgroups_aut(GEN nf, GEN gal, GEN pr, GEN aut)
   if (e%p) return idealramgroupstame(nf, gal, aut, pr);
   (void) u_lvalrem(e,p,&et);
   idx = idealramgroupswild(nf, gal, aut, pr);
-  sub = group_subgroups(galois_group(gal));
+  sub = galoissubgroups(gal);
   tbl = subgroups_tableset(sub, n);
   maxm = vecsmall_max(idx)+1;
   res = cgetg(maxm+1,t_VEC);
@@ -1216,7 +1217,7 @@ nfisisom(GEN a, GEN b)
     if (la != gen_1) t = RgX_Rg_div(t, la);
     gel(y,i) = sw? RgXQ_reverse(t, b): t;
   }
-  return gerepilecopy(av,y);
+  settyp(y, t_VEC); return gerepilecopy(av,y);
 }
 
 static GEN
@@ -1334,9 +1335,6 @@ nfisincl0(GEN fa, GEN fb, long flag)
 GEN
 nfisincl(GEN fa, GEN fb) { return nfisincl0(fa, fb, 0); }
 
-static GEN
-lastel(GEN x) { return gel(x, lg(x)-1); }
-
 static GEN
 RgF_to_Flxq(GEN F, GEN T, ulong p)
 {
@@ -1393,7 +1391,7 @@ nfsplitting_auto(GEN g, GEN R)
   for (i=1; i<=d; i++)
   {
     GEN r = perm_mul(vecsmall_indexsort(gel(N,i)), q);
-    gel(G,i) = FlxV_Flc_mul(Rp, vecpermute(K, r), p);
+    gel(G,i) = FlxV_Flc_mul(Rp, vecsmallpermute(K, r), p);
   }
   return mkvec3(g, G, utoi(p));
 }
@@ -1407,7 +1405,7 @@ nfsplitting_composite(GEN P)
   {
     GEN Fi = gel(F, i);
     if (degpol(Fi) == 1) continue;
-    Q = Q ? lastel(compositum(Q, Fi)): Fi;
+    Q = Q ? veclast(compositum(Q, Fi)): Fi;
   }
   return Q ? Q: pol_x(varn(P));
 }
@@ -1447,7 +1445,7 @@ nfsplitting0(GEN T0, GEN D, long flag)
   T = leafcopy(T); setvarn(T, fetch_var_higher());
   for(F = T;;)
   {
-    GEN P = gel(nffactor(K, F), 1), Q = gel(P,lg(P)-1);
+    GEN P = gel(nffactor(K, F), 1), Q = veclast(P);
     if (degpol(gel(P,1)) == degpol(Q))
     {
       if (!flag) break;
@@ -1506,7 +1504,7 @@ get_roots(GEN x, long r1, long prec)
     ru = (n+r1)>>1;
   }
   for (i=r1+1; i<=ru; i++) gel(z,i) = gel(z, (i<<1)-r1);
-  z[0]=evaltyp(t_VEC)|evallg(ru+1); return z;
+  z[0]=evaltyp(t_VEC)|_evallg(ru+1); return z;
 }
 
 GEN
@@ -1713,7 +1711,7 @@ prec_fix(long prec)
 {
 #ifndef LONG_IS_64BIT
   /* make sure that default accuracy is the same on 32/64bit */
-  if (odd(prec)) prec += EXTRAPRECWORD;
+  if (odd(prec2lg(prec))) prec+=EXTRAPRECWORD;
 #endif
   return prec;
 }
@@ -1872,7 +1870,7 @@ primes_certify(GEN dK, GEN dKP)
   }
   if (!is_pm1(D))
   {
-    if (signe(D) < 0) D = negi(D);
+    D = absi_shallow(D);
     vectrunc_append(isprime(D)? w: v, D);
   }
   return mkvec2(v,w);
@@ -1965,12 +1963,14 @@ ZX_cmp(GEN x, GEN y)
 static int
 ZX_is_better(GEN y, GEN x, GEN *dx)
 {
-  GEN d = ZX_disc(y);
+  pari_sp av;
   int cmp;
+  GEN d;
   if (!*dx) *dx = ZX_disc(x);
+  av = avma; d = ZX_disc(y);
   cmp = abscmpii(d, *dx);
   if (cmp < 0) { *dx = d; return 1; }
-  return cmp? 0: (ZX_cmp(y, x) < 0);
+  return gc_bool(av, cmp? 0: (ZX_cmp(y, x) < 0));
 }
 
 static void polredbest_aux(nfmaxord_t *S, GEN *pro, GEN *px, GEN *pdx, GEN *pa);
@@ -2214,22 +2214,37 @@ nfnewprec_shallow(GEN nf, long prec)
   gel(NF,6) = F.ro; return NF;
 }
 
+/* true rnf */
+GEN
+rnfnewprec_shallow(GEN rnf, long prec)
+{
+  GEN RNF = leafcopy(rnf);
+  gel(RNF,10) = nfnewprec_shallow(gel(RNF,10), prec);
+  if (obj_check(RNF, rnf_NFABS)) rnfcomplete(RNF);
+  return RNF;
+}
+GEN
+rnfnewprec(GEN rnf, long prec)
+{
+  pari_sp av = avma;
+  checkrnf(rnf);
+  return gerepilecopy(av, rnfnewprec_shallow(rnf, prec));
+}
+
 GEN
 nfnewprec(GEN nf, long prec)
 {
+  pari_sp av = avma;
   GEN z;
   switch(nftyp(nf))
   {
     default: pari_err_TYPE("nfnewprec", nf);
-    case typ_BNF: z = bnfnewprec(nf,prec); break;
-    case typ_BNR: z = bnrnewprec(nf,prec); break;
-    case typ_NF: {
-      pari_sp av = avma;
-      z = gerepilecopy(av, nfnewprec_shallow(checknf(nf), prec));
-      break;
-    }
+    case typ_RNF: z = rnfnewprec_shallow(nf,prec); break;
+    case typ_BNF: z = bnfnewprec_shallow(nf,prec); break;
+    case typ_NF:  z = nfnewprec_shallow(nf, prec); break;
+    case typ_BNR: return bnrnewprec(nf,prec);
   }
-  return z;
+  return gerepilecopy(av, z);
 }
 
 /********************************************************************/
@@ -2483,9 +2498,7 @@ filter(GEN y, GEN b, long n)
     GEN yi = gel(y,i), ai = gel(b,i);
     if (degpol(yi) == n)
     {
-      pari_sp av = avma;
-      if (!dx) dx = ZX_disc(yi);
-      else if (!ZX_is_better(yi,x,&dx)) { set_avma(av); continue; }
+      if (!dx) dx = ZX_disc(yi); else if (!ZX_is_better(yi,x,&dx)) continue;
       x = yi; a = ai; continue;
     }
     gel(y,k) = yi;
@@ -2581,7 +2594,7 @@ Polred(GEN x, long flag, GEN fa)
 }
 
 /* finds "best" polynomial in polred_aux list, defaulting to S->T if none of
- * them is primitive. *px is the ZX, characteristic polynomial of Mod(*pb,S->T),
+ * them is primitive. *px a ZX, characteristic polynomial of Mod(*pb,S->T),
  * *pdx its discriminant if pdx != NULL. Set *pro = polroots(S->T) */
 static void
 polredbest_aux(nfmaxord_t *S, GEN *pro, GEN *px, GEN *pdx, GEN *pb)
@@ -2598,8 +2611,7 @@ polredbest_aux(nfmaxord_t *S, GEN *pro, GEN *px, GEN *pdx, GEN *pb)
     for (i=1; i<l; i++)
     {
       GEN yi = gel(y,i);
-      pari_sp av = avma;
-      if (ZX_is_better(yi,x,&dx)) { x = yi; b = gel(a,i); } else set_avma(av);
+      if (ZX_is_better(yi,x,&dx)) { x = yi; b = gel(a,i); }
     }
     *pb = b;
   }
@@ -2609,8 +2621,7 @@ polredbest_aux(nfmaxord_t *S, GEN *pro, GEN *px, GEN *pdx, GEN *pb)
     for (i=1; i<l; i++)
     {
       GEN yi = gel(y,i);
-      pari_sp av = avma;
-      if (ZX_is_better(yi,x,&dx)) x = yi; else set_avma(av);
+      if (ZX_is_better(yi,x,&dx)) x = yi;
     }
   }
   if (pdx) { if (!dx) dx = ZX_disc(x); *pdx = dx; }
@@ -2956,6 +2967,7 @@ rnfpolred_i(GEN nf, GEN R, long flag, long best)
   const long abs = ((flag & nf_ORIG) && (flag & nf_ABSOLUTE));
   GEN listP = NULL, red, pol, A, P, T, rnfeq;
   pari_sp av = avma;
+  long even = 0;
 
   if (typ(R) == t_VEC) {
     if (lg(R) != 3) pari_err_TYPE(f,R);
@@ -2993,13 +3005,15 @@ rnfpolred_i(GEN nf, GEN R, long flag, long best)
     a = gel(v,2);
     l = lg(y); A = cgetg(l, t_VEC);
     for (i = j = 1; i < l; i++)
-      if (ZX_equal(gel(y,i),P))
-      {
-        GEN t = gel(a,i);
-        if (u) t = RgV_RgC_mul(S.basis, ZM_ZC_mul(u,t));
-        gel(A,j++) = t;
-      }
+      if (ZX_equal(gel(y,i),P)) gel(A,j++) = gel(a,i);
     setlg(A,j); /* mod(A[i], pol) are all roots of P in Q[X]/(pol) */
+    even = j > 2 && !odd(degpol(P)) && !odd(ZX_deflate_order(P));
+    for (i = 1; i < j; i++)
+    {
+      GEN t = gel(A,i);
+      if (u) t = ZM_ZC_mul(u,t);
+      gel(A,i) = RgV_RgC_mul(S.basis, t);
+    }
   }
   if (DEBUGLEVEL>1) err_printf("reduced absolute generator: %Ps\n",P);
   if (flag & nf_ABSOLUTE)
@@ -3027,6 +3041,11 @@ rnfpolred_i(GEN nf, GEN R, long flag, long best)
     {
       GEN a = eltabstorel_lift(rnfeq, gel(A,i));
       GEN p = lift_if_rational( RgXQ_charpoly(a, R, v) );
+      if (even)
+      {
+        GEN q = RgX_unscale(p, gen_m1);
+        if (cmp_universal(q, p) < 0) { p = q; a = gneg(a); }
+      }
       if (i == 1 || cmp_universal(p, P) < 0) { P = p; besta = a; }
     }
     A = besta;
