@@ -464,8 +464,7 @@ polcyclo_eval(long n, GEN x)
 {
   pari_sp av= avma;
   GEN P, md, xd, yneg, ypos;
-  long l, s, i, j, q, tx;
-  long root_of_1 = 0;
+  long vpx, l, s, i, j, q, tx, root_of_1 = 0;
 
   if (!x) return polcyclo(n, 0);
   tx = typ(x);
@@ -514,13 +513,25 @@ polcyclo_eval(long n, GEN x)
    * At the end we return ypos/yneg if mu(n)=1 and yneg/ypos if mu(n)=-1 */
   ypos = gsubgs(x,1);
   yneg = gen_1;
+  vpx = (typ(x) == t_PADIC)? valp(x): 0;
   for (i = 1; i <= l; i++)
   {
     long ti = 1L<<(i-1), p = P[i];
     for (j = 1; j <= ti; j++) {
-      GEN X = gpowgs(gel(xd,j), p), t = gsubgs(X,1);
-      gel(xd,ti+j) = X;
+      GEN X = gel(xd,j), t;
+      if (vpx > 0)
+      { /* ypos, X t_PADIC */
+        ulong a = umuluu_or_0(p, valp(X)), b = precp(ypos) - 1;
+        long e = (a && a < b) ? b - a : 0;
+        if (precp(X) > e) X = cvtop(X, gel(ypos,2), e);
+        if (e > 0) X = gpowgs(X, p); /* avoid valp overflow of p-adic 0*/
+      }
+      else
+        X = gpowgs(X, p);
       md[ti+j] = -md[j];
+      gel(xd,ti+j) = X;
+      /* avoid precp overflow */
+      t = (vpx > 0 && gequal0(X))? gen_m1: gsubgs(X,1);
       if (gequal0(t))
       { /* x^d = 1; root_of_1 := the smallest index ti+j such that X == 1
         * (whose bits code d: bit i-1 is set iff P[i] | d). If no such index
@@ -662,7 +673,7 @@ gprec(GEN x, long d)
   return gerepilecopy(av, gprec_w(x, ndec2prec(d)));
 }
 
-/* not GC-safe; precision given in word length (including codewords) */
+/* not GC-safe */
 GEN
 gprec_w(GEN x, long pr)
 {
@@ -674,8 +685,7 @@ gprec_w(GEN x, long pr)
     case t_REAL:
       if (signe(x)) return realprec(x) != pr? rtor(x,pr): x;
       i = -prec2nbits(pr);
-      if (i < expo(x)) return real_0_bit(i);
-      y = cgetr(2); y[1] = x[1]; return y;
+      return real_0_bit(minss(i,expo(x)));
     case t_COMPLEX:
       y = cgetg(3, t_COMPLEX);
       gel(y,1) = gprec_w(gel(x,1),pr);
@@ -691,7 +701,7 @@ gprec_w(GEN x, long pr)
   }
   return y;
 }
-/* not GC-safe; precision given in word length (including codewords) */
+/* not GC-safe */
 GEN
 gprec_wensure(GEN x, long pr)
 {
@@ -703,8 +713,7 @@ gprec_wensure(GEN x, long pr)
     case t_REAL:
       if (signe(x)) return realprec(x) < pr? rtor(x,pr): x;
       i = -prec2nbits(pr);
-      if (i < expo(x)) return real_0_bit(i);
-      y = cgetr(2); y[1] = x[1]; return y;
+      return real_0_bit(minss(i,expo(x)));
     case t_COMPLEX:
       y = cgetg(3, t_COMPLEX);
       gel(y,1) = gprec_wensure(gel(x,1),pr);
@@ -721,8 +730,7 @@ gprec_wensure(GEN x, long pr)
   return y;
 }
 
-/* not GC-safe; precision given in word length (including codewords),
- * truncate mantissa to precision 'pr' but never increase it */
+/* not GC-safe; truncate mantissa to precision 'pr' but never increase it */
 GEN
 gprec_wtrunc(GEN x, long pr)
 {
@@ -760,7 +768,7 @@ gprec_wtrunc(GEN x, long pr)
 static GEN
 serlaplace(GEN x)
 {
-  long i, l = lg(x), e = valp(x);
+  long i, l = lg(x), e = valser(x);
   GEN t, y = cgetg(l,t_SER);
   if (e < 0) pari_err_DOMAIN("laplace","valuation","<",gen_0,stoi(e));
   t = mpfact(e); y[1] = x[1];
@@ -810,12 +818,12 @@ convol(GEN x, GEN y)
   if (typ(x) != t_SER) pari_err_TYPE("convol",x);
   if (typ(y) != t_SER) pari_err_TYPE("convol",y);
   if (varn(y) != vx) pari_err_VAR("convol", x,y);
-  ex = valp(x);
-  ey = valp(y);
+  ex = valser(x);
+  ey = valser(y);
   if (ser_isexactzero(x))
   {
     z = scalarser(gadd(Rg_get_0(x), Rg_get_0(y)), varn(x), 1);
-    setvalp(z, maxss(ex,ey)); return z;
+    setvalser(z, maxss(ex,ey)); return z;
   }
   lx = lg(x) + ex; x -= ex;
   ly = lg(y) + ey; y -= ey;
@@ -825,7 +833,7 @@ convol(GEN x, GEN y)
   if (lx - ex < 3) return zeroser(vx, lx-2);
 
   z = cgetg(lx - ex, t_SER);
-  z[1] = evalvalp(ex) | evalvarn(vx);
+  z[1] = evalvalser(ex) | evalvarn(vx);
   for (j = ex+2; j<lx; j++) gel(z,j-ex) = gmul(gel(x,j),gel(y,j));
   return normalizeser(z);
 }
@@ -1012,23 +1020,23 @@ binomialuu(ulong n, ulong k)
 GEN
 binomial(GEN n, long k)
 {
-  long i, prec;
+  long i, prec, tn = typ(n);
   pari_sp av;
   GEN y;
 
-  if (k <= 1)
-  {
-    if (is_noncalc_t(typ(n))) pari_err_TYPE("binomial",n);
-    if (k < 0) return gen_0;
-    if (k == 0) return gen_1;
-    return gcopy(n);
-  }
   av = avma;
-  if (typ(n) == t_INT)
+  if (tn == t_INT)
   {
-    if (signe(n) > 0)
-    {
-      GEN z = subiu(n, k);
+    long sn;
+    GEN z;
+    if (k == 0) return gen_1;
+    sn = signe(n);
+    if (sn == 0) return gen_0; /* k != 0 */
+    if (sn > 0)
+    { /* n > 0 */
+      if (k < 0) return gen_0;
+      if (k == 1) return icopy(n);
+      z = subiu(n, k);
       if (cmpiu(z, k) < 0)
       {
         switch(signe(z))
@@ -1042,13 +1050,35 @@ binomial(GEN n, long k)
       set_avma(av);
       if (lgefint(n) == 3) return binomialuu(n[2],(ulong)k);
     }
+    else
+    { /* n < 0, k != 0; use Kronenburg's definition */
+      if (k > 0)
+        z = binomial(subsi(k - 1, n), k);
+      else
+      {
+        z = subis(n, k); if (signe(z) < 0) return gen_0;
+        n = stoi(-k-1); k = itos(z);
+        z = binomial(n, k);
+      }
+      if (odd(k)) togglesign_safe(&z);
+      return gerepileuptoint(av, z);
+    }
+    /* n >= 0 and huge, k != 0 */
+    if (k < 0) return gen_0;
+    if (k == 1) return icopy(n);
     /* k > 1 */
     y = cgetg(k+1,t_VEC); gel(y,1) = n;
     for (i = 2; i <= k; i++) gel(y,i) = subiu(n,i-1);
     y = diviiexact(ZV_prod(y), mpfact(k));
     return gerepileuptoint(av, y);
   }
-
+  if (is_noncalc_t(tn)) pari_err_TYPE("binomial",n);
+  if (k <= 1)
+  {
+    if (k < 0) return Rg_get_0(n);
+    if (k == 0) return Rg_get_1(n);
+    return gcopy(n);
+  }
   prec = precision(n);
   if (prec && k > 200 + 0.8*prec2nbits(prec)) {
     GEN A = mpfactr(k, prec), B = ggamma(gsubgs(n,k-1), prec);
@@ -1839,12 +1869,23 @@ long
 vecsearch(GEN v, GEN x, GEN k)
 {
   pari_sp av = avma;
+  long r;
   void *E;
   int (*CMP)(void*,GEN,GEN) = sort_function(&E, v, k);
-  long r, tv = typ(v);
-  if (tv == t_VECSMALL)
-    x = (GEN)itos(x);
-  else if (!is_matvec_t(tv)) pari_err_TYPE("vecsearch", v);
+  switch(typ(v))
+  {
+    case t_VECSMALL: x = (GEN)itos(x); break;
+    case t_VEC: case t_COL: break;
+    case t_LIST:
+      if (list_typ(v)==t_LIST_RAW)
+      {
+        v = list_data(v); if (!v) v = cgetg(1, t_VEC);
+        break;
+      }
+      /* fall through */
+    default:
+      pari_err_TYPE("vecsearch", v);
+  }
   r = CMP? gen_search(v, x, E, CMP): key_search(v, x, k);
   return gc_long(av, r < 0? 0: r);
 }
@@ -2072,6 +2113,13 @@ merge_factor(GEN fx, GEN fy, void *data, int (*cmp)(void *,GEN,GEN))
   setlg(M, m);
   setlg(E, m); return mkmat2(M, E);
 }
+
+GEN
+ZM_merge_factor(GEN A, GEN B)
+{
+  return merge_factor(A, B, (void*)&cmpii, cmp_nodata);
+}
+
 /* merge two sorted vectors, removing duplicates. Shallow */
 GEN
 merge_sort_uniq(GEN x, GEN y, void *data, int (*cmp)(void *,GEN,GEN))
