@@ -80,12 +80,14 @@ direuler_Sbad(GEN V, GEN v, GEN Sbad, ulong *n)
   GEN pbad = gen_1;
   for (i = 1; i < l; i++)
   {
-    GEN ai = gel(Sbad,i);
+    GEN ai = gel(Sbad,i), gq;
     ulong q;
     if (typ(ai) != t_VEC || lg(ai) != 3)
       pari_err_TYPE("direuler [bad primes]",ai);
-    q = gtou(gel(ai,1));
-    if (q <= X)
+    gq = gel(ai,1);
+    if (typ(gq) != t_INT || signe(gq) <= 0)
+      pari_err_TYPE("direuler [prime expected]",gq);
+    if (lgefint(gq) == 3 && (q = uel(gq,2)) <= X)
     {
       long d = ulogint(X, q) + 1;
       GEN s = direuler_factor(gel(ai,2), d);
