@@ -384,7 +384,7 @@ ellL1_der(GEN e, GEN vec, struct lcritical *C, GEN t, long der, long prec)
 }
 
 GEN
-ellL1_bitprec(GEN E, long r, long bitprec)
+ellL1(GEN E, long r, long bitprec)
 {
   pari_sp av = avma;
   struct lcritical C;
@@ -396,15 +396,12 @@ ellL1_bitprec(GEN E, long r, long bitprec)
   if (r == 0 && ellrootno_global(e) < 0) { set_avma(av); return gen_0; }
   vec = Lpoints(&C, e, 0., bitprec);
   t = r ? scalarser(gen_1, 0, r):  zeroser(0, 0);
-  setvalp(t, 1);
+  setvalser(t, 1);
   return gerepileupto(av, ellL1_der(e, vec, &C, t, r, prec));
 }
 
 GEN
-ellL1(GEN E, long r, long prec) { return ellL1_bitprec(E, r, prec2nbits(prec)); }
-
-GEN
-ellanalyticrank_bitprec(GEN E, GEN eps, long bitprec)
+ellanalyticrank(GEN E, GEN eps, long bitprec)
 {
   pari_sp av = avma, av2;
   long prec = nbits2prec(bitprec);
@@ -428,7 +425,7 @@ ellanalyticrank_bitprec(GEN E, GEN eps, long bitprec)
   {
     GEN Lrk;
     GEN t = rk ? scalarser(gen_1, 0, rk):  zeroser(0, 0);
-    setvalp(t, 1);
+    setvalser(t, 1);
     Lrk = ellL1_der(e, vec, &C, t, rk, prec);
     if (DEBUGLEVEL) timer_printf(&ti, "L^(%ld)=%Ps", rk, Lrk);
     if (abscmprr(Lrk, eps) > 0)
@@ -437,12 +434,6 @@ ellanalyticrank_bitprec(GEN E, GEN eps, long bitprec)
   }
 }
 
-GEN
-ellanalyticrank(GEN E, GEN eps, long prec)
-{
-  return ellanalyticrank_bitprec(E, eps, prec2nbits(prec));
-}
-
 /*        Heegner point computation
 
    This section is a C version by Bill Allombert of a GP script by
@@ -533,7 +524,7 @@ heegner_psi(GEN E, GEN N, GEN points, long bitprec)
   pari_sp av = avma, av2;
   struct baby_giant bb;
   struct bg_data bg;
-  long l, k, L = lg(points)-1, prec = nbits2prec(bitprec)+EXTRAPRECWORD;
+  long l, k, L = lg(points)-1, prec = nbits2prec(bitprec)+EXTRAPREC64;
   GEN Q, pi2 = Pi2n(1, prec), bnd, rbnd, bndmax;
   GEN B = divrr(mulur(bitprec,mplog2(DEFAULTPREC)), pi2);
 
@@ -659,7 +650,7 @@ logplusQ(GEN t, long prec)
   if (typ(t) == t_INT)
   {
     if (!signe(t)) return real_1(prec);
-    if (signe(t) < 0) t = negi(t);
+    t = absi_shallow(t);
   }
   else
   {
@@ -911,7 +902,7 @@ static GEN
 ltwist1(GEN E, GEN d, long bitprec)
 {
   pari_sp av = avma;
-  GEN Ed = elltwist(E, d), z = ellL1_bitprec(Ed, 0, bitprec);
+  GEN Ed = elltwist(E, d), z = ellL1(Ed, 0, bitprec);
   obj_free(Ed); return gerepileuptoleaf(av, z);
 }
 
@@ -1343,7 +1334,7 @@ ellheegner(GEN E)
   GEN z, P, ht, points, coefs, s, om, indmult;
   GEN sel, etal, et, cbb, A, dAi, T, Ag, At;
   long ind, indx, lint, k, l, wtor, etor, ndisc, ltors2, selrank;
-  long bitprec = 16, prec = nbits2prec(bitprec)+1;
+  long bitprec = 16, prec = nbits2prec(bitprec) + EXTRAPRECWORD;
   pari_timer ti;
   GEN N, cb, tam, torsion, nfA;
   E = ellanal_globalred_all(E, &cb, &N, &tam);
@@ -1363,7 +1354,7 @@ ellheegner(GEN E)
     GEN hnaive, l1;
     long bitneeded;
     if (DEBUGLEVEL) timer_start(&ti);
-    l1 = ellL1_bitprec(E, 1, bitprec);
+    l1 = ellL1(E, 1, bitprec);
     if (DEBUGLEVEL) timer_printf(&ti,"ellL1");
     if (expo(l1) < 1 - bitprec/2)
       pari_err_DOMAIN("ellheegner", "analytic rank",">",gen_1,E);
@@ -1373,11 +1364,11 @@ ellheegner(GEN E)
     hnaive = hnaive_max(E, ht);
     if (DEBUGLEVEL) err_printf("Naive height <= %Ps\n", hnaive);
     hnaive = gadd(shiftr(hnaive,-1),glog(dAi,prec));
-    bitneeded = itos(gceil(divrr(hnaive, mplog2(prec)))) + 12;
+    bitneeded = itos(gceil(divrr(hnaive, mplog2(prec)))) + 32;
     if (DEBUGLEVEL) err_printf("precision = %ld\n", bitneeded);
     if (bitprec>=bitneeded) break;
     bitprec = bitneeded;
-    prec = nbits2prec(bitprec)+1;
+    prec = nbits2prec(bitprec) + EXTRAPRECWORD;
   }
   indmult = heegner_indexmult(om, wtor, tam, prec);
   ndisc = maxss(10, (long) rtodbl(ht)/10);
