@@ -456,33 +456,33 @@ derivhasse(GEN f, ulong j)
 }
 
 static GEN
-non_two_torsion_abscissa(GEN E, GEN h0, GEN x)
+non_two_torsion_abscissa(GEN E, GEN h0, long vx)
 {
   GEN mp1, dh0, ddh0, t, u, t1, t2, t3;
   long m = degpol(h0);
   mp1 = gel(h0, m + 1); /* negative of first power sum */
   dh0 = RgX_deriv(h0);
   ddh0 = RgX_deriv(dh0);
-  t = ec_2divpol_evalx(E, x);
-  u = ec_half_deriv_2divpol_evalx(E, x);
+  t = ec_bmodel(E, vx);
+  u = ec_half_deriv_2divpol(E, vx);
   t1 = RgX_sub(RgX_sqr(dh0), RgX_mul(ddh0, h0));
   t2 = RgX_mul(u, RgX_mul(h0, dh0));
   t3 = RgX_mul(RgX_sqr(h0),
-               deg1pol_shallow(stoi(2*m), gmulsg(2L, mp1), varn(x)));
+               deg1pol_shallow(stoi(2*m), gmulsg(2L, mp1), vx));
   /* t * (dh0^2 - ddh0*h0) - u*dh0*h0 + (2*m*x - 2*s1) * h0^2); */
   return RgX_add(RgX_sub(RgX_mul(t, t1), t2), t3);
 }
 
 static GEN
-isog_abscissa(GEN E, GEN kerp, GEN h0, GEN x, GEN two_tors)
+isog_abscissa(GEN E, GEN kerp, GEN h0, long vx, GEN two_tors)
 {
   GEN f0, f2, h2, t1, t2, t3;
-  f0 = (degpol(h0) > 0)? non_two_torsion_abscissa(E, h0, x): pol_0(varn(x));
+  f0 = (degpol(h0) > 0)? non_two_torsion_abscissa(E, h0, vx): pol_0(vx);
   f2 = gel(two_tors, 3);
   h2 = gel(two_tors, 5);
 
   /* Combine f0 and f2 into the final abscissa of the isogeny. */
-  t1 = RgX_mul(x, RgX_sqr(kerp));
+  t1 = RgX_mul(pol_x(vx), RgX_sqr(kerp));
   t2 = RgX_mul(f2, RgX_sqr(h0));
   t3 = RgX_mul(f0, RgX_sqr(h2));
   /* x * kerp^2 + f2 * h0^2 + f0 * h2^2 */
@@ -615,7 +615,7 @@ isogeny_from_kernel_poly(GEN E, GEN kerp, long only_image, long vx, long vy)
                           gadd(w, gel(two_tors, 2)));
   if (only_image) return EE;
 
-  f = isog_abscissa(E, kerp, kerq, x, two_tors);
+  f = isog_abscissa(E, kerp, kerq, vx, two_tors);
   g = isog_ordinate(E, kerp, kerq, x, y, two_tors, f);
   return mkvec2(EE, mkvec3(f,g,kerp));
 }
@@ -1167,69 +1167,61 @@ ellQ_goodl_l(GEN E, long l)
   forprime_t T;
   long i;
   GEN disc = ell_get_disc(E);
-  pari_sp av = avma;
-  u_forprime_init(&T, 17UL, ULONG_MAX);
-  for (i=1; i<=20; i++)
+  pari_sp av;
+  u_forprime_init(&T, 17UL, ULONG_MAX); av = avma;
+  for (i = 1; i <= 20; i++, set_avma(av))
   {
     ulong p = u_forprime_next(&T);
+    long t;
     if (umodiu(disc,p)==0) { i--; continue; }
+    t = itos(ellap(E, utoipos(p)));
+    if (l==2)
+    {
+      if (odd(t)) return 0;
+    }
     else
     {
-      long t = itos(ellap(E, utoipos(p)));
-      if (l==2)
-      {
-        if (odd(t)) return 0;
-      }
-      else
-      {
-        long D = t*t - 4*p;
-        if (kross(D,l) == -1) return 0;
-      }
-      set_avma(av);
+      long D = t*t - 4*p;
+      if (kross(D,l) == -1) return 0;
     }
   }
   return 1;
 }
 
-static ulong
+static GEN
 ellnf_goodl_l(GEN E, GEN v)
 {
   forprime_t T;
-  GEN nf = ellnf_get_nf(E), disc = ell_get_disc(E);
   long i, lv = lg(v);
-  ulong w = 0UL;
-  pari_sp av = avma;
-  u_forprime_init(&T, 17UL,ULONG_MAX);
-  for (i = 1; i <= 20; i++)
+  GEN nf = ellnf_get_nf(E), disc = ell_get_disc(E), w = const_F2v(lv-1);
+  pari_sp av;
+  u_forprime_init(&T, 17UL,ULONG_MAX); av = avma;
+  for (i = 1; i <= 20; i++, set_avma(av))
   {
     ulong p = u_forprime_next(&T);
     GEN pr = idealprimedec(nf, utoipos(p));
-    long j, k, lv = lg(v), g = lg(pr)-1;
-    for (j=1; j<=g; j++)
+    long t, j, k, g = lg(pr)-1;
+    for (j = 1; j <= g; j++)
     {
       GEN prj = gel(pr, j);
-      if (idealval(nf,disc,prj) != 0) {i--; continue;}
-      else
+      if (nfval(nf, disc, prj)) {i--; continue;}
+      t = itos(ellap(E, prj));
+      for(k = 1; k < lv; k++)
       {
-        long t = itos(ellap(E, prj));
-        for(k = 1; k < lv; k++)
+        GEN l = gel(v,k);
+        if (equaliu(l,2))
         {
-          long l = v[k];
-          if (l==2)
-          {
-            if (odd(t)) w |= 1<<(k-1);
-          }
-          else
-          {
-            GEN D = subii(sqrs(t), shifti(pr_norm(prj),2));
-            if (krois(D,l)==-1) w |= 1<<(k-1);
-          }
+          if (odd(t)) F2v_clear(w, k);
+        }
+        else
+        {
+          GEN D = subii(sqrs(t), shifti(pr_norm(prj),2));
+          if (kronecker(D,l)==-1) F2v_clear(w, k);
         }
       }
     }
-    set_avma(av);
   }
-  return w^((1UL<<(lv-1))-1);
+  return w;
 }
 
 static GEN
@@ -1251,7 +1243,7 @@ startor(GEN p, long r)
 }
 
 static GEN
-ellnf_get_degree(GEN E, GEN p)
+stariter_easy(GEN E, GEN p)
 {
   GEN nf = ellnf_get_nf(E), dec = idealprimedec(nf, p);
   long d = nf_get_degree(nf), l = lg(dec), i, k;
@@ -1265,31 +1257,108 @@ ellnf_get_degree(GEN E, GEN p)
   return R;
 }
 
+/* pr^h assumed principal */
+static GEN
+idealgen_minpoly(GEN bnf, GEN pr, GEN h)
+{
+  GEN e = isprincipalfact(bnf, NULL, mkvec(pr), mkvec(h), nf_GEN);
+  return minpoly(basistoalg(bnf, gel(e,2)), 0);
+}
+
+static GEN
+stariter(GEN p, long r)
+{
+  GEN starl = deg1pol_shallow(gen_1, gen_m1, 0);
+  long i;
+  for (i = 1; i <= r; i++) starl = starlaw(starl, p);
+  return starl;
+}
+
+static GEN
+stariter_hard(GEN E, GEN bnf, GEN pr)
+{
+  GEN nf = bnf_get_nf(bnf);
+  long k, d = nf_get_degree(nf), d2 = d>>1;
+  GEN h = cyc_get_expo(bnf_get_cyc(bnf)); /* could use order of pr in Cl_K */
+  GEN pol = idealgen_minpoly(bnf, pr, h);
+  GEN S = startor(pol,12), P = gen_1, polchar;
+  for (k = 0; k <= d2; k++) P = gmul(P, stariter(S,k));
+  polchar = ellnf_charpoly(E, pr);
+  return ZX_resultant(startor(polchar,12*itou(h)), P);
+}
+
 /* Based on a GP script by Nicolas Billerey and Theorems 2.4 and 2.8 of
  * N. Billerey, Criteres d'irreductibilite pour les representations des
  * courbes elliptiques, Int. J. Number Theory 7 (2011), no. 4, 1001-1032. */
 static GEN
-ellnf_prime_degree(GEN E)
+ellnf_prime_degree_hard(GEN E, GEN bad)
+{
+  GEN nf = ellnf_get_nf(E), bnf = ellnf_get_bnf(E), R = gen_0;
+  forprime_t T;
+  long i;
+  if (!bnf) bnf = bnfinit0(nf,1,NULL,DEFAULTPREC);
+  u_forprime_init(&T, 5UL, ULONG_MAX);
+  for (i = 1; i <= 20; i++)
+  {
+    long j, lpr;
+    GEN pri;
+    ulong p = u_forprime_next(&T);
+    if (dvdiu(bad, p)) { i--; continue; }
+    pri = idealprimedec(nf,utoi(p)); lpr = lg(pri);
+    for(j = 1; j < lpr; j++)
+    {
+      GEN R0 = stariter_hard(E,bnf,gel(pri,j)), r;
+      R = gcdii(R, mului(p, R0));
+      if (Z_issquareall(R, &r)) R = r;
+    }
+  }
+  return R;
+}
+
+/* Based on a GP script by Nicolas Billerey and Theorems 2.4 and 2.8 of
+ * N. Billerey, Criteres d'irreductibilite pour les representations des
+ * courbes elliptiques, Int. J. Number Theory 7 (2011), no. 4, 1001-1032. */
+static GEN
+ellnf_prime_degree_easy(GEN E, GEN bad)
 {
   forprime_t T;
   long i;
-  GEN nf = ellnf_get_nf(E), nf_bad = nf_get_ramified_primes(nf);
-  GEN g = ellglobalred(E);
-  GEN N = idealnorm(nf,gel(g,1)), Nfa = prV_primes(gmael(g,4,1));
-  GEN bad = mulii(N, nf_get_disc(nf)), B = gen_0, P;
+  GEN B = gen_0;
   u_forprime_init(&T, 5UL,ULONG_MAX);
   for (i = 1; i <= 20; i++)
   {
     ulong p = u_forprime_next(&T);
-    GEN r;
+    GEN b;
     if (dvdiu(bad, p)) { i--; continue; }
-    B = gcdii(B, ellnf_get_degree(E, utoipos(p)));
-    if (Z_issquareall(B, &r)) B = r;
+    B = gcdii(B, stariter_easy(E, utoipos(p)));
+    if (Z_issquareall(B, &b)) B = b;
   }
-  if (!signe(B)) pari_err_DOMAIN("ellisomat", "E","has",strtoGENstr("CM"),E);
-  P = ZV_union_shallow(ZV_union_shallow(Nfa,nf_bad), gel(Z_factor(absi(B)),1));
-  P = vec_to_vecsmall(P);
-  return shallowextract(P, utoi(ellnf_goodl_l(E, P)));
+  return B;
+}
+
+static GEN
+ellnf_prime_degree(GEN E)
+{
+  GEN nf = ellnf_get_nf(E), nf_bad = nf_get_ramified_primes(nf);
+  GEN g = ellglobalred(E);
+  GEN N = idealnorm(nf,gel(g,1)), Nfa = prV_primes(gmael(g,4,1));
+  GEN bad = mulii(N, nf_get_disc(nf)), P;
+  GEN B = ellnf_prime_degree_easy(E, bad);
+  if (!signe(B))
+  {
+    B = ellnf_prime_degree_hard(E, bad);
+    if (!signe(B))
+    {
+      long D = elliscm(E);
+      if (!D || isintzero(nfisincl(quadpoly(stoi(D)), ellnf_get_nf(E))))
+        pari_err_IMPL("ellisomat, very hard case");
+      return stoi(D);
+    }
+  }
+  if (!signe(B)) return NULL;
+  B = muliu(absi(B), 6);
+  P = ZV_union_shallow(ZV_union_shallow(Nfa,nf_bad), gel(Z_factor(B),1));
+  return vec_to_vecsmall(RgV_F2v_extract_shallow(P, ellnf_goodl_l(E, P)));
 }
 
 static GEN
@@ -1448,24 +1517,28 @@ ellnf_isomat(GEN E, long flag)
 {
   GEN nf = ellnf_get_nf(E);
   GEN v = ellnf_prime_degree(E);
-  long vy = fetch_var_higher();
-  long vx = fetch_var_higher();
-  GEN P = ellnf_modpoly(v,vx,vy);
-  GEN LM = ellnf_isocrv(nf, E, v, P, flag), L = gel(LM,1), M = gel(LM,2);
-  long i, l = lg(L);
-  GEN R = cgetg(l, t_MAT);
-  gel(R,1) = M;
-  for(i = 2; i < l; i++)
+  if (typ(v)!=t_INT)
   {
-    GEN Li = gel(L,i);
-    GEN e = mkvec2(gdivgs(gel(Li,1), -48), gdivgs(gel(Li,2), -864));
-    GEN LMi = ellnf_isocrv(nf, ellinit(e, nf, DEFAULTPREC), v, P, 1);
-    GEN LLi = gel(LMi, 1), Mi = gel(LMi, 2);
-    GEN r = isomat_perm(nf, L, LLi);
-    gel(R,i) = vecpermute(Mi, r);
-  }
-  delete_var(); delete_var();
-  return mkvec2(L, R);
+    long vy = fetch_var_higher();
+    long vx = fetch_var_higher();
+    GEN P = ellnf_modpoly(v,vx,vy);
+    GEN LM = ellnf_isocrv(nf, E, v, P, flag), L = gel(LM,1), M = gel(LM,2);
+    long i, l = lg(L);
+    GEN R = cgetg(l, t_MAT);
+    gel(R,1) = M;
+    for(i = 2; i < l; i++)
+    {
+      GEN Li = gel(L,i);
+      GEN e = mkvec2(gdivgs(gel(Li,1), -48), gdivgs(gel(Li,2), -864));
+      GEN LMi = ellnf_isocrv(nf, ellinit(e, nf, DEFAULTPREC), v, P, 1);
+      GEN LLi = gel(LMi, 1), Mi = gel(LMi, 2);
+      GEN r = isomat_perm(nf, L, LLi);
+      gel(R,i) = vecpermute(Mi, r);
+    }
+    delete_var(); delete_var();
+    return mkvec2(L, R);
+  } else
+    return v;
 }
 
 static GEN
@@ -1487,7 +1560,7 @@ ellisomat(GEN E, long p, long flag)
 {
   pari_sp av = avma;
   GEN r = NULL, nf = NULL;
-  long good = 1;
+  int good = 1;
   if (flag < 0 || flag > 1) pari_err_FLAG("ellisomat");
   if (p < 0) pari_err_PRIME("ellisomat", stoi(p));
   if (p == 1) { flag = 1; p = 0; } /* for backward compatibility */
@@ -1498,23 +1571,21 @@ ellisomat(GEN E, long p, long flag)
       if (p) good = ellQ_goodl_l(E, p);
       break;
     case t_ELL_NF:
-      if (p) good = ellnf_goodl_l(E, mkvecsmall(p));
+      if (p) good = F2v_coeff(ellnf_goodl_l(E, mkvecs(p)), 1);
       nf = ellnf_get_nf(E);
-      if (nf_get_varn(nf) <= 0)
-        pari_err_PRIORITY("ellisomat", nf_get_pol(nf), "<=", 0);
-      if (flag==0 && nf_get_varn(nf) <= 1)
-        pari_err_PRIORITY("ellisomat", nf_get_pol(nf), "<=", 1);
+      if (nf_get_varn(nf) <= 1 - flag)
+        pari_err_PRIORITY("ellisomat", nf_get_pol(nf), "<=", 1 - flag);
       break;
     default: pari_err_TYPE("ellisomat",E);
   }
-  if (!good) r = mkvec2(mkvec(ellisograph_a4a6(E, flag)),matid(1));
+  if (!good) r = mkvec2(mkvec(ellisograph_a4a6(E, flag)), matid(1));
   else
   {
     if (p)
       r = nfmkisomat(nf, p, ellisograph_p(nf, E, p, flag));
     else
       r = nf? ellnf_isomat(E, flag): ellQ_isomat(E, flag);
-    gel(r,1) = list_to_crv(gel(r,1));
+    if (typ(r) == t_VEC) gel(r,1) = list_to_crv(gel(r,1));
   }
   return gerepilecopy(av, r);
 }
@@ -1612,3 +1683,64 @@ ellisotree(GEN E)
   for (i = 1; i <= n; i++) obj_free(gel(vE,i));
   return gerepilecopy(av, mkvec2(vE,M));
 }
+
+/* Shortest path between x and y on the adjencc graph of A-A~
+ * Dijkstra algorithm. */
+static GEN
+shortest_path(GEN A, long x, long y)
+{
+  GEN C, v, w;
+  long n = lg(A)-1, i, l, z, t, m;
+  v = const_vecsmall(n, n); v[x] = 0;
+  w = const_vecsmall(n, 0);
+  for (l = 1; l < n; l++)
+    for (z = 1; z <= n; z++)
+    {
+      if (v[z] != l-1) continue;
+      for (t = 1; t <= n; t++)
+        if (!gequal(gcoeff(A,z,t),gcoeff(A,t,z)) && v[t]>l)
+          { v[t]=l; w[t]=z; }
+    }
+  m = v[y]+1; if (m > n) return NULL;
+  C = cgetg(m+1, t_VECSMALL);
+  for (i = 1; i <= m; i++)
+  {
+    C[m+1-i] = y;
+    y = w[y];
+  }
+  return C;
+}
+
+static GEN
+path_to_manin(GEN A, long i, long k)
+{
+  GEN C = shortest_path(A, i, k);
+  long j, lC = lg(C);
+  GEN c = gen_1;
+  for (j = 1; j < lC-1; j++)
+  {
+    GEN t = gsub(gcoeff(A,C[j], C[j+1]), gcoeff(A,C[j+1], C[j]));
+    if (signe(t) < 0) c = gmul(c, gneg(t));
+  }
+  return c;
+}
+
+GEN
+ellmaninconstant(GEN E)
+{
+  pari_sp av = avma;
+  GEN M, A, vS, L;
+  long i, k, lvS, is_ell = checkell_i(E);
+  L = is_ell ? ellisomat(E,0,1): E;
+  A = gel(ellisotree(L), 2);
+  vS = gel(ellweilcurve(L, NULL), 2); lvS = lg(vS);
+  for (i = 1; i < lvS; i++)
+    if (equali1(gmael(vS,i,1) ) && equali1(gmael(vS,i,2)))
+      break;
+  if (is_ell)
+    return gerepileupto(av, path_to_manin(A, i, 1));
+  M = cgetg(lvS, t_VEC);
+  for (k = 1; k < lvS; k++)
+    gel(M,k) = path_to_manin(A, i, k);
+  return gerepileupto(av, M);
+}
