@@ -45,6 +45,7 @@ ZX_to_nx(GEN z)
 {
   long i, r = lg(z);
   GEN x = cgetg(r, t_VECSMALL);
+  x[1] = ((ulong) z[1])&VARNBITS;
   for (i = 2; i < r; i++) x[i] = itou(gel(z, i));
   return x;
 }
@@ -572,22 +573,14 @@ set_R(GEN T, GEN F, GEN Rs, GEN p, long nf, long r, long s, long u)
 static GEN
 set_e0_e1(ulong el, ulong e, GEN p)
 {
-  ulong s, d0, e0, e1, f0, n, phin, g, up = (lgefint(p)==3)?p[2]:0;
-#ifdef DEBUG
-  ulong d;
-#endif
+  ulong s, d0, e0, e1, f0, n, phin, g, up = itou_or_0(p);
 
   if (el == 2)
   {
-    ulong pmod4 = up ? up&3 : p[2]&3;
+    ulong pmod4 = up ? up&3 : mod4(p);
     if (pmod4 == 3)  /* p+1 = 0 (mod 4) */
     {
       s = up ? vals(up+1) : vali(addiu(p, 1));
-#ifdef DEBUG
-      if (e == 1) d = 1;
-      else if (e <= s) d = 2;
-      else d = 1L<<(e-s);
-#endif
       if (e <= s+1) { e0 = e; e1 = 0;}
       else { e0 = s+1; e1= e-s-1;}
       d0 = e == 1? 1: 2;
@@ -595,9 +588,6 @@ set_e0_e1(ulong el, ulong e, GEN p)
     else  /* p-1 = 0 (mod 4) */
     {
       s = up ? vals(up-1) : vali(subiu(p, 1));
-#ifdef DEBUG
-      d = (e<=s)?1:1L<<(e-s);
-#endif
       if (e <= s) { e0 = e; e1 = 0; }
       else { e0 = s; e1 = e-s; }
       d0 = 1;
@@ -608,15 +598,8 @@ set_e0_e1(ulong el, ulong e, GEN p)
   {
     ulong pmodel = up ? up%el : umodiu(p, el);
     d0 = Fl_order(pmodel, el-1, el);
-#ifdef DEBUG
-    d = d0;
-#endif
     s = Z_lval(subiu(powiu(p, d0), 1), el);
-#ifdef DEBUG
-    if (s < e) d *= upowuu(el, e-s);
-#endif
-    if (e <= s) { e0 = e; e1 = 0; }
-    else {e0 = s; e1= e-s; }
+    if (e <= s) { e0 = e; e1 = 0; } else { e0 = s; e1 = e-s; }
     phin = (el-1)*upowuu(el, e0-1);
   }
   n = upowuu(el, e0); f0 = phin/d0;
@@ -625,7 +608,7 @@ set_e0_e1(ulong el, ulong e, GEN p)
 }
 
 /* return 1 if newton is fast, return 0 if gen is fast */
-static long
+static int
 use_newton(long d, long f)
 {
   if (2*d <= f) return 0;
@@ -640,28 +623,39 @@ use_newton(long d, long f)
   else return 1;
 }
 
-/* return 1 if newton_general is fast, return 0 otherwise */
-static long
-use_newton_general(long d, long f, long max_deg)
+/* return 1 if newton_general is fast, return 0 otherwise. Assume f > 40 */
+static int
+use_newton_general(long d, long f, long maxdeg)
 {
-  if (max_deg == 1) return 0;
-  else if (f <= 40) return 1;
-  else if (max_deg < 20) return 0;
+  if (maxdeg < 20) return 0;
   else if (f <= 50) return 1;
-  else if (max_deg < 30) return 0;
+  else if (maxdeg < 30) return 0;
   else if (f <= 60) return 1;
-  else if (max_deg < 40) return 0;
+  else if (maxdeg < 40) return 0;
   else if (f <= 70) return 1;
-  else if (max_deg < 50) return 0;
+  else if (maxdeg < 50) return 0;
   else if (f <= 80) return 1;
   else if (d < 200) return 0;
   else if (f <= 100) return 1;
   else if (d < 300) return 0;
   else if (f <= 120) return 1;
-  else if (6*max_deg < f*f) return 0;
+  else if (6*maxdeg < f*f) return 0;
   else return 1;
 }
 
+static int
+use_general(long d, long maxdeg)
+{
+  if (d <= 50) return 1;
+  else if (maxdeg <= 3*d) return 0;
+  else if (d <= 200) return 1;
+  else if (maxdeg <= 10*d) return 0;
+  else if (d <= 500) return 1;
+  else if (maxdeg <= 20*d) return 0;
+  else if (d <= 1000) return 1;
+  else return 0;
+}
+
 static void
 update_dfm(long *pd, long *pf, long *pm, long di, long fi)
 {
@@ -669,12 +663,13 @@ update_dfm(long *pd, long *pf, long *pm, long di, long fi)
   *pd = d1 / c; *pf = c * f1; *pm += d1 * d1 * f1;
   if (DEBUGLEVEL == 4) err_printf("(%ld,%ld), ",d1,f1);
 }
+/* assume ord(p mod f) > 1 */
 static ulong
 set_action(GEN fn, GEN p, long d, long f)
 {
   GEN EL = gel(fn, 1), E = gel(fn, 2);
   long i, d0, f0, m0, m1, maxdeg, max, l = lg(EL);
-  ulong action = 0;
+  ulong action = 0, up = itou_or_0(p);
   GEN D = cgetg(l, t_VECSMALL), F = cgetg(l, t_VECSMALL);
 
   d += 10*(lgefint(p)-3);
@@ -693,6 +688,7 @@ set_action(GEN fn, GEN p, long d, long f)
   else if (d <= 40) action |= GENERAL;
   else if (f <= 40) action |= NEWTON_GENERAL_NEW;
   if (action) return action;
+  /* can assume that d > 40 and f > 40 */
 
   maxdeg = max = 1;
   for (i = 1; i < l; i++)
@@ -706,23 +702,15 @@ set_action(GEN fn, GEN p, long d, long f)
     if (DEBUGLEVEL == 4) err_printf("(%ld,%ld), ", D[i], F[i]);
   }
   if (maxdeg == 1) return action;
-  if ((p[2] != 2 && use_newton_general(d, f, maxdeg)) ||
-      (p[2] == 2 && f <= 40))  /* does not decompose n */
+  if (up != 2)
   {
-    action |= (20 < d)? NEWTON_GENERAL_NEW: NEWTON_GENERAL;
-    return action;
+    if (use_newton_general(d, f, maxdeg))
+    { /* does not decompose n */
+      action |= (20 < d)? NEWTON_GENERAL_NEW: NEWTON_GENERAL;
+      return action;
+    }
+    if (use_general(d, maxdeg)) action |= GENERAL;
   }
-
-  if (d <= 20) action |= GENERAL;
-  else if (p[2] == 2) action &= ~GENERAL;
-  else if (d <= 50) action |= GENERAL;
-  else if (maxdeg <= 3*d) action &= ~GENERAL;
-  else if (d <= 200) action |= GENERAL;
-  else if (maxdeg <= 10*d) action &= ~GENERAL;
-  else if (d <= 500) action |= GENERAL;
-  else if (maxdeg <= 20*d) action &= ~GENERAL;
-  else if (d <= 1000) action |= GENERAL;
-  else action &= ~GENERAL;
   if (l < 4) return action; /* n has two factors */
 
   d0 = f0 = 1; m0 = 0;
@@ -1301,18 +1289,14 @@ FpX_factcyclo_prime_power(long el, long e, GEN p, long m)
 static GEN
 FpX_factcyclo_fact(GEN fn, GEN p, ulong m, long ascent)
 {
-  GEN EL = gel(fn, 1), E = gel(fn, 2), v1, v2;
-  long l = lg(EL), i, j, n1, n2;
+  GEN EL = gel(fn, 1), E = gel(fn, 2), v1 = NULL;
+  long i, l = lg(EL), n1 = 1;
 
-  j = ascent? 1: l-1;
-  n1 = upowuu(EL[j], E[j]);
-  v1 = FpX_factcyclo_prime_power(EL[j], E[j], p, m);
-  for (i = 2; i < l; i++)
+  for (i = 1; i < l; i++)
   {
-    j = ascent? i: l-i;
-    n2 = upowuu(EL[j], E[j]);
-    v2 = FpX_factcyclo_prime_power(EL[j], E[j], p, m);
-    v1 = FpX_factcyclo_lift(n1, v1, n2, v2, p, m);
+    long j = ascent? i: l-i, n2 = upowuu(EL[j], E[j]);
+    GEN v2 = FpX_factcyclo_prime_power(EL[j], E[j], p, m);
+    v1 = v1? FpX_factcyclo_lift(n1, v1, n2, v2, p, m): v2;
     n1 *= n2;
   }
   return v1;
@@ -1322,19 +1306,17 @@ static GEN
 Flv_FlvV_factorback(GEN g, GEN x, ulong q)
 { pari_APPLY_ulong(Flv_factorback(g, gel(x,i), q)) }
 
-/* return the structure and the generators of G/H. G=(Z/nZ)^, H=<p> */
+/* return the structure and the generators of G/H. G=(Z/nZ)^, H=<p>.
+ * For efficiency assume p mod n != 1 (trivial otherwise) */
 static GEN
 get_GH_gen(long n, long pmodn)
 {
-  GEN H = znstar_generate(n, mkvecsmall(pmodn)), gH = gel(H, 1);
   GEN G = znstar0(utoipos(n), 1), cycG = znstar_get_cyc(G);
-  long i, lgH = lg(gH);
-  GEN cycGH, gG, gGH, U, Ui;
+  GEN cycGH, gG, gGH, Ui, P;
   ulong expG;
-  GEN P = cgetg(lgH, t_MAT);
-  for (i = 1; i < lgH; i++) gel(P, i) = Zideallog(G, utoi(gH[i]));
-  cycGH = ZV_to_nv(ZM_snf_group(hnfmodid(P, cycG), &U, &Ui));
-  expG = itou(gel(cycG, 1));
+  P = hnfmodid(mkmat(Zideallog(G, utoi(pmodn))), cycG);
+  cycGH = ZV_to_nv(ZM_snf_group(P, NULL, &Ui));
+  expG = itou(cyc_get_expo(cycG));
   gG = ZV_to_Flv(znstar_get_gen(G), n);
   gGH = Flv_FlvV_factorback(gG, ZM_to_Flm(Ui, expG), n);
   return mkvec2(gGH, cycGH);
@@ -1365,20 +1347,13 @@ FpX_factcyclo_just_conductor_init(GEN *pData, ulong n, GEN p, ulong m)
   long phin = eulerphiu_fact(fn), pmodn = umodiu(p, n);
   long d = Fl_order(pmodn, phin, n), f = phin/d; /* d > 1 */
   ulong action = set_action(fn, p, d, f);
-  if (action & GENERAL)
-  {
+  if (action & ~NEWTON_POWER)
+  { /* needed for GENERAL* */
     GEN H = znstar_generate(n, mkvecsmall(pmodn));
     GH = znstar_cosets(n, phin, H); /* representatives of G/H */
     if (action & (NEWTON_GENERAL_NEW | NEWTON_GENERAL))
       GHgen = get_GH_gen(n, pmodn);  /* gen and order of G/H */
   }
-  else if (action & NEWTON_POWER);
-  else if (action & (NEWTON_GENERAL_NEW | NEWTON_GENERAL))
-  {
-    GEN H = znstar_generate(n, mkvecsmall(pmodn));
-    GH = znstar_cosets(n, phin, H); /* representatives of G/H */
-    GHgen = get_GH_gen(n, pmodn);  /* gen and order of G/H */
-  }
   *pData = mkvec5(GHgen, GH, fn, p, mkvecsmall4(n, d, f, m));
   if (DEBUGLEVEL >= 1)
   {
@@ -1401,9 +1376,7 @@ FpX_factcyclo_just_conductor(ulong n, GEN p, ulong m)
   if (action & GENERAL)
     return FpX_factcyclo_gen(gel(Data,2), n, p, m);
   else if (action & NEWTON_POWER)
-  {
     return FpX_factcyclo_prime_power_i(ucoeff(fn,1,1), ucoeff(fn,1,2), p, m);
-  }
   else if (action & NEWTON_GENERAL)
     return FpX_factcyclo_newton_general(Data);
   else if (action & NEWTON_GENERAL_NEW)
@@ -1566,6 +1539,7 @@ Flx_1st_lift_2(GEN z, ulong p)
 {
   long i, r = lg(z);
   GEN x = cgetg(r, t_VECSMALL);
+  x[1] = z[1];
   if (odd(r))
     for (i = 2; i<r; i++) uel(x,i) = odd(i)? Fl_neg(uel(z,i), p) : uel(z,i);
   else
