@@ -245,14 +245,12 @@ hnf_block(GEN m, long r0, long nr, long c0, long nc)
   block = rowreverse(block);
 
   (void)ZM_hnfall(block, &u, 1);
-  vecreverse_inplace(u);
-  uu = matid(nm);
-  /* embed in matid */
-  for (k = 1; k <= nc; k++)
-    gel(uu,c0+k) = embedcol(gel(u,k),nm,c0);
+  vecreverse_inplace(u); uu = matid(nm); /* embed in matid */
+  for (k = 1; k <= nc; k++) gel(uu,c0+k) = embedcol(gel(u,k),nm,c0);
   return gerepilecopy(av, uu);
 }
 
+/* (matrix, starting row, nb rows, starting column, nb columns) */
 static GEN
 lll_block(GEN m, long r0, long nr, long c0, long nc)
 {
@@ -261,9 +259,8 @@ lll_block(GEN m, long r0, long nr, long c0, long nc)
   pari_sp av = avma;
 
   block = matslice(m, r0+1, r0+nr, c0+1, c0+nc);
-  u = lll(block); vecreverse_inplace(u);
-  if (lg(u) <= nc) return NULL;
-  uu = matid(nm); /* embed in matid */
+  u = lll(block); if (lg(u) <= nc) return NULL;
+  vecreverse_inplace(u); uu = matid(nm); /* embed in matid */
   for (k = 1; k <= nc; k++) gel(uu,c0+k) = embedcol(gel(u,k),nm,c0);
   return gerepilecopy(av, uu);
 }
@@ -296,13 +293,13 @@ static GEN
 cm_select(GEN nf, GEN cm, long prec)
 {
   GEN emb, keys, v, m_sel, imag_emb;
-  long nc, d_cm, r_cm, c, i, j, r2 = nf_get_r2(nf);
+  long nalg, d_cm, r_cm, c, i, j, r2 = nf_get_r2(nf);
   pari_sp av;
 
   d_cm = degpol(gel(cm, 1)); /* degree of the cm field; even */
-  nc = d_cm / 2; /* nb of clusters */
-  r_cm = nf_get_degree(nf) / d_cm; /* nb by cluster; nc * r_cm = r2 */
-  m_sel = zeromatcopy(nc, r2); /* selection matrix */
+  nalg = d_cm / 2; /* nb of clusters */
+  r_cm = nf_get_degree(nf) / d_cm; /* nb by cluster; nalg * r_cm = r2 */
+  m_sel = zeromatcopy(nalg, r2); /* selection matrix */
   av = avma;
   /* group complex embeddings */
   emb = nfeltembed(nf, gel(cm, 2), NULL, prec);
@@ -311,7 +308,7 @@ cm_select(GEN nf, GEN cm, long prec)
   keys = gadd(gmul(mppi(prec), real_i(emb)), gabs(imag_emb, prec));
   v = indexsort(keys);
 
-  for (j = c = 1; c <= nc; c++)
+  for (j = c = 1; c <= nalg; c++)
   {
     int ref = gsigne(gel(imag_emb, v[j]));
     gcoeff(m_sel, c, v[j]) = gen_1;
@@ -406,7 +403,7 @@ gcharinit(GEN bnf, GEN mod, long prec)
   long n, k, r1, r2, ns, nc, nu, nm, order;
   long evalprec = prec, nfprec, mprec, embprec;
 
-  prec = evalprec + 1; /* default 1 extra word */
+  prec = evalprec + EXTRAPREC64; /* default 64 extra bits */
 
   /* note on precision:
 
@@ -430,7 +427,7 @@ gcharinit(GEN bnf, GEN mod, long prec)
      If prec is not sufficient, we call gcharnewprec.
 
      To mitigate precision increase, we always initialize the structure
-     with one extra word.
+     with 64 extra bits.
 
      Final remark: a gchar struct may have inconsistent values
      for prec and nfprec, for example if it has been saved and loaded at
@@ -502,7 +499,7 @@ gcharinit(GEN bnf, GEN mod, long prec)
 
   if (DEBUGLEVEL>2) err_printf("start matrix m\n");
   m = cgetg(nm + 1, t_MAT);
-  mprec = 3 + nbits2extraprec(nm+10);
+  mprec = nbits2prec(nm+10) + EXTRAPREC64;
   embprec = mprec;
   for(;;)
   {
@@ -573,7 +570,7 @@ static GEN
 gchar_hnfreduce_shallow(GEN gc, GEN cm)
 {
   GEN bnf = gchar_get_bnf(gc), nf = gchar_get_nf(gc), u, u0, m;
-  long order, r1, r2, ns, nc, n, nu, nm, ncm = 0, mprec;
+  long order, r1, r2, ns, nc, n, nu, nm, nalg = 0, mprec;
 
   nf_get_sign(nf, &r1, &r2);
   n = r1 + 2*r2;
@@ -612,7 +609,7 @@ gchar_hnfreduce_shallow(GEN gc, GEN cm)
     /* reduce on Norm arguments */
     v = cm_select(nf, cm, gchar_get_nfprec(gc));
     if (DEBUGLEVEL>2) err_printf("cm_select -> %Ps\n", v);
-    ncm = nbrows(v);
+    nalg = nbrows(v);
     gchar_set_u0(gc, u0);
     for(;;)
     {
@@ -627,29 +624,26 @@ gchar_hnfreduce_shallow(GEN gc, GEN cm)
       }
       if (emax < bit) break;
       if (DEBUGLEVEL>1) err_printf("cm select: doubling prec\n");
-      mprec = precdbl(mprec);
-      m = gcharmatnewprec_shallow(gc, mprec);
+      mprec = precdbl(mprec); m = gcharmatnewprec_shallow(gc, mprec);
     }
     if (DEBUGLEVEL>2) err_printf("rounded Nargs -> %Ps\n", Nargs);
-    u = hnf_block(Nargs, 0, ncm, ns+nc, n-1);
+    u = hnf_block(Nargs, 0, nalg, ns+nc, n-1);
     u0 = ZM_mul(u0, u); m = RgM_ZM_mul(m, u);
     if (DEBUGLEVEL>2) err_printf("after cm reduction -> %Ps\n", m);
   }
 
   /* apply LLL on Lambda_m, may need to increase prec */
-  gchar_set_nalg(gc, ncm);
+  gchar_set_nalg(gc, nalg);
   gchar_set_u0(gc, u0);
 
   /* TODO factor these two LLL reduction codes in a function? */
-  if (ncm > 0)
+  if (nalg > 0)
   {
     GEN u = NULL;
     while (1)
     {
-      u = lll_block(m, ns+nc, n, ns+nc, ncm); if (u) break;
-      mprec = precdbl(mprec);
-      /* recompute m0 * u0 to higher prec */
-      m = gcharmatnewprec_shallow(gc, mprec);
+      if ((u = lll_block(m, ns+nc, n, ns+nc, nalg))) break;
+      mprec = precdbl(mprec); m = gcharmatnewprec_shallow(gc, mprec);
     }
     u0 = ZM_mul(u0, u); m = RgM_ZM_mul(m, u);
     if (DEBUGLEVEL>1) err_printf("after LLL reduction (CM block) -> %Ps\n", m);
@@ -661,10 +655,8 @@ gchar_hnfreduce_shallow(GEN gc, GEN cm)
     GEN u = NULL;
     while (1)
     {
-      u = lll_block(m, ns+nc, n, ns+nc+ncm, n-1-ncm); if (u) break;
-      mprec = precdbl(mprec);
-      /* recompute m0 * u0 to higher prec */
-      m = gcharmatnewprec_shallow(gc, mprec);
+      if ((u = lll_block(m, ns+nc, n, ns+nc+nalg, n-1-nalg))) break;
+      mprec = precdbl(mprec); m = gcharmatnewprec_shallow(gc, mprec);
     }
     u0 = ZM_mul(u0, u); m = RgM_ZM_mul(m, u);
     if (DEBUGLEVEL>1) err_printf("after LLL reduction (trans block) -> %Ps\n", m);
@@ -727,12 +719,12 @@ static void
 gcharmat_tinverse(GEN gc, GEN m, long prec)
 {
   GEN m_inv;
-  long k, r1, r2, ns, nc, ncm, nm, mprec, bitprec = prec2nbits(prec);
+  long k, r1, r2, ns, nc, nalg, nm, mprec, bitprec = prec2nbits(prec);
 
   nf_get_sign(gchar_get_nf(gc), &r1, &r2);
   ns = gchar_get_ns(gc);
   nc = gchar_get_nc(gc);
-  ncm = gchar_get_nalg(gc);
+  nalg = gchar_get_nalg(gc);
   nm = ns + nc + r1 + 2*r2;
   if (lg(m)==1) { gchar_set_basis(gc,zeromat(0,nm)); return; }
 
@@ -743,14 +735,14 @@ gcharmat_tinverse(GEN gc, GEN m, long prec)
     GEN v0, mm;
     /* insert at column ns+nc+r1+r2, or last column if cm */
     v0 = vec_v0(nm, ns+nc+1, r1, r2);
-    mm = shallowmatinsert(m, v0, ncm? nm: nm-r2);
+    mm = shallowmatinsert(m, v0, nalg? nm: nm-r2);
     if (DEBUGLEVEL>1) err_printf("add v0 -> %Ps\n", mm);
     mm = shallowtrans(mm);
     m_inv = RgM_inv(mm); /* invert matrix, may need to increase prec */
     if (m_inv)
     {
       if (DEBUGLEVEL>1) err_printf("inverse: %Ps\n",m_inv);
-      m_inv = vecsplice(m_inv, ncm? nm: nm-r2); /* remove v0 */
+      m_inv = vecsplice(m_inv, nalg? nm: nm-r2); /* remove v0 */
       if (DEBUGLEVEL>1) err_printf("v0 removed: %Ps\n", m_inv);
       m_inv = shallowtrans(m_inv);
       if (!mprec) break;
@@ -766,7 +758,7 @@ gcharmat_tinverse(GEN gc, GEN m, long prec)
       else targetmprec = 0;
     }
     mprec = maxss(precdbl(mprec), targetmprec);
-    if (mprec < 3) mprec = 3;
+    if (mprec < DEFAULTPREC) mprec = DEFAULTPREC;
     m = gcharmatnewprec_shallow(gc, mprec); /* m0 * u0 to higher prec */
   }
   /* clean rationals */
@@ -782,11 +774,11 @@ gcharmat_tinverse(GEN gc, GEN m, long prec)
     for (k = 1; k <= r2; k++)
       shallow_clean_rat(gel(m_inv,nm-k+1), 1, nm-1, NULL, prec);
   }
-  if (ncm)
+  if (nalg)
   {
     long i, j, e;
     for (i = 1; i<=r1+r2; i++)
-      for (j = 1; j <= ncm; j++)
+      for (j = 1; j <= nalg; j++)
       {
         e = gexpo(gcoeff(m_inv, ns+nc+j, ns+nc+i));
         if (e > -20) /* TODO is this bound too permissive? */
@@ -860,15 +852,14 @@ gcharmatnewprec_shallow(GEN gc, long mprec)
   }
   gchar_set_nf(gc, nf);
   gchar_set_nfprec(gc, nfprec);
-  return gmul(m0, u0);
+  return RgM_ZM_mul(m0, u0);
 }
 
 static void _check_gchar_group(GEN gc, long flag);
 static void
 check_gchar_group(GEN gc) { _check_gchar_group(gc, 0); }
 
-/* increase prec if needed. FIXME: hardcodes gc[8] and gc[11] */
-/* newprec: requested precision for m_inv */
+/* increase prec if needed. newprec: requested precision for m_inv */
 static GEN
 gcharnewprec_i(GEN gc, long newprec)
 {
@@ -884,8 +875,7 @@ gcharnewprec_i(GEN gc, long newprec)
     if (DEBUGLEVEL) pari_warn(warnprec,"gcharnewprec",newprec);
     nfprec += newprec - prec;
     prec = newprec;
-    gel(gc2, 8) = shallowcopy(gel(gc,8));
-    gmael(gc2, 8, 1) = shallowcopy(gmael(gc, 8, 1));
+    gchar_copy_precs(gc, gc2);
     gchar_set_prec(gc2, prec);
     gchar_set_nfprec(gc2, nfprec);
   }
@@ -900,10 +890,10 @@ gcharnewprec_i(GEN gc, long newprec)
   prec0 = gprecision(gchar_get_basis(gc2));
   if (prec0 && prec > prec0)
   {
-    GEN m, cyc;
+    GEN cyc, m, m0 = gchar_get_m0(gc);
     if (DEBUGLEVEL) pari_warn(warnprec,"gcharnewprec (minv)", prec);
-    gel(gc2, 11) = shallowcopy(gel(gc2, 11));
-    mprec = prec + mextraprec(gchar_get_basis(gc), gchar_get_m0(gc), gc);
+    gchar_set_m0(gc2, shallowcopy(m0));
+    mprec = prec + mextraprec(gchar_get_basis(gc), m0, gc);
     m = gcharmatnewprec_shallow(gc2, mprec);
     if (DEBUGLEVEL>2) err_printf("m0*u0 recomputed -> %Ps\n", m);
     gcharmat_tinverse(gc2, m, prec);
@@ -919,7 +909,7 @@ GEN
 gcharnewprec(GEN gc, long newprec)
 {
   pari_sp av = avma;
-  GEN gc2 = gcharnewprec_i(gc, newprec + EXTRAPRECWORD);
+  GEN gc2 = gcharnewprec_i(gc, newprec + EXTRAPREC64);
   gchar_set_evalprec(gc2, newprec);
   return gerepilecopy(av, gc2);
 }
@@ -1239,7 +1229,7 @@ gcharlog_conductor_f(GEN gc, GEN chi, GEN *faN)
   Lsprk = locs_get_Lsprk(zm);
   ufil = locs_get_Lgenfil(zm);
   P = gel(locs_get_famod(zm), 1);
-  l = lg(Lsprk); E = cgetg(l, t_VEC);
+  l = lg(Lsprk); E = cgetg(l, t_COL);
   for (i = 1, ic = gchar_get_ns(gc); i < l ; i++)
   {
     GEN gens = gel(ufil, i), chip;
@@ -1864,7 +1854,7 @@ cleanup_vga(GEN vga, long prec)
   GEN ind;
   long bitprec, i, l;
   if (!prec) return vga; /* already exact */
-  bitprec = bit_accuracy(prec);
+  bitprec = prec2nbits(prec);
   vga = shallowcopy(vga); l = lg(vga);
   for (i = 1; i < l; i++)
   {
