@@ -110,7 +110,7 @@ hnffinal(GEN matgen,GEN perm,GEN* ptdep,GEN* ptB,GEN* ptC)
   lig = nlze + lnz;
   /* H: lnz x lnz [disregarding initial 0 cols], U: col x col */
   H = ZM_hnflll(matgen, &U, 0);
-  H += (lg(H)-1 - lnz); H[0] = evaltyp(t_MAT) | evallg(lnz+1);
+  H += lg(H)-1 - lnz; H[0] = evaltyp(t_MAT) | _evallg(lnz+1);
   /* Only keep the part above the H (above the 0s is 0 since the dep rows
    * are dependent from the ones in matgen) */
   zc = col - lnz; /* # of 0 columns, correspond to units */
@@ -718,7 +718,7 @@ is_RgX(GEN a, long v) { return typ(a) == t_POL && varn(a)==v; }
 static GEN
 gbezout_step(GEN *pa, GEN *pb, GEN *pu, GEN *pv, long vx)
 {
-  GEN a = *pa, b = *pb, d;
+  GEN a = *pa, b = *pb, d, l;
   if (gequal0(a))
   {
     *pa = gen_0; *pu = gen_0;
@@ -727,6 +727,10 @@ gbezout_step(GEN *pa, GEN *pb, GEN *pu, GEN *pv, long vx)
   a = is_RgX(a,vx)? RgX_renormalize(a): scalarpol(a, vx);
   b = is_RgX(b,vx)? RgX_renormalize(b): scalarpol(b, vx);
   d = RgX_extgcd(a,b, pu,pv);
+  l = pollead(d,vx);
+  d = RgX_Rg_div(d,l);
+  *pu = RgX_Rg_div(*pu,l);
+  *pv = RgX_Rg_div(*pv,l);
   if (degpol(d)) { a = RgX_div(a, d); b = RgX_div(b, d); }
   else if (typ(gel(d,2)) == t_REAL && lg(gel(d,2)) <= 3)
 #if 1
@@ -926,7 +930,7 @@ remove_0cols(long r, GEN *pA, GEN *pB, long remove)
 {
   GEN A = *pA, B = *pB;
   long l = lg(A);
-  A += r; A[0] = evaltyp(t_MAT) | evallg(l-r);
+  A += r; A[0] = evaltyp(t_MAT) | _evallg(l-r);
   if (B && remove == 2) { B += r; B[0] = A[0]; }
   *pA = A; *pB = B;
 }
@@ -1072,7 +1076,7 @@ ZpM_echelon(GEN x, long early_abort, GEN p, GEN pm)
   if (co > li)
   {
     x += co - li;
-    x[0] = evaltyp(t_MAT) | evallg(li);
+    x[0] = evaltyp(t_MAT) | _evallg(li);
   }
   return gerepilecopy(av0, x);
 }
@@ -1133,7 +1137,7 @@ zlm_echelon(GEN x, long early_abort, ulong p, ulong pm)
   if (co > li)
   {
     x += co - li;
-    x[0] = evaltyp(t_MAT) | evallg(li);
+    x[0] = evaltyp(t_MAT) | _evallg(li);
   }
   return gerepilecopy(av0, x);
 }
@@ -1291,7 +1295,7 @@ ZM_hnfmodall_i(GEN x, GEN D, long flag)
   else
   {
     x += co - li;
-    x[0] = evaltyp(t_MAT) | evallg(li); /* kill 0 columns */
+    x[0] = evaltyp(t_MAT) | _evallg(li); /* kill 0 columns */
     if (moddiag) x = shallowconcat(x, zerocol(nli));
   }
   if (moddiag)
@@ -1331,7 +1335,7 @@ ZM_hnfmodall_i(GEN x, GEN D, long flag)
     }
     D = optimal_D(x,D);
   }
-  x[0] = evaltyp(t_MAT) | evallg(li); /* kill 0 columns / discard accumulator */
+  x[0] = evaltyp(t_MAT) | _evallg(li); /* kill 0 columns, discard accumulator */
   if (flag & hnf_PART) return x;
 
   for (i = nli; i > 0; i--)
@@ -1424,7 +1428,7 @@ FpM_echelon(GEN x, GEN *pP, GEN p)
   /* rank = iP - 1 */
   setlg(P, iP); vecsmall_sort(P);
   if (co > iP) x += co - iP;
-  x[0] = evaltyp(t_MAT) | evallg(iP);
+  x[0] = evaltyp(t_MAT) | _evallg(iP);
   return x;
 }
 /* given x square of maximal rank with 1 or p on diagonal from hnfmodid
@@ -1571,6 +1575,52 @@ findi_normalize(GEN Aj, GEN B, long j, GEN lambda)
   return r;
 }
 
+static void
+subzi(GEN *a, GEN b)
+{
+  pari_sp av = avma;
+  b = subii(*a, b);
+  if (lgefint(b)<=lg(*a) && isonstack(*a)) { affii(b,*a); set_avma(av); }
+  else *a = b;
+}
+
+static void
+addzi(GEN *a, GEN b)
+{
+  pari_sp av = avma;
+  b = addii(*a, b);
+  if (lgefint(b)<=lg(*a) && isonstack(*a)) { affii(b,*a); set_avma(av); }
+  else *a = b;
+}
+
+/* x + y*z */
+static void
+addmulzii(GEN *x, GEN y, GEN z)
+{
+  long lx = lgefint(*x);
+  pari_sp av;
+  GEN t;
+  long ly = lgefint(y), lz;
+  if (ly == 2) return;
+  lz = lgefint(z);
+  av = avma; (void)new_chunk(lx+ly+lz); /*HACK*/
+  t = mulii(z, y);
+  set_avma(av); return addzi(x,t);
+}
+
+static void
+ZC_lincomb1z_i(GEN X, GEN Y, GEN v, long n)
+{
+  long i;
+  for (i = n; i; i--) addmulzii(&gel(X,i), gel(Y,i), v);
+}
+/* X <- X + v Y (elementary col operation) */
+static void
+ZC_lincomb1z(GEN X, GEN Y, GEN v)
+{
+  if (lgefint(v) != 2) return ZC_lincomb1z_i(X, Y, v, lg(X)-1);
+}
+
 static void
 reduce2(GEN A, GEN B, long k, long j, long *row0, long *row1, GEN lambda, GEN D)
 {
@@ -1581,42 +1631,90 @@ reduce2(GEN A, GEN B, long k, long j, long *row0, long *row1, GEN lambda, GEN D)
   *row1 = findi_normalize(gel(A,k), B,k,lambda);
   if (*row0)
     q = truedivii(gcoeff(A,*row0,k), gcoeff(A,*row0,j));
-  else if (abscmpii(shifti(gcoeff(lambda,j,k), 1), gel(D,j)) > 0)
-    q = diviiround(gcoeff(lambda,j,k), gel(D,j));
   else
-    return;
+  {
+    pari_sp btop = avma;
+    int c = abscmpii(shifti(gcoeff(lambda,j,k), 1), gel(D,j));
+    set_avma(btop);
+    if (c > 0)
+      q = diviiround(gcoeff(lambda,j,k), gel(D,j));
+    else
+      return;
+  }
 
   if (signe(q))
   {
     GEN Lk = gel(lambda,k), Lj = gel(lambda,j);
     togglesign_safe(&q);
-    if (*row0) ZC_lincomb1_inplace(gel(A,k),gel(A,j),q);
-    if (B) ZC_lincomb1_inplace(gel(B,k),gel(B,j),q);
-    gel(Lk,j) = addmulii(gel(Lk,j), q, gel(D,j));
+    if (*row0) ZC_lincomb1z(gel(A,k),gel(A,j),q);
+    if (B) ZC_lincomb1z(gel(B,k),gel(B,j),q);
+    addmulzii(&gel(Lk,j), q, gel(D,j));
     if (is_pm1(q))
     {
       if (signe(q) > 0)
       {
         for (i=1; i<j; i++)
-          if (signe(gel(Lj,i))) gel(Lk,i) = addii(gel(Lk,i), gel(Lj,i));
+          if (signe(gel(Lj,i))) addzi(&gel(Lk,i), gel(Lj,i));
       }
       else
       {
         for (i=1; i<j; i++)
-          if (signe(gel(Lj,i))) gel(Lk,i) = subii(gel(Lk,i), gel(Lj,i));
+          if (signe(gel(Lj,i))) subzi(&gel(Lk,i), gel(Lj,i));
       }
     }
     else
     {
       for (i=1; i<j; i++)
-        if (signe(gel(Lj,i))) gel(Lk,i) = addmulii(gel(Lk,i), q, gel(Lj,i));
+        if (signe(gel(Lj,i))) addmulzii(&gel(Lk,i), q, gel(Lj,i));
     }
   }
 }
-
+static void
+affii2_or_copy_gc(pari_sp av, GEN x, GEN *y, GEN x1, GEN *y1)
+{
+  long l = lg(*y), l1 = lg(*y1);
+  if(x==*y && x1==*y1) return;
+  if (lgefint(x) <= l && isonstack(*y) && lgefint(x1) <= l1 && isonstack(*y1))
+  {
+    affii(x,*y);
+    affii(x1,*y1);
+    set_avma(av);
+  }
+  else if (lgefint(x) <= l && isonstack(*y))
+  {
+    affii(x,*y);
+    *y1 = gerepilecopy(av, x1);
+  }
+  else if (lgefint(x1) <= l1 && isonstack(*y1))
+  {
+    affii(x1,*y1);
+    *y = gerepilecopy(av, x);
+  }
+  else
+  {
+    *y  = cgeti(2*lg(x));
+    *y1 = cgeti(2*lg(x1));
+    affii(x,*y);
+    affii(x1,*y1);
+    gerepileall(av,2,y,y1);
+  }
+}
+static void
+affii_or_copy_gc(pari_sp av, GEN x, GEN *y)
+{
+  long l = lg(*y);
+  if (lgefint(x) <= l && isonstack(*y))
+  {
+    affii(x,*y);
+    set_avma(av);
+  }
+  else
+    *y = gerepileuptoint(av, x);
+}
 static void
 hnfswap(GEN A, GEN B, long k, GEN lambda, GEN D)
 {
+  pari_sp av;
   GEN t, p1, p2, Lk = gel(lambda,k);
   long i,j,n = lg(A);
 
@@ -1625,19 +1723,21 @@ hnfswap(GEN A, GEN B, long k, GEN lambda, GEN D)
   for (j=k-2; j; j--) swap(gcoeff(lambda,j,k-1), gel(Lk,j));
   for (i=k+1; i<n; i++)
   {
+    pari_sp btop = avma;
     GEN Li = gel(lambda,i);
+    if (signe(gel(Li,k-1))==0 && signe(gel(Li,k))==0) continue;
     p1 = mulii(gel(Li,k-1), gel(D,k));
     p2 = mulii(gel(Li,k), gel(Lk,k-1));
     t = subii(p1,p2);
-
     p1 = mulii(gel(Li,k), gel(D,k-2));
     p2 = mulii(gel(Li,k-1), gel(Lk,k-1));
-    gel(Li,k-1) = diviiexact(addii(p1,p2), gel(D,k-1));
-    gel(Li,k) = diviiexact(t, gel(D,k-1));
+    affii2_or_copy_gc(btop, diviiexact(addii(p1,p2), gel(D,k-1)),
+                      &gel(Li,k-1), diviiexact(t, gel(D,k-1)), &gel(Li,k));
   }
+  av = avma;
   p1 = mulii(gel(D,k-2), gel(D,k));
   p2 = sqri(gel(Lk,k-1));
-  gel(D,k-1) = diviiexact(addii(p1,p2), gel(D,k-1));
+  affii_or_copy_gc(av, diviiexact(addii(p1,p2), gel(D,k-1)), &gel(D,k-1));
 }
 
 /* reverse row order in matrix A, IN PLACE */
@@ -2612,7 +2712,7 @@ gsnf_no_divide(GEN x, long i, long vx)
       if (!is_RgX(z,vx)) z = scalarpol(z, vx);
       r = RgX_rem(z, b);
       if (signe(r) && (! isinexactreal(r) ||
-             gexpo(r) > 16 + gexpo(b) - prec2nbits(gprecision(r)))
+             gexpo(r) > 16 + gexpo(b) - gprecision(r))
          ) return k;
     }
   return 0;
@@ -2946,6 +3046,7 @@ snfrank(GEN D, GEN p)
 {
   long i, l;
   if (typ(D) != t_VEC) pari_err_TYPE("snfrank", D);
+  if (!p) p = gen_0;
   l = lg(D);
   if (l == 4 && typ(gel(D,3)) == t_MAT)
   { /* from matsnf(,1) */
