@@ -1321,6 +1321,24 @@ hyperellminimaldisc(GEN W, GEN pr)
   return gerepileuptoint(av, hyperelldisc(C));
 }
 
+static GEN
+qfb3_SL2_apply(GEN q, GEN M)
+{
+  GEN a = gel(q,1), b = gel(q,2), c = gel(q,3);
+  GEN x = gcoeff(M,1,1), y = gcoeff(M,2,1);
+  GEN z = gcoeff(M,1,2), t = gcoeff(M,2,2);
+  GEN by = mulii(b,y), bt = mulii(b,t), bz  = mulii(b,z);
+  GEN a2 = shifti(a,1), c2 = shifti(c,1);
+
+  GEN A1 = mulii(x, addii(mulii(a,x), by));
+  GEN A2 = mulii(c, sqri(y));
+  GEN B1 = mulii(x, addii(mulii(a2,z), bt));
+  GEN B2 = mulii(y, addii(mulii(c2,t), bz));
+  GEN C1 = mulii(z, addii(mulii(a,z), bt));
+  GEN C2 = mulii(c, sqri(t));
+  retmkvec3(addii(A1,A2), addii(B1,B2), addii(C1, C2));
+}
+
 static GEN
 redqfbsplit(GEN a, GEN b, GEN c, GEN d)
 {
@@ -1329,7 +1347,7 @@ redqfbsplit(GEN a, GEN b, GEN c, GEN d)
 
   if (!equali1(w)) { p = diviiexact(p, w); q = diviiexact(q, w); }
   U = mkmat22(p, negi(v), q, u);
-  Q = qfb_ZM_apply(mkvec3(a,b,c), U);
+  Q = qfb3_SL2_apply(mkvec3(a,b,c), U);
   b = gel(Q, 2); c = gel(Q,3);
   if (signe(b) < 0) gel(U,2) = mkcol2(v, negi(u));
   gel(U,2) = ZC_lincomb(gen_1, truedivii(negi(c), d), gel(U,2), gel(U,1));
