@@ -167,7 +167,7 @@ static GEN
 lambertW(GEN z, long k, long prec)
 {
   pari_sp av = avma;
-  long bit = prec2nbits(prec), L = -(bit / 3 + 10), ct = 0, p, pb;
+  long bit = prec2nbits(prec), L = -(bit / 3 + 10), ct = 0, p;
   double wd;
   GEN w, vp;
 
@@ -184,7 +184,7 @@ lambertW(GEN z, long k, long prec)
     wd = dbllambertWfritsch(z, 0);
   if (fabs(wd + 1) < 1e-5)
   {
-    long prec2 = prec + EXTRAPRECWORD;
+    long prec2 = prec + EXTRAPREC64;
     GEN Z = rtor(z, prec2);
     GEN t = addrs(mulrr(Z, gexp(gen_1, prec2)), 1);
     if (signe(t) <= 0) { set_avma(av); return real_m1(prec); }
@@ -196,15 +196,16 @@ lambertW(GEN z, long k, long prec)
     }
     t = sqrtr(shiftr(t, 1));
     w = gprec_w(k == -1? subsr(-1, t) : subrs(t, 1), prec);
-    p = prec - 2; vp = NULL;
+    p = prec; vp = NULL;
   }
   else
   { /* away from -1/e: can reduce accuracy and self-correct */
+    long pb;
     w = wd == 0.? z: dbltor(wd);
-    vp = cgetg(30, t_VECSMALL); ct = 0; pb = bit;
+    vp = cgetg(30, t_VECSMALL); pb = bit;
     while (pb > BITS_IN_LONG * 3/4)
-    { vp[++ct] = (pb + BITS_IN_LONG-1) >> TWOPOTBITS_IN_LONG; pb = (pb + 2) / 3; }
-    p = vp[ct]; w = gprec_w(w, p + 2);
+    { vp[++ct] = nbits2prec(pb); pb = (pb + 2) / 3; }
+    p = vp[ct]; w = gprec_w(w, p);
   }
   if ((k == -1 && (bit < 192 || bit > 640)) || (k == 0 && bit > 1024))
   {
@@ -214,8 +215,8 @@ lambertW(GEN z, long k, long prec)
       ew = mplog(divrr(w, z)); n = addrr(w, ew); d = addrs(w, 1);
       t = divrr(n, shiftr(d, 1));
       w = mulrr(w, subsr(1, divrr(n, addrr(d, t))));
-      if (p >= prec-2 && expo(n) - expo(d) - expo(w) <= L) break;
-      if (vp) { if (--ct) p = vp[ct]; w = gprec_w(w, ct? p + 2: prec); }
+      if (p >= prec && expo(n) - expo(d) - expo(w) <= L) break;
+      if (vp) { if (--ct) p = vp[ct]; w = gprec_w(w, ct? p: prec); }
     }
   }
   else
@@ -226,8 +227,8 @@ lambertW(GEN z, long k, long prec)
       ew = mpexp(w); wew = mulrr(w, ew); n = subrr(wew, z); d = addrr(ew, wew);
       t = divrr(mulrr(addrs(w, 2), n), shiftr(addrs(w, 1), 1));
       w = subrr(w, divrr(n, subrr(d, t)));
-      if (p >= prec-2 && expo(n) - expo(d) - expo(w) <= L) break;
-      if (vp) { if (--ct) p = vp[ct]; w = gprec_w(w, ct? p + 2: prec); }
+      if (p >= prec && expo(n) - expo(d) - expo(w) <= L) break;
+      if (vp) { if (--ct) p = vp[ct]; w = gprec_w(w, ct? p: prec); }
     }
   }
   return gerepileupto(av, w);
@@ -317,7 +318,7 @@ serexp0(long v, long n)
 {
   GEN y = cgetg(n+3, t_SER), t;
   long i;
-  y[1] = evalsigne(1) | evalvarn(v) | evalvalp(0);
+  y[1] = evalsigne(1) | evalvarn(v) | evalvalser(0);
   gel(y,2) = gen_1; if (!n) return y;
   gel(y,3) = gen_1; if (n == 1) return y;
   for (i=2, t = gen_2; i < n; i++, t = muliu(t,i)) gel(y,i+2) = mkfrac(gen_1,t);
@@ -357,7 +358,7 @@ serlambertW(GEN y, long branch, long prec)
   GEN t = NULL;
 
   if (!signe(y)) return gcopy(y);
-  v = valp(y);
+  v = valser(y);
   if (v < 0) pari_err_DOMAIN("lambertw","valuation", "<", gen_0, y);
   if (v > 0 && branch)
     pari_err_DOMAIN("lambertw [k != 0]", "x", "~", gen_0, y);
@@ -367,7 +368,7 @@ serlambertW(GEN y, long branch, long prec)
   if (v)
   {
     t = serexp0(vy, n / val);
-    setvalp(t, 1); t = reverse(t); /* rev(x exp(x)) */
+    setvalser(t, 1); t = reverse(t); /* rev(x exp(x)) */
   }
   else
   {
@@ -446,10 +447,7 @@ glambertW_i(void *E, GEN y, long prec)
 }
 
 GEN
-glambertW(GEN y, long k, long prec)
-{
-  return glambertW_i((void*)k, y, prec);
-}
+glambertW(GEN y, long k, long prec) { return glambertW_i((void*)k, y, prec); }
 GEN
 mplambertW(GEN y, long prec) { return lambertW(y, 0, prec); }
 
