@@ -557,11 +557,7 @@ lfunrtopoles(GEN r)
 {
   long j, l = lg(r);
   GEN v = cgetg(l, t_VEC);
-  for (j = 1; j < l; j++)
-  {
-    GEN rj = gel(r,j), a = gel(rj,1);
-    gel(v,j) = a;
-  }
+  for (j = 1; j < l; j++) gel(v,j) = gmael(r, j, 1);
   gen_sort_inplace(v, (void*)&cmp_universal, cmp_nodata, NULL);
   return v;
 }
@@ -578,7 +574,8 @@ normalize_simple_pole(GEN r, GEN k)
   if (!is_scalar_t(tx)) pari_err_TYPE("lfunrootres [poles]", r);
   return mkvec(mkvec2(k, simple_pole(r)));
 }
-/* normalize the description of a polar part */
+/* check and normalize the description of a polar part as a t_VEC (r for L)
+ * or t_COL (R for Lambda) of pairs [a, ra] */
 static GEN
 normalizepoles(GEN r, GEN k)
 {
@@ -591,7 +588,6 @@ normalizepoles(GEN r, GEN k)
     GEN rj = gel(r,j), a = gel(rj,1), ra = gel(rj,2);
     if (!is_scalar_t(typ(a)) || typ(ra) != t_SER)
       pari_err_TYPE("lfunrootres [poles]",r);
-    if (valser(ra) >= 0) continue;
     gel(v,iv++) = rj;
   }
   setlg(v, iv); return v;
@@ -635,11 +631,14 @@ lfunrtoR_i(GEN ldata, GEN r, GEN eno, long prec)
   for (j = jR = 1; j < lr; j++)
   {
     GEN rj = gel(r,j), a = gel(rj,1), ra = gel(rj,2);
-    GEN Ra = rtoR(a, ra, FVga, N, prec);
-    GEN b = gsub(k, conj_i(a));
-    if (lg(Ra)-2 < -valser(Ra))
+    GEN b, Ra = rtoR(a, ra, FVga, N, prec);
+    long o = -valser(Ra); /* Lambda pole order */
+    if (o <= 0) continue;
+    b = gsub(k, conj_i(a));
+    if (lg(Ra)-2 < o)
       pari_err(e_MISC,
         "please give more terms in L function's Taylor expansion at %Ps", a);
+    setlg(Ra, o + 2); /* truncate useless terms */
     gel(R,jR++) = mkvec2(a, Ra);
     if (!tablesearch(vr, b, (int (*)(GEN,GEN))&cmp_universal))
     {
@@ -689,6 +688,17 @@ lfunthetainit0(GEN ldata, GEN tdom, GEN an2, long m,
   return mkvec3(mkvecsmall(t_LDESC_THETA), ldata, tech);
 }
 
+static long
+antwist_extrabit(GEN Vga, long L)
+{
+  GEN c = vecmin(Vga);
+  if (gsigne(c) > 0)
+  {
+    long e = gtodouble(c) * log2(L);
+    if (e > 32) return e;
+  }
+  return 0;
+}
 /* tdom: 1) positive real number r, t real, t >= r; or
  *       2) [r,a], describing the cone |t| >= r, |arg(t)| <= a */
 static GEN
@@ -697,9 +707,16 @@ lfunthetainit_i(GEN data, GEN tdom, long m, long bit)
   GEN ldata = lfunmisc_to_ldata_shallow(data);
   long b = 32, L = lfunthetacost(ldata, tdom, m, bit), prec = nbits2prec(bit);
   GEN ldatan = ldata_newprec(ldata, prec);
-  GEN an = ldata_vecan(ldata_get_an(ldatan), L, prec);
-  GEN Vga = ldata_get_gammavec(ldatan);
-  if (m == 0 && Vgaeasytheta(Vga)) an = antwist(an, Vga, prec);
+  GEN an = ldata_get_an(ldatan), Vga = ldata_get_gammavec(ldatan);
+  if (m == 0 && Vgaeasytheta(Vga))
+  {
+    long bit = antwist_extrabit(Vga, L);
+    if (bit) prec += nbits2extraprec(bit);
+    an = ldata_vecan(an, L, prec);
+    an = antwist(an, Vga, prec);
+  }
+  else
+    an = ldata_vecan(an, L, prec);
   if (typ(an) != t_VECSMALL) b = maxss(b, gexpo(an));
   return lfunthetainit0(ldatan, tdom, an, m, bit, b);
 }
@@ -1383,7 +1400,7 @@ static GEN
 lfun_init_theta(GEN ldata, GEN eno, struct lfunp *S)
 {
   GEN an2, dual, tdom = NULL, Vga = ldata_get_gammavec(ldata);
-  long L, prec = S->precmax;
+  long L, extrabit = 0, prec = S->precmax;
   if (eno)
     L = S->nmax;
   else
@@ -1392,6 +1409,11 @@ lfun_init_theta(GEN ldata, GEN eno, struct lfunp *S)
     L = maxss(S->nmax, lfunthetacost(ldata, tdom, 0, S->D));
   }
   dual = ldata_get_dual(ldata);
+  if (Vgaeasytheta(Vga))
+  {
+    extrabit = antwist_extrabit(Vga, L);
+    prec += nbits2extraprec(extrabit);
+  }
   S->an = ldata_vecan(ldata_get_an(ldata), L, prec);
   S->bn = typ(dual)==t_INT? NULL: ldata_vecan(dual, S->nmax, prec);
   if (!vgaell(Vga)) lfunparams2(S);
@@ -1402,7 +1424,7 @@ lfun_init_theta(GEN ldata, GEN eno, struct lfunp *S)
     S->vgaell = 1;
   }
   an2 = lg(Vga)-1 == 1? antwist(S->an, Vga, prec): S->an;
-  return lfunthetainit0(ldata, tdom, an2, 0, S->Dmax, 0);
+  return lfunthetainit0(ldata, tdom, an2, 0, S->Dmax, extrabit);
 }
 
 GEN
