@@ -243,10 +243,10 @@ mul_addsub(GEN A, GEN a, GEN b, GEN E)
 }
 
 static GEN
-wd(GEN VCALL, GEN pmd, GEN x, long prec)
+wd(GEN VCALL, GEN xpmd, long prec)
 {
   GEN VC = get_chivec(VCALL), E = get_signat(VCALL), Z = get_chiZ(VCALL);
-  GEN ex, emx, xpmd = gmul(x, pmd), y = NULL;
+  GEN ex, emx, y = NULL;
   long md = get_modulus(VCALL), N = 2*md, k;
   ex = gexp(mulcxI(xpmd), prec); emx = ginv(ex);
   for (k = 1; k <= (md-1) / 2; k++)
@@ -304,14 +304,16 @@ integrand_h0(GEN sel, GEN s, GEN VCALL, GEN x, long prec)
   pari_sp av = avma;
   long md = get_modulus(VCALL);
   GEN r0 = m_r0(sel), aleps = m_aleps(sel), zn, p1;
-  GEN pmd = divru(mppi(prec), md), ix = ginv(x);
+  GEN znpmd, pmd = divru(mppi(prec), md), ix = ginv(x);
+
   zn = gadd(r0, gdivgs(gmul(aleps, gsub(x, ix)), 2));
-  p1 = gmul(expIxy(pmd, gsqr(zn), prec),
-            gmul(gpow(zn, gneg(s), prec), gmul(aleps, gadd(x, ix))));
+  znpmd = gmul(pmd, zn);
+  p1 = gsub(mulcxI(gmul(znpmd, zn)), gmul(s, glog(zn, prec)));
+  p1 = gmul(gexp(p1, prec), gmul(aleps, gadd(x, ix)));
   if (md == 1)
-    p1 = gdiv(mkvec(mulcxI(p1)), gmul2n(gsin(gmul(pmd, zn), prec), 2));
+    p1 = gdiv(mkvec(mulcxI(p1)), gmul2n(gsin(znpmd, prec), 2));
   else
-    p1 = gdivgs(gmul(p1, wd(VCALL, pmd, zn, prec)), -2);
+    p1 = gdivgs(gmul(p1, wd(VCALL, znpmd, prec)), -2);
   return gerepileupto(av, p1);
 }
 
