@@ -1914,12 +1914,19 @@ lfunmfspec_i(GEN lmisc, long bit)
     gel(ve,j) = gel(v,2*j);
     gel(vo,j+1) = gel(v,2*j+1);
   }
-  if (k2 == 1) { om = gen_1;    op = gel(v,1); }
-  else         { om = gel(v,2); op = gel(v,3); }
+  if (k2 == 1) { om = gen_1; op = gel(v,1); }
+  else
+  {
+    om = gel(v,2); op = gel(v,3);
+    if (gexpo(op) < -bit/2) op = gel(v,1);
+    /* should happen only in weight 6 */
+  }
   if (maxss(gexpo(imag_i(om)), gexpo(imag_i(op))) > -bit/2)
     pari_err_TYPE("lfunmfspec", lmisc);
-  ve = gdiv(ve, om);
-  vo = gdiv(vo, op);
+  if(gexpo(om) > -bit/2) ve = gdiv(ve, om);
+  else om = gen_0;
+  if(gexpo(op) > -bit/2) vo = gdiv(vo, op);
+  else op = gen_0;
   return mkvec4(bestappr(ve,B), bestappr(vo,B), om, op);
 }
 GEN
