@@ -124,7 +124,7 @@ gammapoles(GEN Vga, long *pdV, long bit)
 static GEN
 sercoeff(GEN x, long n, long prec)
 {
-  long N = n - valp(x);
+  long N = n - valser(x);
   return (N < 0)? gen_0: gprec_wtrunc(gel(x, N+2), prec);
 }
 
@@ -157,7 +157,7 @@ Kderivsmallinit(GEN ldata, GEN Vga, long m, long bit)
   LA = gammapoles(Vga, &dLA, bit); N = lg(LA)-1;
   prec2 = nbits2prec(dLA + bit * (1 + M_PI*d/C2));
 #if BITS_IN_LONG == 32
-  prec2 += prec2 & 1L;
+  prec2 += odd(prec2lg(prec2)) ? EXTRAPRECWORD: 0;
 #endif
   if (ldata) Vga = ldata_get_gammavec(ldata_newprec(ldata, prec2));
   L = cgetg(N+1, t_VECSMALL);
@@ -247,8 +247,16 @@ static long
 GMi_get_m(GEN K) { return itos( gel(K,3) ); }
 static GEN /* [lj,mj,mat,prec2], Kderivsmall only */
 GMi_get_VS(GEN K) { return gel(K,4); }
-static GEN /* [Ms,cd,A2], Kderivlarge only */
-GMi_get_VL(GEN K) { return gel(K,5); }
+/* K[5] = [Ms,cd,A2], Kderivlarge only */
+static long/*Kderivlarge*/
+GMi_get_status(GEN K) { return itos(gmael3(K,5,1,2)); }
+static GEN/*Kderivlarge*/
+GMi_get_M(GEN K) { return gmael3(K,5,1,1); }
+static GEN/*Kderivlarge*/
+GMi_get_cd(GEN K) { return gmael(K,5,2); }
+static GEN/*Kderivlarge*/
+GMi_get_A2(GEN K) { return gmael(K,5,3); }
+
 static double
 GMi_get_tmax(GEN K, long bitprec)
 { return (typ(GMi_get_VS(K)) == t_INT)? -1.0 : get_tmax(bitprec); }
@@ -261,8 +269,7 @@ Kderivsmall(GEN K, GEN x, GEN x2d, long bitprec)
 {
   GEN VS = GMi_get_VS(K), L = gel(VS,1), M = gel(VS,2), mat = gel(VS,3);
   GEN d2, Lx, x2, S, pi, piA = gel(VS,5);
-  long prec = gel(VS,4)[1], d = GMi_get_degree(K);
-  long j, k, limn, N = lg(L)-1;
+  long j, k, prec = gel(VS,4)[1], d = GMi_get_degree(K), limn, N = lg(L)-1;
   double xd, Wd, Ed = M_LN2*bitprec / d;
 
   xd = maxdd(M_PI*dblmodulus(x2d), 1E-13); /* pi |x|^2/d unless x tiny */
@@ -311,7 +318,7 @@ get_D(long d) { return d <= 2 ? 157. : 180.; }
 static void
 Kderivlarge_optim(GEN K, int abs, GEN t2d, double cd, long *pbitprec, long *pnlim)
 {
-  GEN VL = GMi_get_VL(K), A2 = gel(VL,3);
+  GEN A2 = GMi_get_A2(K);
   long bitprec = *pbitprec, d = GMi_get_degree(K);
   const double D = get_D(d), td = dblmodulus(t2d);
   double a, rtd, E = M_LN2*bitprec;
@@ -341,8 +348,8 @@ Kderivlarge(GEN K, GEN t, GEN t2d, long bitprec0)
 {
   GEN tdA, P, S, pi, z;
   const long d = GMi_get_degree(K);
-  GEN M, VL = GMi_get_VL(K), Ms = gel(VL,1), cd = gel(VL,2), A2 = gel(VL,3);
-  long prec, nlim, status = itos(gel(Ms,2)), m = GMi_get_m(K), bitprec = bitprec0;
+  GEN M = GMi_get_M(K), cd = GMi_get_cd(K), A2 = GMi_get_A2(K);
+  long prec, nlim, status = GMi_get_status(K), m = GMi_get_m(K), bitprec = bitprec0;
 
   Kderivlarge_optim(K, !t, t2d, gtodouble(cd), &bitprec, &nlim);
   if (bitprec <= 0) return gen_0;
@@ -355,10 +362,8 @@ Kderivlarge(GEN K, GEN t, GEN t2d, long bitprec0)
   pi = mppi(prec); z = gmul(pi, t2d);
   P = gmul(gmul(cd, tdA), gexp(gmulsg(-d, z), prec));
   if (m) P = gmul(P, gpowgs(mulsr(-2, pi), m));
-  M = gel(Ms,1);
-  if (status == 2)
-    S = (lg(M) == 2)? gel(M,1) /* constant continued fraction */
-                    : poleval(RgV_to_RgX(M, 0), ginv(z));
+  if (status == 2) /* finite continued fraction */
+    S = (lg(M) == 2)? gel(M,1): poleval(M, ginv(z));
   else
   {
     S = contfraceval_inv(M, z, nlim/2);
@@ -461,9 +466,10 @@ Klargeinit(GEN Vga, long nlimmax, long *status, long prec)
 
   if (Vgaeasytheta(Vga)) { *status = 2; return mkvec(gen_1); }
   /* d >= 2 */
-  *status = 0; prec += prec >> 1;
+  *status = 0;
+  if (prec) prec += nbits2extraprec((prec >> 1) + BITS_IN_LONG);
   SMd = get_SMd(Vga);
-  se = gsinh(RgX_to_ser(pol_x(0), d+2), 0); setvalp(se,0);
+  se = gsinh(RgX_to_ser(pol_x(0), d+2), 0); setvalser(se,0);
   se = gdeflate(se, 0, 2); /* se(x^2) = sinh(x)/x */
   vsinh = gpowers(se, d);
   vd = gpowers(utoipos(2*d), d);
@@ -563,9 +569,10 @@ gammamellininvasymp(GEN Vga, long nlim, long m)
 /* Does the continued fraction of the asymptotic expansion M at oo of inverse
  * Mellin transform attached to Vga have zero Hankel determinants ? */
 static long
-ishankelspec(GEN Vga, GEN M)
+ishankelspec(GEN Vga)
 {
   long status, i, d = lg(Vga)-1;
+  GEN M;
 
   if (d == 5 || d == 7)
   { /* known bad cases: a x 5 and a x 7 */
@@ -588,11 +595,9 @@ ishankelspec(GEN Vga, GEN M)
     }
     if (s0==d2 && (s1==d2 || sm1==d2)) return 1;
   }
-  status = 0;
-  /* Heuristic: if 6 first terms in contfracinit don't fail, assume it's OK */
-  pari_CATCH(e_INV) { status = 1; }
-  pari_TRY { contfracinit(M, minss(lg(M)-2,6)); }
-  pari_ENDCATCH; return status;
+  /* Heuristic: if 6 first terms in contfracinit don't fail, assume OK */
+  M = Klargeinit(Vga, 7, &status, 0);
+  return !contfracinit_i(M, 6);
 }
 
 /* Initialize data for computing m-th derivative of inverse Mellin */
@@ -622,7 +627,7 @@ gammamellininvinit(GEN Vga, long m, long bitprec)
   else
   {
     VS = Kderivsmallinit(ldata, Vga, m, bitprec);
-    if (status == 0 && ishankelspec(Vga, M)) status = 1;
+    if (status == 0 && ishankelspec(Vga)) status = 1;
     if (status == 1)
     { /* a Hankel determinant vanishes => contfracinit is undefined.
          So compute K(t) / (1 - 1/(pi^2*t)) instead of K(t)*/
