@@ -21,6 +21,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #include "pari.h"
 #include "paripriv.h"
 
+#define DEBUGLEVEL DEBUGLEVEL_nfsubfields
+
 typedef struct _poldata {
   GEN pol;
   GEN dis; /* |disc(pol)| */
@@ -1084,14 +1086,7 @@ twoembequation(GEN pol, GEN fa, GEN lambda)
       poly = RgXQX_rem(RgX_shift(poly,1), gel(fa,i), pol);
     }
   }
-
-  for(j=1; j<=dx; j++)
-  {
-    settyp(gel(m,j),t_VEC);
-    gel(m,j) = shallowconcat1(gel(m,j));
-    settyp(gel(m,j),t_COL);
-  }
-
+  for(j=1; j<=dx; j++) gel(m,j) = shallowconcat1(gel(m,j));
   return QM_ker(m);
 }
 
@@ -1253,7 +1248,7 @@ galoissubfieldcm(GEN G, long fl)
 
   /* compute a totally imaginary generator */
   pol = gel(sub,1);
-  emb = liftpol(gel(sub,2));
+  emb = liftpol_shallow(gel(sub,2));
   d = degpol(pol);
   if (!(ZX_deflate_order(pol)%2) && sturm(RgX_deflate(pol,2))==d/2)
   {
@@ -1410,7 +1405,7 @@ maxgen_subfields(GEN pol, GEN fa, long flag)
 {
   pari_sp av = avma;
   GEN principal, ismax, isgene, Lmax = NULL, Lgene, res, V, W, W1;
-  long i, i2, j, flmax, flgene, nbmax = 0, nbgene = 0, dup;
+  long i, i2, j, flmax, flgene, nbmax = 0, nbgene = 0;
 
   if (!flag) return cgetg(1,t_VEC);
   flmax = (flag & subf_MAXIMAL)!=0;
@@ -1425,19 +1420,15 @@ maxgen_subfields(GEN pol, GEN fa, long flag)
   i2 = 1;
   for (i=1; i<lg(principal)-1; i++)
   {
+    long dup = 0;
     V = gel(principal,i);
-    dup = 0;
     j = i2-1;
-    while(j>0 && lg(gel(principal,j))==lg(V))
+    while (j > 0 && lg(gel(principal,j)) == lg(V))
     {
       if (field_is_contained(gel(principal,j),V,0)) { dup=1; break; }
       j--;
     }
-    if (!dup)
-    {
-      gel(principal,i2) = V;
-      i2++;
-    }
+    if (!dup) gel(principal,i2++) = V;
   }
   setlg(principal, i2);
 
@@ -1476,11 +1467,7 @@ maxgen_subfields(GEN pol, GEN fa, long flag)
     Lmax = cgetg(nbmax+1, t_VEC);
     j=1;
     for (i=1; i<lg(principal); i++)
-      if (ismax[i])
-      {
-        gel(Lmax,j) = gel(principal,i);
-        j++;
-      }
+      if (ismax[i]) gel(Lmax,j++) = gel(principal,i);
   }
 
   if (flgene)
@@ -1488,11 +1475,7 @@ maxgen_subfields(GEN pol, GEN fa, long flag)
     Lgene = cgetg(nbgene+1, t_VEC);
     j=1;
     for (i=1; i<lg(principal); i++)
-      if (isgene[i])
-      {
-        gel(Lgene,j) = gel(principal,i);
-        j++;
-      }
+      if (isgene[i]) gel(Lgene,j++) = gel(principal,i);
   }
 
   if (!flgene) res = Lmax;
