@@ -786,14 +786,17 @@ static GEN
 eint1p(GEN x, GEN expx)
 {
   pari_sp av;
-  long l = realprec(x), bit = prec2nbits(l), prec, i;
+  long prec = realprec(x), bit = prec2nbits(prec), i;
   double mx;
   GEN z, S, t, H, run;
 
   if (gamma_use_asymp(x, bit)
-      && (z = eint1r_asymp(x, expx, l))) return z;
+      && (z = eint1r_asymp(x, expx, prec))) return z;
   mx = rtodbl(x);
-  prec = l + nbits2extraprec((mx+log(mx))/M_LN2 + 10);
+  if (mx > 1)
+    prec += nbits2extraprec((mx+log(mx))/M_LN2 + 10);
+  else
+    prec += EXTRAPREC64;
   bit = prec2nbits(prec);
   run = real_1(prec); x = rtor(x, prec);
   av = avma; S = z = t = H = run;
@@ -1648,15 +1651,17 @@ veczetas(long a, long b, long N, long prec)
     gel(z,j+1) = rdivii(shifti(gel(z,j+1), k), subii(shifti(d,k), d), prec);
   return z;
 }
-/* zeta(a*j+b), j=0..N-1, b>1, using sumalt */
+/* zeta(a*j+b), j=0..N-1, b > 1, a*(N-1) + b > 1, using sumalt.
+ * a <= 0 is allowed (including the silly a = 0) */
 GEN
 veczeta(GEN a, GEN b, long N, long prec)
 {
   pari_sp av = avma;
   long n, j, k;
-  GEN L, c, d, z = zerovec(N);
+  GEN L, c, d, z;
   if (typ(a) == t_INT && typ(b) == t_INT)
     return gerepilecopy(av, veczetas(itos(a),  itos(b), N, prec));
+  z = zerovec(N);
   n = ceil(2 + prec2nbits_mul(prec, M_LN2/1.7627));
   c = d = int2n(2*n-1);
   for (k = n; k; k--)
@@ -1882,7 +1887,7 @@ czeta(GEN s0, long prec)
   if (funeq_factor) y = gmul(y, funeq_factor);
   set_avma(av); return affc_fixlg(y,res);
 }
-/* v a t_VEC/t_COL */
+/* v a t_VEC/t_COL; is v[i] = a + b (i-1) for some a,b ? */
 int
 RgV_is_arithprog(GEN v, GEN *a, GEN *b)
 {
@@ -1919,11 +1924,12 @@ gzeta(GEN x, long prec)
     {
       GEN a, b;
       long n = lg(x) - 1;
-      if (n > 1 && RgV_is_arithprog(x, &a, &b))
+      if (n > 1 && RgV_is_arithprog(x, &b, &a))
       {
-        if (!is_real_t(typ(a)) || !is_real_t(typ(b)) || gcmpgs(a, 1) <= 0)
-        { set_avma(av); break; }
-        a = veczeta(b, a, n, prec);
+        if (!is_real_t(typ(a)) || !is_real_t(typ(b))
+            || gcmpgs(gel(x,1), 1) <= 0
+            || gcmpgs(gel(x,n), 1) <= 0) { set_avma(av); break; }
+        a = veczeta(a, b, n, prec);
         settyp(a, typ(x)); return a;
       }
     }
