@@ -4,7 +4,8 @@ This file is part of the PARI/GP package.
 
 PARI/GP is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
-Foundation. It is distributed in the hope that it will be useful, but WITHOUT
+Foundation; either version 2 of the License, or (at your option) any later
+version. It is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
@@ -41,8 +42,7 @@ whatnow(PariOUT *out, const char *s, int flag)
    * whatnow() is never used in a case where speed would be necessary */
   if (!wp->old)
   {
-    if (!flag)
-      msg(out, "As far as I can recall, this function never existed");
+    if (!flag) msg(out, "This function did not exist in Pari 1.39");
     return 0;
   }
   def = wp->name;
