@@ -40,6 +40,8 @@ enum {
   ROt_MV = 1, /* Move */
   ROt_PT,  /* Point */
   ROt_LN,  /* Line */
+  ROt_AC,  /* Arc */
+  ROt_FAC, /* Filled Arc */
   ROt_BX,  /* Box */
   ROt_FBX, /* Filled Box */
   ROt_MP,  /* Multiple point */
@@ -545,6 +547,37 @@ void
 plotrbox(long ne, GEN gx2, GEN gy2, long f)
 { rectbox0(ne, gtodouble(gx2), gtodouble(gy2), 1, f); }
 
+static void
+rectarc0(long ne, double gx2, double gy2, long relative, long filled)
+{
+  double xx, yy, x1, y1, x2, y2, xmin, ymin, xmax, ymax;
+  PariRect *e = check_rect_init(ne);
+  RectObj *z = (RectObj*) pari_malloc(sizeof(RectObj2P));
+
+  x1 = RXcursor(e)*RXscale(e) + RXshift(e);
+  y1 = RYcursor(e)*RYscale(e) + RYshift(e);
+  if (relative)
+  { xx = RXcursor(e)+gx2; yy = RYcursor(e)+gy2; }
+  else
+  {  xx = gx2; yy = gy2; }
+  x2 = xx*RXscale(e) + RXshift(e);
+  y2 = yy*RYscale(e) + RYshift(e);
+  xmin = maxdd(mindd(mindd(x1,x2),RXsize(e)),0);
+  xmax = maxdd(mindd(maxdd(x1,x2),RXsize(e)),0);
+  ymin = maxdd(mindd(mindd(y1,y2),RYsize(e)),0);
+  ymax = maxdd(mindd(maxdd(y1,y2),RYsize(e)),0);
+
+  RoType(z) = filled ? ROt_FAC: ROt_AC;
+  RoACx1(z) = xmin; RoACy1(z) = ymin;
+  RoACx2(z) = xmax; RoACy2(z) = ymax;
+  Rchain(e, z);
+  RoCol(z) = current_color[ne];
+}
+
+void
+plotarc(long ne, GEN gx2, GEN gy2, long f)
+{ rectarc0(ne, gtodouble(gx2), gtodouble(gy2), 0, f); }
+
 static void
 freeobj(RectObj *z) {
   switch(RoType(z)) {
@@ -1511,35 +1544,33 @@ plotrecthrawin(GEN fmt, PARI_plot *W, long ne, dblPointList *data, long flags)
   set_range(x.ysml, x.ybig, &ysml, &ybig);
   if (W)
   { /* actual output; else output to rectwindow: no labels */
-    const long se = NUMRECT-2;
-    long lm, rm, tm, bm;
     char YBIG[16], YSML[16], XSML[16], XBIG[16];
+    long lm = W->hunit-1, rm = W->hunit-1, tm = W->vunit-1;
+    long bm = W->vunit+W->fheight-1;
     /* left/right/top/bottom margin */
-    if (flags&PLOT_NOMINMAX)
-    {
-      YBIG[0]=0; YSML[0]=0; XSML[0]=0; XBIG[0]=0;
-    } else
+    if (!(flags&PLOT_NOMINMAX))
     {
       sprintf(YSML,"%.5g", ysml); sprintf(YBIG,"%.5g", ybig);
       sprintf(XSML,"%.5g", xsml); sprintf(XBIG,"%.5g", xbig);
+      /* left margin has y labels with hgap on both sides of text */
+      lm = maxss(strlen(YSML), strlen(YBIG)) * W->fwidth + 2*W->hunit-1;
     }
-    /* left margin has y labels with hgap on both sides of text */
-    lm = maxss(strlen(YSML),strlen(YBIG))*W->fwidth + 2*W->hunit-1;
-    rm = W->hunit-1;
-    tm = W->vunit-1;
-    bm = W->vunit+W->fheight-1;
-    w[0] = wx[0] = wy[0] = evaltyp(t_VECSMALL) | evallg(3);
-    w[1] = se; wx[1] = 0;  wy[1] = 0;
-    w[2] = ne; wx[2] = lm; wy[2] = tm;
+    w[0] = evaltyp(t_VECSMALL) | _evallg((flags&PLOT_NOMINMAX)? 2: 3);
+    wx[0] = wy[0] = w[0];
+    w[1] = ne; wx[1] = lm; wy[1] = tm;
    /* Window (width x height) is given in pixels, correct pixels are 0..n-1,
     * whereas rect functions work with windows whose pixel range is [0,n] */
-    initrect_i(se, W->width - 1, W->height - 1);
     initrect_i(ne, W->width - (lm+rm) - 1, W->height - (tm+bm) - 1);
-    /* draw labels on se */
-    _move(se,lm,0); plotstring(se, YBIG, RoSTdirRIGHT|RoSTdirHGAP|RoSTdirTOP);
-    _move(se,lm,W->height-bm); plotstring(se,YSML, RoSTdirRIGHT|RoSTdirHGAP|RoSTdirVGAP);
-    _move(se,lm,W->height-bm); plotstring(se, XSML, RoSTdirLEFT|RoSTdirTOP);
-    _move(se,W->width-rm-1, W->height-bm); plotstring(se, XBIG, RoSTdirRIGHT|RoSTdirTOP);
+    if (!(flags&PLOT_NOMINMAX))
+    { /* draw labels on se */
+      const long se = NUMRECT-2;
+      w[2] = se; wx[2] = 0;  wy[2] = 0;
+      initrect_i(se, W->width - 1, W->height - 1);
+      _move(se,lm,0); plotstring(se, YBIG, RoSTdirRIGHT|RoSTdirHGAP|RoSTdirTOP);
+      _move(se,lm,W->height-bm); plotstring(se,YSML, RoSTdirRIGHT|RoSTdirHGAP|RoSTdirVGAP);
+      _move(se,lm,W->height-bm); plotstring(se, XSML, RoSTdirLEFT|RoSTdirTOP);
+      _move(se,W->width-rm-1, W->height-bm); plotstring(se, XBIG, RoSTdirRIGHT|RoSTdirTOP);
+    }
   }
   if (!(flags & PLOT_NO_RESCALE)) plotscale0(ne, xsml, xbig, ysml, ybig);
   if (!(flags & PLOT_NO_FRAME))
@@ -1613,8 +1644,7 @@ plotrecthrawin(GEN fmt, PARI_plot *W, long ne, dblPointList *data, long flags)
   {
     GEN s = NULL;
     if (fmt) s = fmt_convert(fmt, w, wx, wy, W); else Draw(W, w,wx,wy);
-    plotkill(w[1]);
-    plotkill(w[2]);
+    for (i = 1; i < lg(w); i++) plotkill(w[i]);
     if (fmt) return s;
   }
   set_avma(av);
@@ -1806,9 +1836,9 @@ gendraw(PARI_plot *T, GEN wxy, long flag)
   l = lg(w);
   /* malloc mandatory in case draw() forks then pari_close(). Done after
    * wxy_init to avoid leak on error */
-  W = cgetalloc(t_VECSMALL, l);
-  X = cgetalloc(t_VECSMALL, l);
-  Y = cgetalloc(t_VECSMALL, l);
+  W = cgetalloc(l, t_VECSMALL);
+  X = cgetalloc(l, t_VECSMALL);
+  Y = cgetalloc(l, t_VECSMALL);
   for (i = 1; i < l; i++) { W[i] = w[i]; X[i] = x[i]; Y[i] = y[i]; }
   Draw(T,W,X,Y);
   pari_free(W); pari_free(X); pari_free(Y);
@@ -1858,6 +1888,22 @@ gen_draw(struct plot_eng *eng, GEN w, GEN x, GEN y, double xs, double ys)
         eng->ln(data, DTOL((RoLNx1(R)+x0)*xs), DTOL((RoLNy1(R)+y0)*ys),
                       DTOL((RoLNx2(R)+x0)*xs), DTOL((RoLNy2(R)+y0)*ys));
         break;
+      case ROt_AC:
+        eng->sc(data,col);
+        eng->ac(data,
+                DTOL((RoBXx1(R)+x0)*xs),
+                DTOL((RoBXy1(R)+y0)*ys),
+                DTOL((RoBXx2(R)-RoBXx1(R))*xs),
+                DTOL((RoBXy2(R)-RoBXy1(R))*ys));
+        break;
+      case ROt_FAC:
+        eng->sc(data,col);
+        eng->fa(data,
+                DTOL((RoBXx1(R)+x0)*xs),
+                DTOL((RoBXy1(R)+y0)*ys),
+                DTOL((RoBXx2(R)-RoBXx1(R))*xs),
+                DTOL((RoBXy2(R)-RoBXy1(R))*ys));
+        break;
       case ROt_BX:
         eng->sc(data,col);
         eng->bx(data,
@@ -1969,6 +2015,27 @@ svg_line(void *data, long x1, long y1, long x2, long y2)
   str_printf(S, "style='fill:none;stroke:%s;'/>", data_hexcolor(data));
 }
 
+static void
+svg_ell(void *data, long x, long y, long w, long h)
+{
+  pari_str *S = data_str(data);
+  float sx = svg_rescale(x), sy = svg_rescale(y), sw = svg_rescale(w), sh = svg_rescale(h);
+  str_printf(S, "<ellipse cx='%.2f' cy='%.2f' rx='%.2f' ry='%.2f' ",
+    sx+sw/2, sy+sh/2, sw/2, sh/2);
+  str_printf(S, "style='fill:none;stroke:%s;'/>", data_hexcolor(data));
+}
+
+static void
+svg_fillell(void *data, long x, long y, long w, long h)
+{
+  pari_str *S = data_str(data);
+  const char * color = data_hexcolor(data);
+  float sx = svg_rescale(x), sy = svg_rescale(y), sw = svg_rescale(w), sh = svg_rescale(h);
+  str_printf(S, "<ellipse cx='%.2f' cy='%.2f' rx='%.2f' ry='%.2f' ",
+    sx+sw/2, sy+sh/2, sw/2, sh/2);
+  str_printf(S, "style='fill:%s;stroke:%s;'/>", color, color);
+}
+
 static void
 svg_rect(void *data, long x, long y, long w, long h)
 {
@@ -2076,6 +2143,8 @@ rect2svg(GEN w, GEN x, GEN y, PARI_plot *T)
   pl.sc = &svg_color;
   pl.pt = &svg_point;
   pl.ln = &svg_line;
+  pl.ac = &svg_ell;
+  pl.fa = &svg_fillell;
   pl.bx = &svg_rect;
   pl.fb = &svg_fillrect;
   pl.mp = &svg_points;
@@ -2119,6 +2188,22 @@ ps_line(void *data, long x1, long y1, long x2, long y2)
   str_printf(S,"stroke\n");
 }
 
+static void
+ps_arc(void *data, long x, long y, long w, long h)
+{
+  pari_str *S = (pari_str*)data;
+  str_printf(S,"matrix currentmatrix %ld %ld translate %ld %ld scale 1 0 moveto 0 0 1 0 360 arc closepath setmatrix currentlinejoin 0 setlinejoin stroke setlinejoin\n",
+         y+h/2,x+w/2,h/2,w/2);
+}
+
+static void
+ps_fillarc(void *data, long x, long y, long w, long h)
+{
+  pari_str *S = (pari_str*)data;
+  str_printf(S,"matrix currentmatrix %ld %ld translate %ld %ld scale 1 0 moveto 0 0 1 0 360 arc closepath setmatrix currentlinejoin 0 setlinejoin fill setlinejoin\n",
+         y+h/2,x+w/2,h/2,w/2);
+}
+
 static void
 ps_rect(void *data, long x, long y, long w, long h)
 {
@@ -2207,6 +2292,8 @@ PS_SCALE, PS_SCALE, DTOL(T->fheight * xs));
   pl.sc = &ps_sc;
   pl.pt = &ps_point;
   pl.ln = &ps_line;
+  pl.ac = &ps_arc;
+  pl.fa = &ps_fillarc;
   pl.bx = &ps_rect;
   pl.fb = &ps_fillrect;
   pl.mp = &ps_points;
@@ -2229,6 +2316,7 @@ pari_plot_by_file(const char *env, const char *suf, const char *img)
 {
   const char *cmd, *s = pari_unique_filename_suffix("plotfile", suf);
   FILE *f = fopen(s, "w");
+  pari_timer ti;
   if (!f) pari_err_FILE("image file", s);
   fputs(img, f); (void)fclose(f);
   cmd = os_getenv(env);
@@ -2238,8 +2326,12 @@ pari_plot_by_file(const char *env, const char *suf, const char *img)
   if (!cmd) cmd = "open -W";
 #endif
   cmd = pari_sprintf("%s \"%s\" 2>/dev/null", cmd, s);
+  walltimer_start(&ti);
   gpsystem(cmd);
-  pari_unlink(s);
+  /* Some image viewers will return immediately and will not display
+     files that have been deleted. In that case we have to leak the file.
+   */
+  if (walltimer_delay(&ti)>1000) pari_unlink(s);
   pari_free((char*)s);
 }
 
