@@ -26,7 +26,6 @@ typedef struct pari_str {
   int use_stack; /* use stack_malloc instead of malloc ? */
 } pari_str;
 
-typedef unsigned char *byteptr;
 typedef ulong pari_sp;
 
 struct pari_sieve
@@ -41,9 +40,10 @@ typedef struct {
   int strategy; /* 1 to 4 */
   GEN bb; /* iterate through primes <= bb */
   ulong c, q; /* congruent to c (mod q) */
+  GEN qq;
 
   /* strategy 1: private prime table */
-  byteptr d; /* diffptr + n */
+  ulong n; /* 1 <= n <= pari_PRIMES[0] */
   ulong p; /* current p = n-th prime */
   ulong b; /* min(bb, ULONG_MAX) */
 
@@ -211,6 +211,7 @@ extern THREAD struct pari_mainstack *pari_mainstack;
 
 struct pari_global_state
 {
+  long debugvar;
   long bitprec;
   GEN primetab;
   GEN seadata;
@@ -253,6 +254,14 @@ typedef struct
   struct parfor_iter iter;
 } parfor_t;
 
+typedef struct
+{
+  GEN a, b, s;
+  long i;
+  int (*cmp)(GEN,GEN);
+  struct parfor_iter iter;
+} parforstep_t;
+
 typedef struct
 {
   GEN x, W;
@@ -348,6 +357,7 @@ typedef struct {
   int secure, simplify, strictmatch, strictargs, chrono;
   pari_timer *T, *Tw;
   ulong primelimit; /* deprecated */
+  ulong factorlimit;
   ulong threadsizemax, threadsize;
 } gp_data;
 extern gp_data *GP_DATA;
@@ -372,7 +382,7 @@ enum { TEXSTYLE_PAREN=2, TEXSTYLE_BREAK=4 };
 
 extern THREAD pari_sp avma;
 #define DISABLE_MEMUSED (size_t)-1
-extern byteptr diffptr;
+extern pari_prime *pari_PRIMES;
 extern char *current_psfile, *pari_datadir;
 
 #define gcopyifstack(x,y)  STMT_START {pari_sp _t=(pari_sp)(x); \
@@ -385,10 +395,17 @@ extern char *current_psfile, *pari_datadir;
   (y)=(_t>=pari_mainstack->bot &&_t<pari_mainstack->top)? \
       icopy((GEN)_t): (GEN)_t;} STMT_END
 
+#define DEBUG_TUNE_SYSTEM(var,s) \
+{\
+  static long _once, _lastt;\
+  if(_lastt!=var) { _lastt=var; _once=0; }\
+  if (_once==0) {_once=1; err_printf("t=%ld s=%ld\n",var,s); }\
+}
+
 /* Define this to thoroughly check "random" garbage collecting */
 #ifdef DEBUG_LOWSTACK
 #  define low_stack(x,l) 1
-#  define gc_needed(av,n) 1
+#  define gc_needed(av,n) ((void)av,1)
 #else
 #  define low_stack(x,l) ((void)(x),avma < (pari_sp)(l))
 #  define gc_needed(av,n) (avma < (pari_sp)stack_lim(av,n) && (av) > ((pari_mainstack->bot >> 1) + (pari_mainstack->top >> 1)))
