@@ -23,8 +23,8 @@ NOASM bfffo divll
 */
 #ifdef ASMINLINE
 
-#define LOCAL_HIREMAINDER  register ulong hiremainder
-#define LOCAL_OVERFLOW     register ulong overflow
+#define LOCAL_HIREMAINDER  ulong hiremainder
+#define LOCAL_OVERFLOW     ulong overflow
 
 #define addll(a,b) \
 __extension__ ({ ulong __value, __arg1 = (a), __arg2 = (b); \
