@@ -36,7 +36,7 @@ extern ulong hiremainder;
 
 #if defined(__GNUC__) && !defined(DISABLE_INLINE)
 #undef LOCAL_HIREMAINDER
-#define LOCAL_HIREMAINDER register ulong hiremainder
+#define LOCAL_HIREMAINDER ulong hiremainder
 
 #define divll(n0, d)                                                    \
 __extension__ ({                                                        \
