@@ -17,6 +17,13 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
  * These functions can be inline; they are also defined externally in
  * mpinl.c, which includes this file and never needs to be changed */
 
+INLINE long
+nbits2lg(long x) {
+  return (long)(((ulong)x+3*BITS_IN_LONG-1) >> TWOPOTBITS_IN_LONG);
+}
+INLINE long
+lg2prec(long x){ return (x-2) * BITS_IN_LONG; }
+
 INLINE long
 evallg(long x)
 {
@@ -31,6 +38,13 @@ evalvalp(long x)
   return v;
 }
 INLINE long
+evalvalser(long x)
+{
+  long v = _evalvalser(x);
+  if (v & ~VALSERBITS) pari_err_OVERFLOW("valser()");
+  return v;
+}
+INLINE long
 evalexpo(long x)
 {
   long v = _evalexpo(x);
@@ -118,6 +132,12 @@ INLINE GEN
 gc_NULL(pari_sp av) { set_avma(av); return NULL; }
 INLINE GEN
 gc_const(pari_sp av, GEN x) { set_avma(av); return x; }
+INLINE GEN
+gc_stoi(pari_sp av, long x) { set_avma(av); return stoi(x); }
+INLINE GEN
+gc_utoi(pari_sp av, ulong x) { set_avma(av); return utoi(x); }
+INLINE GEN
+gc_utoipos(pari_sp av, ulong x) { set_avma(av); return utoipos(x); }
 
 INLINE GEN
 new_chunk(size_t x) /* x is a number of longs */
@@ -218,15 +238,17 @@ cgetineg(long x)
 INLINE GEN
 cgetr_block(long x)
 {
-  GEN z = newblock((size_t)x);
-  z[0] = CLONEBIT | evaltyp(t_REAL) | evallg(x);
+  long l = nbits2lg(x);
+  GEN z = newblock((size_t)l);
+  z[0] = CLONEBIT | evaltyp(t_REAL) | evallg(l);
   return z;
 }
 INLINE GEN
 cgetr(long x)
 {
-  GEN z = new_chunk((size_t)x);
-  z[0] = evaltyp(t_REAL) | evallg(x);
+  long l = nbits2lg(x);
+  GEN z = new_chunk((size_t)l);
+  z[0] = evaltyp(t_REAL) | evallg(l);
   return z;
 }
 
@@ -407,25 +429,25 @@ umuluu_le(ulong x, ulong y, ulong n)
 }
 
 INLINE GEN
-real_0_bit(long bitprec) { GEN x=cgetr(2); x[1]=evalexpo(bitprec); return x; }
+real_0_bit(long bitprec) { GEN x=cgetg(2, t_REAL); x[1]=evalexpo(bitprec); return x; }
 INLINE GEN
-real_0(long prec) { return real_0_bit(-prec2nbits(prec)); }
+real_0(long prec) { return real_0_bit(-prec); }
 INLINE GEN
 real_1_bit(long bit) { return real_1(nbits2prec(bit)); }
 INLINE GEN
 real_1(long prec) {
   GEN x = cgetr(prec);
-  long i;
+  long i, l = lg(x);
   x[1] = evalsigne(1) | _evalexpo(0);
-  x[2] = (long)HIGHBIT; for (i=3; i<prec; i++) x[i] = 0;
+  x[2] = (long)HIGHBIT; for (i=3; i<l; i++) x[i] = 0;
   return x;
 }
 INLINE GEN
 real_m1(long prec) {
   GEN x = cgetr(prec);
-  long i;
+  long i, l = lg(x);
   x[1] = evalsigne(-1) | _evalexpo(0);
-  x[2] = (long)HIGHBIT; for (i=3; i<prec; i++) x[i] = 0;
+  x[2] = (long)HIGHBIT; for (i=3; i<l; i++) x[i] = 0;
   return x;
 }
 
@@ -891,7 +913,7 @@ remis(GEN x, long y)
 {
   pari_sp av = avma;
   long r;
-  (void)divis_rem(x,y, &r); set_avma(av); return stoi(r);
+  (void)divis_rem(x,y, &r); return gc_stoi(av, r);
 }
 
 INLINE GEN
@@ -922,20 +944,20 @@ rdivss(long x, long y, long prec)
 INLINE void
 rdiviiz(GEN x, GEN y, GEN z)
 {
-  long prec = realprec(z), lx = lgefint(x), ly = lgefint(y);
+  long lz = lg(z), lx = lgefint(x), ly = lgefint(y);
   if (lx == 2) { affur(0, z); return; }
   if (ly == 3)
   {
     affir(x, z); if (signe(y) < 0) togglesign(z);
     affrr(divru(z, y[2]), z);
   }
-  else if (lx > prec + 1 || ly > prec + 1)
+  else if (lx > lz + 1 || ly > lz + 1)
   {
     affir(x,z); affrr(divri(z, y), z);
   }
   else
   {
-    long b = bit_accuracy(prec) + expi(y) - expi(x) + 1;
+    long b = lg2prec(lz) + expi(y) - expi(x) + 1;
     GEN q = divii(b > 0? shifti(x, b): x, y);
     affir(q, z); if (b > 0) shiftr_inplace(z, -b);
   }
@@ -1341,7 +1363,7 @@ affsr(long x, GEN y)
 
   if (!x)
   {
-    y[1] = evalexpo(-prec2nbits(ly));
+    y[1] = evalexpo(-bit_accuracy(ly));
     return;
   }
   if (x < 0) {
@@ -1363,7 +1385,7 @@ affur(ulong x, GEN y)
 
   if (!x)
   {
-    y[1] = evalexpo(-prec2nbits(ly));
+    y[1] = evalexpo(-bit_accuracy(ly));
     return;
   }
   sh = bfffo(x);
