@@ -1343,7 +1343,6 @@ compilefunc(entree *ep, long n, int mode, long flag)
   Gtype ret_typ;
   char const *p,*q;
   char c;
-  const char *flags = NULL;
   const char *str;
   PPproto mod;
   GEN arg=listtogen(y,Flistarg);
@@ -1474,9 +1473,31 @@ compilefunc(entree *ep, long n, int mode, long flag)
     set_avma(ltop);
     compilevec(y, mode, OCcol);
     return;
+  } else if(x==OPlength && tree[y].f==Ffunction && tree[y].x==OPtrans)
+  {
+    arg[1] = tree[y].y;
+    lnc = first_safe_arg(arg,COsafelex|COsafedyn);
+    lnl = first_safe_arg(arg,COsafelex);
+    ep = is_entry("#_~");
+  }
+  else if (x==OPpow && nb==2)
+  {
+    long a = arg[2];
+    if (tree[a].f==Fsmall)
+    {
+      if(tree[a].x==2) { nb--; ep=is_entry("sqr"); }
+      else ep=is_entry("_^s");
+    }
+    else if (tree[a].f == Ffunction && tree[a].x == OPn)
+    {
+      long ay = tree[a].y;
+      if (tree[ay].f==Fsmall)
+      {
+        if (tree[ay].x==1) {nb--; ep=is_entry("_inv"); }
+        else ep=is_entry("_^s");
+      }
+    }
   }
-  else if (x==OPpow && nb==2 && tree[arg[2]].f==Fsmall)
-    ep=is_entry("_^s");
   else if (x==OPcat)
     compile_err("expected character: ',' or ')' instead of",
         tree[arg[1]].str+tree[arg[1]].len);
@@ -1527,7 +1548,7 @@ compilefunc(entree *ep, long n, int mode, long flag)
         case 'M':
           if (tree[arg[j]].f!=Fsmall)
           {
-            if (!flags) flags = ep->code;
+            const char *flags = ep->code;
             flags = strchr(flags, '\n'); /* Skip to the following '\n' */
             if (!flags)
               compile_err("missing flag in string function signature",
@@ -1540,9 +1561,8 @@ compilefunc(entree *ep, long n, int mode, long flag)
               j++;
             } else
             {
-              compilenode(arg[j++],Ggen,0);
-              op_push(OCpushlong,(long)flags,n);
-              op_push(OCcallgen2,(long)is_entry("_eval_mnemonic"),n);
+              compilenode(arg[j++],Ggen,FLnocopy);
+              op_push(OCevalmnem,(long)ep,n);
             }
             break;
           }
@@ -2252,6 +2272,9 @@ compilenode(long n, int mode, long flag)
       GEN vep = cgetg_copy(arg, &lgarg);
       GEN text=cgetg(3,t_VEC);
       gel(text,1)=strntoGENstr(tree[x].str,tree[x].len);
+      if (lgarg==2 && tree[x].str[0]!='~' && tree[x].f==Findarg)
+        /* This occurs for member functions */
+        gel(text,1)=shallowconcat(strntoGENstr("~",1),gel(text,1));
       gel(text,2)=strntoGENstr(tree[y].str,tree[y].len);
       getcodepos(&pos);
       dbgstart=tree[x].str+tree[x].len;
