@@ -101,21 +101,38 @@ ellconvertname(GEN n)
   return NULL; /*LCOV_EXCL_LINE*/
 }
 
-static GEN
-ellcondfile(long f)
+THREAD GEN ellcondfile_cache;
+THREAD long ellcondfile_cache_cond;
+
+void
+pari_init_ellcondfile(void)
 {
-  pari_sp av = avma;
-  long n = f / 1000;
-  char *s = stack_malloc(strlen(pari_datadir) + 12 + 20 + 1);
-  pariFILE *F;
-  GEN V;
-  sprintf(s, "%s/elldata/ell%ld", pari_datadir, n);
-  F = pari_fopengz(s);
-  if (!F) pari_err_FILE("elldata file",s);
-  set_avma(av);
-  V = gp_read_stream(F->file);
-  if (!V || typ(V)!=t_VEC ) pari_err_FILE("elldata file [read]",s);
-  pari_fclose(F); return V;
+  ellcondfile_cache = NULL;
+  ellcondfile_cache_cond = -1;
+}
+
+static GEN
+ellcondfile(long n)
+{
+  if (ellcondfile_cache_cond >= 0 && n == ellcondfile_cache_cond)
+    return gcopy(ellcondfile_cache);
+  else
+  {
+    pari_sp av = avma;
+    pariFILE *F;
+    GEN V;
+    char *s = stack_sprintf("%s/elldata/ell%ld", pari_datadir, n);
+    F = pari_fopengz(s);
+    if (!F) pari_err_FILE("elldata file",s);
+    set_avma(av);
+    V = gp_read_stream(F->file);
+    if (!V || typ(V)!=t_VEC ) pari_err_FILE("elldata file [read]",s);
+    ellcondfile_cache_cond = -1; /* disable cache until update */
+    if (ellcondfile_cache) gunclone(ellcondfile_cache);
+    ellcondfile_cache = gclone(V);
+    ellcondfile_cache_cond = n; /* re-enable cache */
+    pari_fclose(F); return V;
+  }
 }
 
 /* return the vector of all curves of conductor f */
@@ -124,7 +141,7 @@ static GEN
 ellcondlist(long f)
 {
   pari_sp av = avma;
-  GEN V = ellcondfile(f);
+  GEN V = ellcondfile(f/1000);
   long i = tablesearch(V, utoipos(f), &cmpi1);
   if (i)
   {
@@ -240,7 +257,7 @@ forell(void *E, long call(void*, GEN), long a, long b, long flag)
   for(i=ca; i<=cb; i++)
   {
     pari_sp ltop=avma;
-    GEN V = ellcondfile(i*1000);
+    GEN V = ellcondfile(i);
     for (j=1; j<lg(V); j++)
     {
       GEN ells = gel(V,j);
