@@ -414,29 +414,38 @@ gen_sieve(ratpoints_args *args)
   args->sieves0 = gen_sieves0(listprimes);
 }
 
+/* sub-intervals of [-h,h] where squarefree P is positive. */
 static GEN
 ZX_positive_region(GEN P, long h, long bitprec)
 {
   long prec = nbits2prec(bitprec);
-  GEN it = mkvec2(stoi(-h),stoi(h));
-  GEN R = realroots(P, it, prec);
-  long nR = lg(R)-1;
-  long s = signe(ZX_Z_eval(P,gel(it,1)));
-  long i=1, j;
-  GEN iv, st, en;
-  if (s<0 && nR==0) return NULL;
-  iv = cgetg(((nR+1+(s>=0))>>1)+1, t_VEC);
-  if (s>=0) st = itor(gel(it,1),prec);
-  else    { st = gel(R,i); i++; }
-  for (j=1; i<nR; j++)
-  {
-    gel(iv, j) = mkvec2(st, gel(R,i));
-    st = gel(R,i+1);
-    i+=2;
+  GEN A = stoi(-h), B = stoi(h), R = realroots(P, mkvec2(A,B), prec);
+  long s, j, i = 1, nR = lg(R)-1;
+  GEN v, st, en, r;
+
+  P = RgX_div_by_X_x(P, A, &r); /* r = P(A) */
+  s = signe(r); if (s < 0 && nR == 0) return NULL;
+  v = cgetg(nR + 1, t_VEC);
+  if (!s)
+  { /* P(A) = 0 */
+    i = 2; /* skip first real root r1 = A */
+    if (signe(ZX_Z_eval(P, A)) > 0)
+      st = gel(R,1); /* P > 0 on ]A, r2] */
+    else
+      st = gel(R,2); /* P < 0 on ]A, r2] */
   }
-  if (i==nR) en = gel(R,i); else en = itor(gel(it,2),prec);
-  gel(iv,j) = mkvec2(st, en);
-  return iv;
+  else if (s > 0)
+    st = itor(A,prec); /* P > 0 on [A, r1] */
+  else
+    st = gel(R,i++);   /* P < 0 on [A, r1] */
+  for (j = 1; i < nR; j++, i += 2)
+  { /* P changes sign at each real root */
+    gel(v, j) = mkvec2(st, gel(R,i));
+    st = gel(R,i+1);
+  }
+  en = i == nR? gel(R,i): itor(B,prec);
+  gel(v,j++) = mkvec2(st, en);
+  setlg(v, j); return v;
 }
 
 static long
@@ -746,13 +755,13 @@ static const int squares16[16] = {1,1,0,0,1,0,0,0,0,1,0,0,0,0,0,0};
  **************************************************************************/
 
 static ratpoints_sieve_entry*
-alloc_sieve(long nbprime, long maxprime)
+alloc_sieve(long nbprime, long maxp)
 {
   long i;
   ratpoints_sieve_entry * s = (ratpoints_sieve_entry*)
                         stack_malloc(nbprime*sizeof(ratpoints_sieve_entry));
   for (i=0; i<nbprime; i++)
-    s[i].sieve = (ratpoints_bit_array**) new_chunk(maxprime);
+    s[i].sieve = (ratpoints_bit_array**) new_chunk(maxp);
   return s;
 }
 
@@ -760,14 +769,13 @@ alloc_sieve(long nbprime, long maxprime)
 static void
 find_points_init(ratpoints_args *args, long bit_primes)
 {
-  long need = 0;
-  long n, nbprime,maxprime;
+  long need = 0, n, nbprime, maxp;
   args->listprime = primes_interval_zv(3, 1<<bit_primes);
   nbprime = lg(args->listprime)-1;
-  maxprime = args->listprime[nbprime];
+  maxp = args->listprime[nbprime];
 
   /* allocate space for se_buffer */
-  args->se_buffer = alloc_sieve(nbprime, maxprime);
+  args->se_buffer = alloc_sieve(nbprime, maxp);
   args->se_next = args->se_buffer;
   for (n = 1; n <= nbprime; n++)
   {
@@ -779,7 +787,7 @@ find_points_init(ratpoints_args *args, long bit_primes)
   args->ba_next = args->ba_buffer;
 
   /* allocate space for int_buffer */
-  args->int_buffer = (int *) stack_malloc(nbprime*(maxprime+1)*sizeof(int));
+  args->int_buffer = (int *) stack_malloc(nbprime*(maxp+1)*sizeof(int));
   args->int_next = args->int_buffer;
 
   args->forb_ba   = (forbidden_entry*)
@@ -1794,15 +1802,14 @@ QX_hyperellratpoints(GEN P, GEN h, long flag, GEN *den)
 }
 
 static GEN
-QX_homogenous_evalpow(GEN P, GEN A, GEN B)
+ZX_homogenous_evalpow(GEN Q, GEN A, GEN B)
 {
   pari_sp av = avma;
-  GEN den, Q = Q_remove_denom(P, &den);
   long i, d = degpol(Q);
   GEN s = gel(Q, d+2);
   for (i = d-1; i >= 0; i--)
     s = addii(mulii(s, A), mulii(gel(B,d+1-i), gel(Q,i+2)));
-  return gerepileupto(av, den ? gdiv(s,den): s);
+  return d? gerepileupto(av, s): s;
 }
 
 static GEN
@@ -1832,7 +1839,7 @@ GEN
 hyperellratpoints(GEN PQ, GEN h, long flag)
 {
   pari_sp av = avma;
-  GEN P, Q, H, L, den;
+  GEN P, Q, H, L, den, denQ;
   long i, l, dy, dQ;
 
   if (flag<0 || flag>1) pari_err_FLAG("ellratpoints");
@@ -1854,13 +1861,14 @@ hyperellratpoints(GEN PQ, GEN h, long flag)
   H = RgX_add(RgX_muls(P,4), RgX_sqr(Q));
   dy = (degpol(H)+1)>>1; dQ = degpol(Q);
   L = QX_hyperellratpoints(H, h, flag|2L, &den);
+  Q = Q_remove_denom(Q, &denQ);
   l = lg(L);
   for (i = 1; i < l; i++)
   {
     GEN Li = gel(L,i), x = gel(Li,1), y = gel(Li,2), z = gel(Li,3);
-    GEN B = gpowers(z, dQ);
-    GEN Qx = gdiv(QX_homogenous_evalpow(Q, x, B), gel(B, dQ+1));
-    GEN zdy = powiu(z, dy);
+    GEN Qx, B = gpowers(z, dQ), zdy = powiu(z, dy), dQx = gel(B, dQ+1);
+    if (denQ) dQx = mulii(dQx, denQ);
+    Qx = gdiv(ZX_homogenous_evalpow(Q, x, B), dQx);
     if (den) zdy = mulii(zdy, den);
     gel(L,i) = mkvec2(gdiv(x,z), gmul2n(gsub(gdiv(y,zdy),Qx),-1));
   }
@@ -1878,7 +1886,7 @@ ellratpoints(GEN E, GEN h, long flag)
   if (!RgV_is_QV(vecslice(E,1,5))) pari_err_TYPE("ellratpoints",E);
   a1 = ell_get_a1(E);
   a3 = ell_get_a3(E);
-  L = QX_hyperellratpoints(ec_bmodel(E), h, flag|2L, &den);
+  L = QX_hyperellratpoints(ec_bmodel(E,0), h, flag|2L, &den);
   l = lg(L);
   for (i = 1; i < l; i++)
   {
