@@ -67,10 +67,9 @@ mtsingle_is_thread(void) { return single_is_thread; }
 void
 mtsingle_err_recover(long er)
 {
-  (void) er;
   if (single_is_thread)
   {
-    evalstate_set_trace(single_trace_level);
+    if (er) evalstate_set_trace(single_trace_level);
     single_is_thread = 0;
   }
 }
