@@ -4,7 +4,8 @@ This file is part of the PARI/GP package.
 
 PARI/GP is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
-Foundation. It is distributed in the hope that it will be useful, but WITHOUT
+Foundation; either version 2 of the License, or (at your option) any later
+version. It is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
@@ -16,13 +17,14 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #include <windows.h>
 #include <stdio.h>
 
-char* win32_datadir(void)
+char*
+win32_datadir(void)
 {
   char datadir[1024];
   char* slash;
-  GetModuleFileNameA(0, datadir, sizeof(datadir) );
+  GetModuleFileNameA(0, datadir, sizeof(datadir));
   slash = strrchr(datadir, '\\');
-  if( slash ) *(slash+1) = 0;
+  if (slash) slash[1] = 0;
   strcat(datadir, "data");
   return strdup(datadir);
 }
