@@ -79,7 +79,7 @@ rand_INT(long n)
 }
 /* real, n words */
 static GEN
-rand_REAL(long n) { return gmul2n(itor(rand_INT(n), n+2),-BITS_IN_LONG*n); }
+rand_REAL(long n) { return gmul2n(itor(rand_INT(n), bit_accuracy(n+2)),-BITS_IN_LONG*n); }
 
 static GEN
 rand_FpX(long n)
@@ -278,22 +278,41 @@ dftmod(speed_param *s)
   return speed_endtime();                \
 }
 
+#define m_menable_REAL(s,var,min) (*(s->var)=bit_accuracy(minss(lg(s->x),s->min+2)))
+#define  m_enable_REAL(s,var) (*(s->var)=bit_accuracy(lg(s->x)-2))/* enable  asymptotically fastest */
+#define m_disable_REAL(s,var) (*(s->var)=bit_accuracy(lg(s->x)+1))/* disable asymptotically fastest */
+
 #define m_menable(s,var,min) (*(s->var)=minss(lg(s->x)-2,s->min))
 #define  m_enable(s,var) (*(s->var)=lg(s->x)-2)/* enable  asymptotically fastest */
 #define m_disable(s,var) (*(s->var)=lg(s->x)+1)/* disable asymptotically fastest */
 
 static void enable(speed_param *s)
 {
-  m_enable(s,var); s->enabled = 1;
-  if (s->var_disable) m_disable(s,var_disable);
-  if (s->var_enable) m_menable(s,var_enable,var_enable_min);
+  if (s->type==t_REAL)
+  {
+    m_enable_REAL(s,var); s->enabled = 1;
+    if (s->var_disable) m_disable_REAL(s,var_disable);
+    if (s->var_enable) m_menable_REAL(s,var_enable,var_enable_min);
+  } else
+  {
+    m_enable(s,var); s->enabled = 1;
+    if (s->var_disable) m_disable(s,var_disable);
+    if (s->var_enable) m_menable(s,var_enable,var_enable_min);
+  }
 }
 
 static void disable(speed_param *s)
 {
-  m_disable(s,var); s->enabled = 0;
-  if (s->var_disable) m_disable(s,var_disable);
-  if (s->var_enable) m_menable(s,var_enable,var_enable_min);
+  if (s->type==t_REAL)
+  {
+    m_disable_REAL(s,var); s->enabled = 0;
+    if (s->var_disable) m_disable_REAL(s,var_disable);
+    if (s->var_enable) m_menable_REAL(s,var_enable,var_enable_min);
+  } else {
+    m_disable(s,var); s->enabled = 0;
+    if (s->var_disable) m_disable(s,var_disable);
+    if (s->var_enable) m_menable(s,var_enable,var_enable_min);
+  }
 }
 
 static double speed_mulrr(speed_param *s)
@@ -502,6 +521,11 @@ static double speed_FpXQX_extgcd(speed_param *s)
 static double speed_FpXQX_gcd(speed_param *s)
 { TIME_FUN(FpXQX_gcd(s->x, s->y, s->T, s->p)); }
 
+static double speed_ZM2_mul(speed_param *s)
+{ TIME_FUN(ZM2_mul(mkmat22(s->x,s->y,s->y,s->x),
+                   mkmat22(s->x,s->x,s->y,s->x)));
+}
+
 /* small coeffs: earlier thresholds for more complicated rings */
 static double speed_RgX_sqr(speed_param *s)
 { TIME_FUN(RgX_sqr_i(s->x)); }
@@ -523,6 +547,7 @@ static tune_param param[] = {
 {PARI,var(SQRI_KARATSUBA_LIMIT),   t_INT, 4,0, speed_sqri,0,0,&SQRI_FFT_LIMIT},
 {PARI,var(MULII_FFT_LIMIT),        t_INT, 500,0, speed_mulii,0.02},
 {PARI,var(SQRI_FFT_LIMIT),         t_INT, 500,0, speed_sqri,0.02},
+{0,   var(ZM2_MUL_LIMIT),          t_INT, 6,0, speed_ZM2_mul,0.02},
 {0,   var(HALFGCD_LIMIT),          t_INT, 6,0, speed_halfgcdii,0.02},
 {PARI,var(EXTGCD_HALFGCD_LIMIT),   t_INT, 5,0, speed_extgcdii,0.02},
 {PARI,var(GCD_HALFGCD_LIMIT),      t_INT, 5,0, speed_gcdii,0.02},
@@ -678,9 +703,22 @@ diag(const char *format, ...)
   vfprintf(stderr, format, ap);
   va_end(ap);
 }
+
 void
-print_define(const char *name, long value)
-{ printf("#define __%-30s %ld\n", name, value); }
+fdiag(FILE *stream, const char *format, ...)
+{
+  va_list ap;
+  va_start(ap, format);
+  vfprintf(stream, format, ap);
+  va_end(ap);
+}
+
+void
+print_define(tune_param *param, long value)
+{
+  if (param->type==t_REAL && value > 0) value = bit_accuracy(value);
+  printf("#define __%-30s %ld\n", param->name, value);
+}
 
 long
 analyze_dat(int final)
@@ -728,7 +766,7 @@ Test(tune_param *param, long linear)
   pari_sp av=avma;
   long good = -1, bad = param->min_size;
 
-  if (param->kernel == AVOID) { print_define(param->name, -1); return; }
+  if (param->kernel == AVOID) { print_define(param, -1); return; }
 
 #define DEFAULT(x,n)  if (! (param->x))  param->x = (n);
   DEFAULT(step_factor, Step_Factor);
@@ -838,29 +876,31 @@ Test(tune_param *param, long linear)
   thresh = dat[analyze_dat(1)].size;
   if (option_trace >= 1)
     diag("Total time: %gs\n", (double)timer_delay(&T)/1000.);
-  print_define(param->name, thresh);
+  print_define(param, thresh);
   *(param->var) = thresh; /* set to optimal value for next tests */
   if (param->var_disable) *(param->var_disable) = save_var_disable;
   if (param->var_enable) *(param->var_enable) = s.var_enable_min;
   set_avma(av);
 }
 
-void error(char **argv) {
+void error(int err, char **argv) {
   long i;
-  diag("This is the PARI/GP tuning utility. Usage: tune [OPTION] var1 var2...\n");
-  diag("Options:\n");
-  diag("  -t:     verbose output\n");
-  diag("  -tt:    very verbose output\n");
-  diag("  -ttt:   output everything\n");
-  diag("  -l:     use linear search (slower)\n");
-  diag("  -d xxx: set finite field degree to xxx (default 10)\n");
-  diag("  -p xxx: set Flx modulus to xxx (default 27449)\n");
-  diag("  -s xxx: set step factor between successive sizes to xxx (default 0.01)\n");
-  diag("  -u xxx: set speed_unittime to xxx (default 1e-4s)\n");
-  diag("Tunable variables (omitting variable indices tunes everybody):\n");
+  FILE *stream = err ? stderr: stdout;
+  fdiag(stream, "This is the PARI/GP tuning utility. Usage: tune [OPTION] var1 var2...\n");
+  fdiag(stream, "Options:\n");
+  fdiag(stream, "  -h:     this help\n");
+  fdiag(stream, "  -t:     verbose output\n");
+  fdiag(stream, "  -tt:    very verbose output\n");
+  fdiag(stream, "  -ttt:   output everything\n");
+  fdiag(stream, "  -l:     use linear search (slower)\n");
+  fdiag(stream, "  -d xxx: set finite field degree to xxx (default 10)\n");
+  fdiag(stream, "  -p xxx: set Flx modulus to xxx (default 27449)\n");
+  fdiag(stream, "  -s xxx: set step factor between successive sizes to xxx (default 0.01)\n");
+  fdiag(stream, "  -u xxx: set speed_unittime to xxx (default 1e-4s)\n");
+  fdiag(stream, "Tunable variables (omitting variable indices tunes everybody):\n");
   for (i = 0; i < (long)numberof(param); i++)
-    diag("  %2ld: %-25s (default %4ld)\n", i, param[i].name, *(param[i].var));
-  exit(1);
+    fdiag(stream, "  %2ld: %-25s (default %4ld)\n", i, param[i].name, *(param[i].var));
+  exit(err);
 }
 
 int
@@ -888,36 +928,38 @@ main(int argc, char **argv)
         case 'd':
           if (!*++s)
           {
-            if (++i == argc) error(argv);
+            if (++i == argc) error(1,argv);
             s = argv[i];
           }
           DFLT_deg = itou(gp_read_str(s)); break;
+        case 'h':
+          error(0,argv); break;
         case 'p':
           if (!*++s)
           {
-            if (++i == argc) error(argv);
+            if (++i == argc) error(1,argv);
             s = argv[i];
           }
           DFLT_mod = itou(gp_read_str(s)); break;
         case 's':
           if (!*++s)
           {
-            if (++i == argc) error(argv);
+            if (++i == argc) error(1,argv);
             s = argv[i];
           }
           Step_Factor = atof(s); break;
         case 'u': s++;
           if (!*++s)
           {
-            if (++i == argc) error(argv);
+            if (++i == argc) error(1,argv);
             s = argv[i];
           }
           speed_unittime = atof(s); break;
-        default: error(argv);
+        default: error(1,argv);
       }
     } else {
-      if (!isdigit((int)*s)) error(argv);
-      r = atol(s); if (r >= (long)numberof(param) || r < 0) error(argv);
+      if (!isdigit((unsigned char)*s)) error(1,argv);
+      r = atol(s); if (r >= (long)numberof(param) || r < 0) error(1,argv);
       v[n++] = r;
     }
   }
